connect Administrador/a12345

CREATE OR REPLACE
PACKAGE BODY GENERAL AS

/* ------------------------------------------ Comprova que el id existeix

Totes les taules que utilitzen aquesta funci han de tenir el camp identificador amb nom id
Funci generica que comprova l'existncia a una taula d'un ndex

Parmetres d'entrada:
t_id - nmero d'ndex a cercar
taula - nom de la taula a cercar

Parmetre de sortida:
Retorna TRUE si existeix.
Retorna FALSE si no existeix.
*/

FUNCTION fn_id_exist_dinamic
      (t_taula IN VARCHAR2,
      t_id IN NUMBER
      )

   RETURN BOOLEAN
IS

  	t_exist NUMBER;
        taula VARCHAR2(100);
        stmt VARCHAR2(1000);

BEGIN
       taula := dbms_assert.enquote_name ('"' || t_taula || '"');

       stmt := 'BEGIN
               SELECT COUNT(*) INTO :t_exist FROM '|| dbms_assert.simple_sql_name (taula) ||
                  ' WHERE id = '||t_id||' AND data_status IS NULL;
                END;';

   EXECUTE IMMEDIATE stmt USING OUT t_exist;

   RETURN (t_exist > 0);

END;


/* -------------------------------------------- Comprova que el status s inactiu per id
Totes les taules que utilitzen aquesta funci han de tenir el camp identificador amb nom id
Funci generica que comprova di un id existent est actiu a una taula per ndex

Parmetres d'entrada:
t_id - nmero d'ndex a cercar
taula - nom de la taula a cercar

Parmetre de sortida:
Retorna TRUE si el client est inactiu
Retorna FALSE si el client est actiu
*/

FUNCTION fn_id_inactiu_dinamic
      (t_taula IN VARCHAR2,
      t_id IN NUMBER
      )

   RETURN BOOLEAN
IS

  	t_exist NUMBER;
        taula VARCHAR2(100);
        stmt VARCHAR2(1000);

BEGIN
       taula := dbms_assert.enquote_name ('"' || t_taula || '"');

       stmt := 'BEGIN
               SELECT COUNT(*) INTO :t_exist FROM '|| dbms_assert.simple_sql_name (taula) ||
                  ' WHERE id_client = '||t_id||'AND data_status IS NOT NULL;
                END;';

   EXECUTE IMMEDIATE stmt USING OUT t_exist;

   RETURN (t_exist > 0);
END;


/* ------------------------------------------------------------------------ FUNCIONS DE PAISOS

-----------------------------------------------------------------------------------------------
*/


/* ------------------------------------------ Comprova que el pais existeix
Retorna TRUE si existeix i est actiu.
Retorna FALSE si no existeix o est inactiu.
*/

FUNCTION fn_pais_exist
      (t_id_pais IN PAISOS.id_pais%TYPE)

RETURN BOOLEAN
AS

t_exist NUMBER;

BEGIN

    SELECT COUNT(1) INTO t_exist FROM PAISOS pai
          WHERE pai.id_pais=t_id_pais AND data_status IS NULL;

 RETURN (t_exist > 0);
END;


/* ------------------------------------------ Comprova que el pais est inactiu
Retorna TRUE si existeix i est inactiu.
Retorna FALSE si no existeix i est actiu.
*/

FUNCTION fn_pais_inactiu
      (t_id_pais IN PAISOS.id_pais%TYPE)

RETURN BOOLEAN
AS

t_exist NUMBER;

BEGIN

    SELECT COUNT(1) INTO t_exist FROM PAISOS pai
          WHERE pai.id_pais=t_id_pais AND data_status IS NOT NULL;

 RETURN (t_exist > 0);
END;

/* ------------------------------ Comprova que el pais no te cap relaci amb d'altres taules
Noms cal comprovar l'existncia de centrals de producci i de clients
Les lnies no tenen l'atribut pas.
Si no hi ha centrals de producci no hi haur centraletes connectades a aquesta central actives.
Si no hi ha centraletes actives tampoc hi haur comptadors actius connectats a aquestes centraletes


Retorna 'central' si existeix una central d'aquest pas i est activa.
Retorna 'client' si existeix una client d'aquest pas i est actiu.
Retorna NULL si no existeix cao relaci d'aquest pas.

*/
FUNCTION fn_pais_relacio
      (t_id_pais IN PAISOS.id_pais%TYPE)

RETURN VARCHAR2
AS

t_relacio VARCHAR(10);
t_exist_cpo NUMBER;
t_exist_cli NUMBER;

BEGIN

    	SELECT COUNT(1) INTO t_exist_cpo FROM CENTRALS_PRODUCCIO
          	WHERE pais=t_id_pais AND data_status IS NULL;
         SELECT COUNT(1) INTO t_exist_cli FROM CLIENTS
                WHERE pais=t_id_pais AND data_status IS NULL;

        IF (t_exist_cpo >0)
            THEN t_relacio:='central';
            ELSIF (t_exist_cli >0)
                  THEN t_relacio:='client';
                  ELSE t_relacio:=NULL;
        END IF;

 RETURN t_relacio;
END;



/* --------------------------------------------------------------- FUNCIONS DE TIPUS DE CENTRAL

-----------------------------------------------------------------------------------------------
*/

/* ------------------------------------------ Comprova que el tipus de central existeix
Retorna TRUE si existeix iest activa.
Retorna FALSE si no existeix o est inactiu.
*/

FUNCTION fn_tipus_central_exist
      (t_id_tipus IN TIPUS_CENTRALS.id_tipus%TYPE)

RETURN BOOLEAN
AS

t_exist NUMBER;

BEGIN

    SELECT COUNT(1) INTO t_exist FROM TIPUS_CENTRALS
          WHERE id_tipus = t_id_tipus AND data_status IS NULL;

 RETURN (t_exist > 0);
END;

/* ------------------------------------------ Comprova que el tipus de central est inactiu
Retorna TRUE si existeix i est inactiu.
Retorna FALSE si no existeix i est actiu.
*/

FUNCTION fn_tipus_central_inactiu
      (t_id_tipus IN TIPUS_CENTRALS.id_tipus%TYPE)

RETURN BOOLEAN
AS

t_exist NUMBER;

BEGIN

    SELECT COUNT(1) INTO t_exist FROM TIPUS_CENTRALS
          WHERE id_tipus=t_id_tipus AND data_status IS NOT NULL;

 RETURN (t_exist > 0);
END;


/* ----------------------- Comprova que el tipus de central no te centrals de producci assignades
Retorna TRUE si existeix una central d'aquest tipus i est activa.
Retorna FALSE si no existeix cap central d'aquest tipus i est inactiva

*/
FUNCTION fn_tipus_centrals_relacio
      (t_id_tipus_central IN CENTRALS_PRODUCCIO.tipus%TYPE)

RETURN BOOLEAN
AS

t_exist NUMBER;

BEGIN

    	SELECT COUNT(1) INTO t_exist FROM CENTRALS_PRODUCCIO
          	WHERE tipus=t_id_tipus_central AND data_status IS NULL;

 RETURN (t_exist>0);
END;


/* ----------------------------------------------------------- FUNCIONS DE MODELS DE COMPTADORS

-----------------------------------------------------------------------------------------------
*/

/* ------------------------------------------ Comprova que el model de comptador existeix
Retorna TRUE si existeix i est activa.
Retorna FALSE si no existeix i est inactiva.
*/

FUNCTION fn_model_comptador_exist
      (t_id_model IN MODELS_COMPTADOR.id_model%TYPE)

RETURN BOOLEAN
AS

t_exist NUMBER;

BEGIN

    SELECT COUNT(1) INTO t_exist FROM MODELS_COMPTADOR mco
          WHERE mco.id_model=t_id_model AND data_status IS NULL;

 RETURN (t_exist > 0);
END;

/* ------------------------------------------ Comprova que el model de comptador est inactiu
Retorna TRUE si existeix i est inactiu.
Retorna FALSE si no existeix i est actiu.
*/

FUNCTION fn_model_comptador_inactiu
      (t_id_model IN MODELS_COMPTADOR.id_model%TYPE)

RETURN BOOLEAN
AS

t_exist NUMBER;

BEGIN

    SELECT COUNT(1) INTO t_exist FROM MODELS_COMPTADOR
          WHERE id_model=t_id_model AND data_status IS NOT NULL;

 RETURN (t_exist > 0);
END;


/* --------------------------------------- Comprova que el model de comptador no te comptadors assignades
Retorna TRUE si existeix un comptador d'aquest model i est actiu.
Retorna FALSE si no existeix algun ccomptador d'aquest model i est inactiu.
*/

FUNCTION fn_models_comptador_relacio
      (t_id_model IN MODELS_COMPTADOR.id_model%TYPE)

RETURN BOOLEAN
AS

t_exist NUMBER;

BEGIN

    	SELECT COUNT(1) INTO t_exist FROM COMPTADORS
          	WHERE model = t_id_model AND data_status IS NULL;

 RETURN (t_exist>0);
END;


/* ------------------------------------------------------------------------ FUNCIONS DE CLIENTS

-----------------------------------------------------------------------------------------------
*/

/* ------------------------------------------ Comprova que el client existeix i est actiu per DNI_NIF
Retorna TRUE si el client existeix i est actiu.
Retorna FALSE si no existeix o si existeix per est inactiu.
*/

FUNCTION fn_client_exist_x_DNI_NIF
      (t_DNI_NIF IN CLIENTS.DNI_NIF%TYPE)

RETURN BOOLEAN
AS

t_actiu NUMBER;

BEGIN

    SELECT COUNT(1) INTO t_actiu FROM clients
          WHERE DNI_NIF=t_DNI_NIF AND data_status IS NULL;

 RETURN (t_actiu > 0);
END;



/* -------------------------------------------- Comprova que el client est inactiu per DNI_NIF
Retorna TRUE si el client est inactiu
Retorna FALSE si el client est actiu
*/

FUNCTION fn_client_inactiu_x_DNI_NIF
      (t_DNI_NIF IN CLIENTS.DNI_NIF%TYPE)

RETURN BOOLEAN
AS

t_inactiu NUMBER:=0;

BEGIN

    SELECT COUNT(1) INTO t_inactiu FROM clients
          WHERE DNI_NIF=t_DNI_NIF AND data_status IS NOT NULL;

    RETURN (t_inactiu > 0);
END;



/* -------------------------------------------- Cerca id_client per DNI_NIF
Entrada DNI_NIF
Retorna id_client
*/

FUNCTION fn_cerca_id_client_x_DNI_NIF
    (t_DNI_NIF IN CLIENTS.DNI_NIF%TYPE)

RETURN NUMBER
AS

v_return NUMBER;

BEGIN

    SELECT id INTO v_return FROM clients
          WHERE DNI_NIF=t_DNI_NIF;

	RETURN v_return;
END;



/* -------------------------------------------- Cerca DNI_NIF per id_client
Entrada DNI_NIF
Retorna id_client
*/

FUNCTION fn_cerca_DNI_NIF_x_id_client
    (t_id_client IN CLIENTS.id%TYPE)

RETURN VARCHAR2
AS

v_return VARCHAR2(20);

BEGIN

    SELECT DNI_NIF INTO v_return FROM clients
          WHERE id=t_id_client;

	RETURN v_return;
END;



/* -------------------------------------------- Comprova si el client t comptadors actius
Retorna TRUE si el client t comptadors
Retorna FALSE si el client no t comptadors
*/

FUNCTION fn_client_comptadors_actius
      (t_DNI_NIF IN CLIENTS.DNI_NIF%TYPE)

RETURN BOOLEAN
AS

t_actiu NUMBER:=0;

BEGIN


    SELECT COUNT(1) INTO t_actiu FROM comptadors com, clients cli
          WHERE cli.DNI_NIF=t_DNI_NIF AND cli.id=com.client;

    RETURN (NOT(t_actiu > 0));
END;



/* --------------------------------------------------------------------- FUNCIONS DE COMPTADORS

-----------------------------------------------------------------------------------------------
*/

/* ------------------------------------------ Comprova que el comptador existeix i est actiu x numero de contracte
Retorna TRUE si existeix i est actiu
Retorna FALSE si no existeix o est inactiu
*/

FUNCTION fn_comptador_exist_x_NC
      (t_numero_contracte IN COMPTADORS.numero_contracte%TYPE)

RETURN BOOLEAN
AS

t_actiu NUMBER;

BEGIN

    SELECT COUNT(1) INTO t_actiu FROM comptadors
          WHERE numero_contracte=t_numero_contracte AND data_status IS NULL;

 RETURN (t_actiu > 0);
END;



/* -------------------------------------------- Comprova que el comptador est inactiu per NC
Retorna TRUE si est inactiu
Retorna FALSE si no existeix o est actiu
actiu
*/

FUNCTION fn_comptador_inactiu_x_nc
      (t_numero_contracte IN COMPTADORS.numero_contracte%TYPE)

RETURN BOOLEAN
AS

t_inactiu NUMBER:=0;

BEGIN

    SELECT COUNT(1) INTO t_inactiu FROM comptadors com
          WHERE com.numero_contracte=t_numero_contracte AND com.data_status IS NULL;

    RETURN (t_inactiu > 0);
END;


/* -------------------------------------------- Cerca el id_comptador per el numero contracta
Entra el numero de contracte
Retorna id_comptador
*/

FUNCTION fn_cerca_id_comptador_per_NC
      (t_numero_contracte IN COMPTADORS.numero_contracte%TYPE)

RETURN NUMBER
AS

t_id_comptador number;

BEGIN

    SELECT id INTO t_id_comptador FROM comptadors
          WHERE numero_contracte=t_numero_contracte;

    RETURN t_id_comptador;
END;



/* -------------------------------------------- Cerca el numero contracta per el id_comptador
Entra id_comptador
Retorna el numero de contracte
*/

FUNCTION fn_cerca_num_contracte_per_id
      (t_id_comptador IN COMPTADORS.id%TYPE)

RETURN NUMBER
AS

t_numero_contracte number;

BEGIN

    SELECT numero_contracte INTO t_numero_contracte FROM comptadors
          WHERE id=t_id_comptador;

    RETURN t_id_comptador;
END;



/* -------------------------------------------------------------------- FUNCIONS DE CENTRELETES

-----------------------------------------------------------------------------------------------
*/


/* ------------------------------------------ Comprova que la centraleta existeix i est activa per adrea i cp
Retorna TRUE si existeix i est actiu
Retorna FALSE si no existeix o est inactiu
*/

FUNCTION fn_centraleta_exist_x_adr_cp
	(t_adreca IN CENTRALETES.adreca%TYPE,
	t_cp IN CENTRALETES.cp%TYPE
	)

RETURN BOOLEAN
AS

t_actiu NUMBER;

BEGIN

    SELECT COUNT(1) INTO t_actiu FROM centraletes
          WHERE adreca=t_adreca AND cp=t_cp AND data_status IS NULL;

 RETURN (t_actiu > 0);
END;



/* -------------------------------------------- Comprova que la centraleta est activa per adrea i cp
Retorna TRUE si no est actiu
Retorna FALSE si est actiu
*/

FUNCTION fn_centraleta_inactiu_x_adr_cp
	(t_adreca IN CENTRALETES.adreca%TYPE,
	t_cp IN CENTRALETES.cp%TYPE
	)

RETURN BOOLEAN
AS

t_inactiu NUMBER:=0;

BEGIN

    SELECT COUNT(1) INTO t_inactiu FROM centraletes
          WHERE adreca=t_adreca AND cp=t_cp AND data_status IS NOT NULL;

    RETURN (t_inactiu > 0);
END;


/* --------------------------- Obtenir l'adrea de la centraleta a partir del id
Entra el id de la centraleta
Retorna adrea de la centraleta
*/

FUNCTION fn_cerca_adreca_centr_per_id
	(t_id_centraleta IN CENTRALETES.id%TYPE
	)

RETURN VARCHAR2
AS

t_adreca varchar2(50);

BEGIN

    SELECT adreca INTO t_adreca FROM centraletes
          WHERE id =t_id_centraleta;

    RETURN t_adreca;
END;



/* --------------------------- Obtenir cp de la centraleta a partir del id

Entra el id de la centraleta
Retorna cp de la centraleta
*/

FUNCTION fn_cerca_cp_centr_per_id
	(t_id_centraleta IN CENTRALETES.id%TYPE
	)

RETURN VARCHAR2
AS

t_cp varchar2(5);

BEGIN

    SELECT cp INTO t_cp FROM centraletes
          WHERE id =t_id_centraleta;

    RETURN t_cp;
END;



/* --------------------------- Obtenir l'adrea de la centraleta a partir del id
Entra l'adrea i el cp de la centraleta
Retorna id de la centraleta
*/

FUNCTION fn_cerca_id_ctrl_per_adre_cp
	(t_adreca IN CENTRALETES.adreca%TYPE,
	t_cp IN  CENTRALETES.cp%TYPE
	)

RETURN NUMBER
AS

t_id_centraleta NUMBER;

BEGIN

    SELECT id INTO t_id_centraleta FROM centraletes
          WHERE adreca = t_adreca AND cp = t_cp;

    RETURN t_id_centraleta;
END;



/* ---------------------------------------------------------- FUNCIONS DE CENTRALS DE PRODUCCIO

-----------------------------------------------------------------------------------------------
*/


/* ------------------------------------------ Comprova que la central existeix i est activa per nom
Retorna TREU si existeix i est activa
Retorna FAlSE si no existeix o est inactiva
*/

FUNCTION fn_central_exist_x_nom
      (t_nom IN CENTRALS_PRODUCCIO.nom%TYPE)

RETURN BOOLEAN
AS

t_actiu VARCHAR(20);

BEGIN

    SELECT COUNT(1) INTO t_actiu FROM centrals_produccio
          WHERE nom=t_nom AND data_status IS NULL;

 RETURN (t_actiu > 0);
END;



/* -------------------------------------------- Comprova que la central est activa per nom
Retorna TRUE si est activa
Retorna FALS si est innactiva
*/

FUNCTION fn_central_actiu_x_nom
      (t_nom IN CENTRALS_PRODUCCIO.nom%TYPE)

RETURN BOOLEAN
AS

t_actiu VARCHAR2(20);

BEGIN

    SELECT COUNT(1) INTO t_actiu FROM centrals_produccio
          WHERE nom=t_nom AND data_status IS NOT NULL;

    RETURN (NOT(t_actiu > 0));
END;



/* -------------------------------------------- Cerca el id_central per nom
Entra el nom de la central de producci
Retorna id_central
*/

FUNCTION fn_cerca_id_central_x_nom
      (t_nom IN CENTRALS_PRODUCCIO.nom%TYPE)

RETURN NUMBER
AS

t_id_central number;

BEGIN

    SELECT id INTO t_id_central FROM centrals_produccio
          WHERE nom=t_nom;

    RETURN t_id_central;
END;


/* ------------------------------------------ Comprova que la central existeix per nom
Retorna TRUE si existeixen la central.
Retorna FALSE si no existeixen.
*/

FUNCTION fn_exist_central_x_nom
      (t_nom IN CENTRALS_PRODUCCIO.nom%TYPE)


RETURN BOOLEAN
AS

t_exist NUMBER;

BEGIN

    SELECT COUNT(1) INTO t_exist FROM CENTRALS_PRODUCCIO
          WHERE nom=t_nom;

 RETURN (t_exist > 0);
END;



/* -------------------------------------------- Cerca el nom per id_central
Entra el nom de la central de producci
Retorna id_central
*/

FUNCTION fn_cerca_nom_x_id_central
      (t_id_central IN CENTRALS_PRODUCCIO.id%TYPE)

RETURN VARCHAR2
AS

t_nom VARCHAR2(20);

BEGIN

    SELECT nom INTO t_nom FROM centrals_produccio
          WHERE id=t_id_central;

    RETURN t_nom;
END;


/* --------------------------------------------------------- FUNCIONS DE CENTRELETES-COMPTADORS

-----------------------------------------------------------------------------------------------
*/


/* --------------------------- Comprova que la relaci centraleta-comptador existeix i est activa
Retorna TRUE si existeix i est actiu
Retorna FALSE si no existeix o est inactiu
*/
FUNCTION fn_centr_comp_exist
	(t_id_centraleta IN CENTRALETES_COMPTADORS.centraleta%TYPE,
	t_id_comptador IN CENTRALETES_COMPTADORS.comptador%TYPE
	)

RETURN BOOLEAN
AS

t_actiu NUMBER;

BEGIN

    SELECT COUNT(1) INTO t_actiu FROM centraletes_comptadors
          WHERE centraleta=t_id_centraleta AND comptador=t_id_comptador AND data_status IS NULL;

 RETURN (t_actiu > 0);
END;



/* ------------------------------------- Comprova que la relaci centraleta-comptador est activa
Retorna TRUE si est actiu
Retorna FALSE si est innactiu
*/
FUNCTION fn_centr_comp_actiu
	(t_id_centraleta IN CENTRALETES_COMPTADORS.centraleta%TYPE,
	t_id_comptador IN CENTRALETES_COMPTADORS.comptador%TYPE
	)

RETURN BOOLEAN
AS

t_actiu NUMBER:=0;

BEGIN

    SELECT COUNT(1) INTO t_actiu FROM centraletes_comptadors
          WHERE centraleta=t_id_centraleta AND comptador=t_id_comptador AND data_status IS NOT NULL;

    RETURN (NOT(t_actiu > 0));
END;



/* -------------------------------------------- Cerca la central per centraleta
Entra l'adrea i el cp de la centraleta
Cerca a linies el id_central
Retorna el id_central
*/

FUNCTION fn_cerca_central_x_centraleta
	(t_adreca IN CENTRALETES.adreca%TYPE,
	t_CP IN CENTRALETES.cp%TYPE)

RETURN NUMBER
AS

t_id_linia NUMBER;
t_id_central NUMBER;

BEGIN
	select linia INTO t_id_linia FROM CENTRALETES
		WHERE adreca = t_adreca AND cp = t_cp;

	select central INTO t_id_central FROM LINIES
		WHERE id = t_id_linia;

    RETURN t_id_central;
END;



/* ---------------------------------------------------------------------------------------- PROCEDIMENTS

   -----------------------------------------------------------------------------------------------------

   --------------------------------------------------- Alta de log_aplicacio
*/

PROCEDURE pr_alta_log_aplicacio
	(t_procediment IN LOG_APLICACIO.procediment%TYPE,
	 t_entrada IN LOG_APLICACIO.entrada%TYPE,
	 t_sortida IN LOG_APLICACIO.sortida%TYPE
	)
IS

PRAGMA AUTONOMOUS_TRANSACTION;

BEGIN

    INSERT INTO LOG_APLICACIO(usuari, data_hora, procediment, entrada, sortida)
            VALUES (USER, SYSDATE, t_procediment, t_entrada, t_sortida);

    COMMIT;

EXCEPTION
--        ROLLBACK;
            WHEN OTHERS THEN
            DBMS_OUTPUT.PUT_LINE
		('Error'||SQLCODE||SQLERRM||'- Al donar un alta de registre al dietari');
            COMMIT;
END;


/* ------------------------------------------------------------------------------- TRACTAMENT DE PAISOS

--------------------------------------------------------------------------------------------------------
*/

/* --------------------------------------------------- Alta de pais
Si existeix ens dona un error -20010
Si no existeix el dona d'alta
*/

PROCEDURE pr_alta_pais
	(t_id_pais IN PAISOS.id_pais%TYPE,
	t_nom IN PAISOS.nom%TYPE,
	t_poblacio IN PAISOS.poblacio%TYPE,
	t_extensio IN PAISOS.extensio%TYPE
	)
IS
	v_exist BOOLEAN;
	v_inactiu BOOLEAN;
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_alta_pais';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	pais_exist EXCEPTION;

BEGIN
 	v_exist := FN_PAIS_EXIST(t_id_pais);
	v_inactiu := FN_PAIS_INACTIU(t_id_pais);
        entrada := '-'||t_id_pais||'-'||t_nom||'-'||t_poblacio||'-'||t_extensio||'-';

	IF (v_exist)
		THEN RAISE pais_exist;
		ELSIF (v_inactiu)
			THEN UPDATE PAISOS SET
					id_pais = t_id_pais,
					nom = t_nom,
					poblacio = t_poblacio,
					extensio = t_extensio,
					data_status = NULL
                	        	WHERE id_pais = t_id_pais;
        	          		pr_alta_log_aplicacio(procediment, entrada, sortida);
			ELSE INSERT INTO PAISOS (id_pais, nom, poblacio, extensio,data_status)
				VALUES (t_id_pais, t_nom, t_poblacio, t_extensio,NULL);
        	            	pr_alta_log_aplicacio(procediment, entrada, sortida);
	END IF;

EXCEPTION

	WHEN pais_exist THEN
            ROLLBACK;
            sortida := '-20010 - El pais '||t_id_pais||' ja existeix.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
            COMMIT;

        WHEN OTHERS THEN
            ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar un alta de pas';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
            COMMIT;
END;



/* --------------------------------------------------- Modificaci de pais
Si no existeix ens dona un error -20012
Si existeix fa la modificaci
*/

PROCEDURE pr_modifica_pais
	(t_id_pais IN PAISOS.id_pais%TYPE,
	t_nom IN PAISOS.nom%TYPE,
	t_poblacio IN PAISOS.poblacio%TYPE,
	t_extensio IN PAISOS.extensio%TYPE
	)
IS
	v_exist BOOLEAN;
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_modifica_pais';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	pais_not_exist EXCEPTION;

BEGIN
 	v_exist := FN_PAIS_EXIST(t_id_pais);
        entrada := '-'||t_id_pais||'-'||t_nom||'-'||t_poblacio||'-'||t_extensio||'-';

	IF (NOT v_exist)
		THEN RAISE pais_not_exist;
		ELSE UPDATE PAISOS SET
			nom = t_nom,
			poblacio = t_poblacio,
			extensio = t_extensio
                        WHERE id_pais = t_id_pais;
                    	pr_alta_log_aplicacio(procediment, entrada, sortida);
	END IF;

EXCEPTION

	WHEN pais_not_exist THEN
            ROLLBACK;
            sortida := '-20012 - El pais '||t_id_pais||' no existeix.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
            COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una modificaci de pas';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;
END;



/* --------------------------------------------------- Baixa de pais
Si no existeix ens dona un error -20014
Si t dades relacionades a CENTRALS_PRODUCCIO dona l'error -20014
Si t dades relacionades a CLIENTS dona l'error -20016
Si existeix i no t relacions dona de baixa
*/

PROCEDURE pr_baixa_pais
	(t_id_pais IN PAISOS.id_pais%TYPE
	)
IS
	v_exist BOOLEAN;
	v_relacio VARCHAR2(10);
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_baixa_pais';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	pais_not_exist EXCEPTION;
	pais_relacio_central EXCEPTION;
	pais_relacio_client EXCEPTION;
BEGIN

	v_exist := FN_PAIS_EXIST(t_id_pais);
 	v_relacio := FN_PAIS_RELACIO(t_id_pais);
        entrada := '-'||t_id_pais||'-';

	IF (NOT v_exist)
		THEN RAISE pais_not_exist;
	ELSIF (v_relacio = 'central')
		THEN RAISE pais_relacio_central;
                ELSIF (v_relacio = 'client')
                      	THEN RAISE pais_relacio_client;
                      	ELSE UPDATE PAISOS SET
				data_status = SYSDATE
        	                WHERE id_pais = t_id_pais;
      				pr_alta_log_aplicacio(procediment, entrada, sortida);
	END IF;

EXCEPTION
	WHEN pais_not_exist THEN
            ROLLBACK;
	    sortida := '-20012 - El pais '||t_id_pais||' no existeix.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
            COMMIT;

	WHEN pais_relacio_central THEN
            ROLLBACK;
	    sortida := '-20014 - El pais '||t_id_pais||' t centrals assignades.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

	WHEN pais_relacio_client THEN
            ROLLBACK;
	    sortida := '-20016 - El pais '||t_id_pais||' t clients assignats.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
	    sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una baixa de pas';
		    	pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;
END;



/* ----------------------------------------------------------------------- TRACTAMENT DE TIPUS de CENTRAL

   -----------------------------------------------------------------------------------------------------

   --------------------------------------------------- Alta de tipus de central
Si existeix ens dona un error -20020
Si no existeix el dona d'alta
*/

PROCEDURE pr_alta_tipus_centrals
	(t_id_tipus_central IN TIPUS_CENTRALS.id_tipus%TYPE,
	t_descripcio IN TIPUS_CENTRALS.descripcio%TYPE
	)
IS
	v_exist BOOLEAN;
        v_inactiu BOOLEAN;
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_alta_tipus_central';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	tipus_exist EXCEPTION;
BEGIN
 	v_exist := FN_TIPUS_CENTRAL_EXIST(t_id_tipus_central);
	v_inactiu := FN_TIPUS_CENTRAL_INACTIU(t_id_tipus_central);
        entrada := '-'||t_id_tipus_central||'-'||t_descripcio||'-';


	IF (v_exist)
		THEN RAISE tipus_exist;
		ELSIF (v_inactiu)
			THEN UPDATE Tipus_CENTRALS SET
				descripcio = t_descripcio,
				data_status = NULL
                  		WHERE id_tipus = t_id_tipus_central;
                  	 	pr_alta_log_aplicacio(procediment, entrada, sortida);
			ELSE INSERT INTO TIPUS_CENTRALS (id_tipus, descripcio, data_status)
				VALUES (t_id_tipus_central, t_descripcio, NULL);
        	            	pr_alta_log_aplicacio(procediment, entrada, sortida);
	END IF;

EXCEPTION
	WHEN tipus_exist THEN
            ROLLBACK;
	    sortida := '-20020 - El tipus de central '||t_id_tipus_central||' ja existeix.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
            COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una alta de tipus de central';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;
END;



/* --------------------------------------- Modificaci de tipus de centrals de produccio
Si no existeix ens dona un error -20022
Si existeix fa la modificaci
*/

PROCEDURE pr_modifica_tipus_centrals
	(t_id_tipus_central IN TIPUS_CENTRALS.id_tipus%TYPE,
	t_descripcio IN TIPUS_CENTRALS.descripcio%TYPE
	)
IS
	v_exist BOOLEAN;
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_modifica_tipus_central';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	tipus_not_exist EXCEPTION;
BEGIN
 	v_exist := FN_TIPUS_CENTRAL_EXIST(t_id_tipus_central);
        entrada := '-'||t_id_tipus_central||'-'||t_descripcio||'-';

	IF (NOT v_exist)
		THEN RAISE tipus_not_exist;
		ELSE UPDATE Tipus_CENTRALS SET
			descripcio = t_descripcio
                        WHERE id_tipus = t_id_tipus_central;
                   	pr_alta_log_aplicacio(procediment, entrada, sortida);
	END IF;

EXCEPTION
	WHEN tipus_not_exist THEN
            ROLLBACK;
	    sortida := '-20022 - El tipus de central '||t_id_tipus_central||' no existeix.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una modificacio de tipus de central';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;
END;



/* --------------------------------------------------- Baixa de tipus de central de producci
Si no existeix ens dona un error -20022
Si t dades relacionades a CENTRALS_PRODUCCIO dona l'error -20024
Si existeix i no t relacions dona de baixa
*/

PROCEDURE pr_baixa_tipus_centrals
	(t_id_tipus_central IN TIPUS_CENTRALS.id_tipus%TYPE
	)
IS
	v_exist BOOLEAN;
	v_relacio BOOLEAN;
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_baixa_tipus_central';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	tipus_not_exist EXCEPTION;
	tipus_relacio_central EXCEPTION;
BEGIN

 	v_exist := FN_TIPUS_CENTRAL_EXIST(t_id_tipus_central);
 	v_relacio := FN_TIPUS_CENTRALS_RELACIO(t_id_tipus_central);
        entrada := '-'||t_id_tipus_central||'-';

	IF (NOT v_exist)
		THEN RAISE tipus_not_exist;
	ELSIF (v_relacio)
		THEN RAISE tipus_relacio_central;
                ELSE UPDATE TIPUS_CENTRALS SET
			data_status = SYSDATE
        	        WHERE id_tipus=t_id_tipus_central;
      			pr_alta_log_aplicacio(procediment, entrada, sortida);
	END IF;

EXCEPTION
	WHEN tipus_not_exist THEN
            ROLLBACK;
	    sortida := '-20022 - El tipus de central '||t_id_tipus_central||' no existeix.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

	WHEN tipus_relacio_central THEN
            ROLLBACK;
	    sortida := '-20024 - El tipus de central '||t_id_tipus_central||' t centrals assigandes.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una baixa de tipus de central';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

END;



/* ----------------------------------------------------------------------- TRACTAMENT DE MODEL COMPTADOR

   -----------------------------------------------------------------------------------------------------

   ------------------------------------------------------------------- Alta de models de comptador
Si existeix ens dona un error -20030
Si no existeix el dona d'alta
*/

PROCEDURE pr_alta_models_comptador
	(t_id_model IN MODELS_COMPTADOR.id_model%TYPE,
	t_fabricant IN MODELS_COMPTADOR.fabricant%TYPE,
	t_descripcio IN MODELS_COMPTADOR.descripcio%TYPE
	)
IS
	v_exist BOOLEAN;
        v_inactiu BOOLEAN;
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_alta_models_comptador';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	model_exist EXCEPTION;
BEGIN
 	v_exist := FN_MODEL_COMPTADOR_EXIST(t_id_model);
	v_inactiu := FN_MODEL_COMPTADOR_INACTIU(t_id_model);
        entrada := '-'||t_id_model||'-'||t_fabricant||'-'||t_descripcio||'-';

	IF (v_exist)
		THEN RAISE model_exist;
		ELSIF (v_inactiu)
			THEN UPDATE MODELS_COMPTADOR SET
				fabricant = t_fabricant,
				descripcio = t_descripcio,
				data_status = NULL
                   		WHERE id_model = t_id_model;
                   		pr_alta_log_aplicacio(procediment, entrada, sortida);
			ELSE INSERT INTO MODELS_COMPTADOR (id_model, fabricant, descripcio, data_status)
				VALUES (t_id_model, t_fabricant, t_descripcio, NULL);
        	           	pr_alta_log_aplicacio(procediment, entrada, sortida);
	END IF;

EXCEPTION
	WHEN model_exist THEN
            ROLLBACK;
	    sortida := '-20030 - El model '||t_id_model||' ja existeix.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una alta de model de comptador';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;
END;



/* ------------------------------------------------------------------- Modificaci de models de comptador
Si no existeix ens dona un error -20032
Si existeix fa la modificaci
*/

PROCEDURE pr_modifica_models_comptador
	(t_id_model IN MODELS_COMPTADOR.id_model%TYPE,
	t_fabricant IN MODELS_COMPTADOR.fabricant%TYPE,
	t_descripcio IN MODELS_COMPTADOR.descripcio%TYPE
	)
IS
	v_exist BOOLEAN;
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_modifica_models_comptador';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	model_not_exist EXCEPTION;
BEGIN
 	v_exist := FN_MODEL_COMPTADOR_EXIST(t_id_model);
        entrada := '-'||t_id_model||'-'||t_fabricant||'-'||t_descripcio||'-';

	IF (NOT v_exist)
		THEN RAISE model_not_exist;
		ELSE UPDATE MODELS_COMPTADOR SET
			fabricant = t_fabricant,
			descripcio = t_descripcio
                        WHERE id_model = t_id_model;
                   	pr_alta_log_aplicacio(procediment, entrada, sortida);

	END IF;

EXCEPTION
	WHEN model_not_exist THEN
            ROLLBACK;
	    sortida := '-20032 - El model '||t_id_model||' no existeix.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una modificaci de model de comptador';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;
END;



/* --------------------------------------------------------------------------- Baixa de models de comptador
Si no existeix ens dona un error -20032
Si t dades relacionades a COMPTADORS dona l'error -20034
Si existeix i no t relacions dona de baixa
*/

PROCEDURE pr_baixa_models_comptador
	(t_id_model IN MODELS_COMPTADOR.id_model%TYPE
	)
IS
	v_exist BOOLEAN;
	v_relacio BOOLEAN;
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_baixa_models_comptador';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	model_not_exist EXCEPTION;
	model_relacio_comptadors EXCEPTION;
BEGIN

 	v_exist := FN_MODEL_COMPTADOR_EXIST(t_id_model);
 	v_relacio := FN_MODELS_COMPTADOR_RELACIO(t_id_model);
        entrada := '-'||t_id_model||'-';

	IF (NOT v_exist)
		THEN RAISE model_not_exist;
	ELSIF (v_relacio)
		THEN RAISE model_relacio_comptadors;
               ELSE UPDATE MODELS_COMPTADOR SET
			data_status = SYSDATE
        	        WHERE id_model=t_id_model;
      			pr_alta_log_aplicacio(procediment, entrada, sortida);
	END IF;

EXCEPTION
	WHEN model_not_exist THEN
            ROLLBACK;
	    sortida := '-20032 - El model '||t_id_model||' no existeix.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

	WHEN model_relacio_comptadors THEN
            ROLLBACK;
	    sortida := '-20034 - El model '||t_id_model||' t comptadors assignats.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una baixa de model de comptador';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;
END;


/* ------------------------------------------------------------------------------- TRACTAMENT DE CLIENTS

   -----------------------------------------------------------------------------------------------------

   --------------------------------------------------- Alta de client
Si existeix i est actiu ens dona un error -20070
Si el pas no existeix ens dona un error -20072
Si existeix, per no est actiu l'actualitza
Si no existeix el dona d'alta
*/

PROCEDURE pr_alta_client
	(t_DNI_NIF IN CLIENTS.DNI_NIF%TYPE,
	t_nom_rao_social IN CLIENTS.nom_rao_social%TYPE,
	t_adreca IN CLIENTS.adreca%TYPE,
	t_cp IN CLIENTS.cp%TYPE,
	t_localitat IN CLIENTS.localitat%TYPE,
	t_pais IN CLIENTS.pais%TYPE,
	t_telefon IN CLIENTS.telefon%TYPE,
	t_e_mail IN CLIENTS.e_mail%TYPE,
	t_companyia IN CLIENTS.companyia%TYPE
	)
IS

	v_exist BOOLEAN;
	v_inactiu BOOLEAN;
	V_exist_pais BOOLEAN;
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_alta_client';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	client_exist EXCEPTION;
	pais_not_exist EXCEPTION;
BEGIN

	v_exist := FN_CLIENT_EXIST_X_DNI_NIF(t_DNI_NIF);
	v_inactiu := FN_CLIENT_INACTIU_X_DNI_NIF(t_DNI_NIF);
	v_exist_pais := FN_PAIS_EXIST(t_pais);
        entrada := '-'||t_DNI_NIF||'-'||t_nom_rao_social||'-'||t_adreca||'-'||t_cp||'-'||t_localitat||'-'||t_pais||'-'||t_telefon||'-'||t_e_mail||'-'||	t_companyia||'-';


	IF (v_exist)
		THEN RAISE client_exist;
		ELSIF (NOT v_exist_pais)
			THEN RAISE pais_not_exist;
			ELSIF (v_inactiu)
	               		THEN UPDATE CLIENTS SET
					nom_rao_social = t_nom_rao_social, adreca = t_adreca,
					cp = t_cp, localitat = t_localitat,
					pais = t_pais, telefon = t_telefon,
					e_mail = t_e_mail, companyia = t_companyia,
					consum_mitja = 0, data_status = NULL
                	        	WHERE DNI_NIF = t_DNI_NIF;
        	          		pr_alta_log_aplicacio(procediment, entrada, sortida);
				ELSE INSERT INTO CLIENTS (DNI_NIF ,nom_rao_social ,adreca ,cp ,localitat ,pais ,telefon ,e_mail ,
					companyia, consum_mitja, data_status)
				VALUES (t_DNI_NIF ,t_nom_rao_social ,t_adreca ,t_cp ,t_localitat ,t_pais ,t_telefon ,t_e_mail ,
						t_companyia, 0, NULL);
	                  	pr_alta_log_aplicacio(procediment, entrada, sortida);

	END IF;

EXCEPTION
	WHEN client_exist THEN
            ROLLBACK;
	    sortida := '-20070 - El client '||t_DNI_NIF||' ja existeix.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

	WHEN pais_not_exist THEN
            ROLLBACK;
	    sortida := '-20072 - El pas '||t_pais||' no existeix.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una alta de client';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;
END;



/* --------------------------------------------------- Modificaci de client
Si no existeix ens dona un error -20074
Si el pas no existeix ens dona un error -20072
Si existeix i est actiu l'actualitza
*/

PROCEDURE pr_modifica_client
	(t_DNI_NIF IN CLIENTS.DNI_NIF%TYPE,
	t_nom_rao_social IN CLIENTS.nom_rao_social%TYPE,
	t_adreca IN CLIENTS.adreca%TYPE,
	t_cp IN CLIENTS.cp%TYPE,
	t_localitat IN CLIENTS.localitat%TYPE,
	t_pais IN CLIENTS.pais%TYPE,
	t_telefon IN CLIENTS.telefon%TYPE,
	t_e_mail IN CLIENTS.e_mail%TYPE,
	t_companyia IN CLIENTS.companyia%TYPE
	)
IS

	v_exist BOOLEAN;
	V_exist_pais BOOLEAN;
	v_id CLIENTS.id%TYPE;
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_Modifica_client';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	client_not_exist EXCEPTION;
	pais_not_exist EXCEPTION;
BEGIN

	IF (fn_client_exist_x_dni_nif(t_DNI_NIF))
		THEN v_id := fn_cerca_id_client_x_DNI_NIF(t_DNI_NIF);
		 	v_exist := FN_ID_EXIST_DINAMIC('CLIENTS', v_id);
		ELSE v_exist := FALSE;
	END IF;


	v_exist_pais := FN_PAIS_EXIST(t_pais);
        entrada := '-'||t_DNI_NIF||'-'||t_nom_rao_social||'-'||t_adreca||'-'||t_cp||'-'||t_localitat||'-'||t_pais||'-'||t_telefon||'-'||t_e_mail||'-'||	t_companyia||'-';

	IF ( NOT v_exist)
		THEN RAISE client_not_exist;
		ELSIF (NOT v_exist_pais)
			THEN RAISE pais_not_exist;
			ELSE UPDATE CLIENTS SET
				nom_rao_social = t_nom_rao_social, adreca = t_adreca,
				cp = t_cp, localitat = t_localitat,
				pais = t_pais, telefon = t_telefon,
				e_mail = t_e_mail, companyia = t_companyia,
				data_status = NULL
                	        WHERE DNI_NIF = t_DNI_NIF;
                  		pr_alta_log_aplicacio(procediment, entrada, sortida);
	END IF;

EXCEPTION
	WHEN client_not_exist THEN
            ROLLBACK;
	    sortida := '-20074 - El client '||t_DNI_NIF||' no existeix o est inactiu.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

	WHEN pais_not_exist THEN
            ROLLBACK;
	    sortida := '-20072 - El pas '||t_pais||' no existeix.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una modificaci de client';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

END;



/* --------------------------- Procediment que dona de baixa lgica a la relaci centraletes_comptadors

*/
PROCEDURE pr_baixa_rel_centraletes_comp
	(t_id_comptador IN CENTRALETES_COMPTADORS.comptador%TYPE
	)
IS
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_baixa_rel_centraletes_comp';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	CURSOR centrale_comptador IS
		SELECT * FROM CENTRALETES_COMPTADORS
			WHERE comptador = t_id_comptador
			FOR UPDATE OF data_status;
   	reg centrale_comptador%ROWTYPE;
BEGIN

        entrada := '-'||t_id_comptador||'-';

	OPEN centrale_comptador;
	FETCH centrale_comptador INTO reg;
	WHILE centrale_comptador%FOUND LOOP
		UPDATE CENTRALETES_COMPTADORS
		SET data_status = SYSDATE
		WHERE CURRENT OF centrale_comptador;
		FETCH centrale_comptador INTO reg;
      		pr_alta_log_aplicacio(procediment, entrada, sortida);
	END LOOP;

EXCEPTION
        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una baixa de relaci de centraletes-comptadors per comptador';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;
END;



/* --------------------------- Procediment que dona de baixa lgica a la relaci centraletes_comptadors per centraleta

*/
PROCEDURE pr_baixa_centr_comp_x_centr
	(t_id_centraleta IN CENTRALETES_COMPTADORS.centraleta%TYPE
	)
IS
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_baixa_centr_comp_x_centr';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	CURSOR centrale_comptador IS
		SELECT * FROM CENTRALETES_COMPTADORS
			WHERE centraleta = t_id_centraleta
			FOR UPDATE OF data_status;
   	reg centrale_comptador%ROWTYPE;
BEGIN

        entrada := '-'||t_id_centraleta||'-';

	OPEN centrale_comptador;
	FETCH centrale_comptador INTO reg;
	WHILE centrale_comptador%FOUND LOOP
		UPDATE CENTRALETES_COMPTADORS
		SET data_status = SYSDATE
		WHERE CURRENT OF centrale_comptador;
		FETCH centrale_comptador INTO reg;
      		pr_alta_log_aplicacio(procediment, entrada, sortida);
	END LOOP;

EXCEPTION
        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una baixa de relaci de centraletes-comptadors per centraleta';
      		pr_alta_log_aplicacio(procediment, entrada, sortida);
	    COMMIT;
END;



/* --------------------------- Procediment que dona de baixa lgica tots els comptadors d'un client
	Abans de de donar de baixa el comptador dona de baixa la relaci centraletes_comptadors
*/
PROCEDURE pr_baixa_comptadors_client
	(t_id_client IN COMPTADORS.client%TYPE
	)
IS
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_baixa_comptadors_client';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	CURSOR comptadors_client IS
		select * FROM COMPTADORS
			WHERE client = t_id_client
			FOR UPDATE OF data_status;
	reg COMPTADORS%ROWTYPE;
BEGIN

        entrada := '-'||t_id_client||'-';
	OPEN comptadors_client;
	FETCH comptadors_client INTO reg;
	WHILE comptadors_client%FOUND LOOP
                pr_baixa_rel_centraletes_comp(reg.id);
		UPDATE COMPTADORS
		SET data_status = SYSDATE
		WHERE CURRENT OF comptadors_client;
		FETCH comptadors_client INTO reg;
      		pr_alta_log_aplicacio(procediment, entrada, sortida);
	END LOOP;

EXCEPTION
        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una baixa de comptador';
      		pr_alta_log_aplicacio(procediment, entrada, sortida);
	    COMMIT;
END;



/* --------------------------------------------------- baixa de client
Si no existeix ens dona un error -20074
Si existeix i est actiu es dona de baixa lgica,
	dona de baixa lgica els comptadors del client i la relaci comptadors_centraletes
*/

PROCEDURE pr_baixa_client
	(t_DNI_NIF IN CLIENTS.DNI_NIF%TYPE
	)
IS
	v_exist BOOLEAN;
	v_id CLIENTS.id%TYPE;
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_baixa_client';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	client_not_exist EXCEPTION;
BEGIN

	IF (fn_client_exist_x_dni_nif(t_DNI_NIF))
		THEN v_id := fn_cerca_id_client_x_DNI_NIF(t_DNI_NIF);
		 	v_exist := FN_ID_EXIST_DINAMIC('CLIENTS', v_id);
		ELSE v_exist := FALSE;
	END IF;

        entrada := '-'||t_DNI_NIF||'-';

	IF ( NOT v_exist)
		THEN RAISE client_not_exist;
		ELSE pr_baixa_comptadors_client(fn_cerca_id_client_x_DNI_NIF(t_DNI_NIF));
			UPDATE CLIENTS SET
			data_status = SYSDATE
                        WHERE DNI_NIF = t_DNI_NIF;
      			pr_alta_log_aplicacio(procediment, entrada, sortida);
	END IF;

EXCEPTION
        WHEN client_not_exist THEN
	    ROLLBACK;
	    sortida := '-20074 - El client '||t_DNI_NIF||' no existeix o est inactiu.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una baixa de client';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;
END;



/* ---------------------------------------------------------------------------- TRACTAMENT DE COMPTADORS

   -----------------------------------------------------------------------------------------------------

   --------------------------------------------------- Alta de comptador
Si existeix i est actiu ens dona un error -20080
Si existeix, per no est actiu l'actualitza
Si el client no existeix o est desactivat dona un error -20082
Si el model no existeix dona el missatge -20084
Si el pas no existeix dona el missatge -20086
Si no existeix el dona d'alta
*/

PROCEDURE pr_alta_comptador
	(t_numero_contracte IN COMPTADORS.numero_contracte%TYPE,
	t_DNI_NIF IN CLIENTS.DNI_NIF%TYPE,
	t_pot_contractada IN COMPTADORS.pot_contractada%TYPE,
	t_adreca IN COMPTADORS.adreca%TYPE,
	t_cp IN COMPTADORS.cp%TYPE,
	t_localitat IN COMPTADORS.localitat%TYPE,
	t_pais IN COMPTADORS.pais%TYPE,
	t_model IN COMPTADORS.model%TYPE,
	t_any_fabricacio IN COMPTADORS.any_fabricacio%TYPE,
	t_data_instalacio IN COMPTADORS.data_instalacio%TYPE,
	t_data_ultima_revisio IN COMPTADORS.data_ultima_revisio%TYPE,
	t_alta_disponibilitat IN COMPTADORS.alta_disponibilitat%TYPE
	)
IS
	v_exist BOOLEAN;
	v_inactiu BOOLEAN;
	v_exist_client BOOLEAN;
	V_exist_model BOOLEAN;
	V_exist_pais BOOLEAN;
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_alta_comptador';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';


	client_not_exist EXCEPTION;
	model_not_exist EXCEPTION;
	pais_not_exist EXCEPTION;
	comptador_exist EXCEPTION;
        v_id_client NUMBER;
BEGIN

	v_exist := FN_COMPTADOR_EXIST_X_NC(t_numero_contracte);
	v_inactiu := FN_COMPTADOR_INACTIU_X_NC(t_numero_contracte);

  	IF (general.fn_client_exist_x_DNI_NIF(t_DNI_NIF))
		THEN v_id_client := general.fn_cerca_id_client_x_DNI_NIF(t_DNI_NIF);
		 	v_exist_client := general.FN_ID_EXIST_DINAMIC('CLIENTS', v_id_client);
		ELSE v_exist_client := FALSE;
	END IF;

	v_exist_model := FN_MODEL_COMPTADOR_EXIST(t_model);
	v_exist_pais := FN_PAIS_EXIST(t_pais);
        entrada := '-'||t_numero_contracte||'-'||t_DNI_NIF||'-'||t_pot_contractada||'-'||t_adreca||'-'||t_cp||'-'
			||t_localitat||'-'||t_pais||'-'||t_model||'-'||t_any_fabricacio||'-'||t_data_instalacio||'-'
			||t_data_ultima_revisio||'-'||t_alta_disponibilitat||'-';

	IF (NOT v_exist_client)
		THEN RAISE client_not_exist;
		ELSIF (NOT v_exist_model)
			THEN RAISE model_not_exist;
			ELSIF (NOT v_exist_pais)
				THEN RAISE pais_not_exist;
				ELSIF (v_exist)
					THEN RAISE comptador_exist;
					ELSIF (v_inactiu)
					        THEN UPDATE COMPTADORS SET
							numero_contracte = t_numero_contracte,
							client = v_id_client,
							pot_contractada = t_pot_contractada,
							adreca = t_adreca,
							cp = t_cp,
							localitat = t_localitat,
							pais = t_pais,
							lectura_acumulada = 0,
							model = t_model,
							any_fabricacio = t_any_fabricacio,
							data_instalacio = t_data_instalacio,
							data_ultima_revisio = t_data_ultima_revisio,
							data_ultima_lectura = NULL,
							consum_mitja_anual = 0,
							alta_disponibilitat = t_alta_disponibilitat,
							data_status = NULL
 				               		WHERE numero_contracte = t_numero_contracte;
							pr_alta_log_aplicacio(procediment, entrada,sortida);
						ELSE INSERT INTO COMPTADORS(numero_contracte, client, pot_contractada, adreca,cp,localitat,
								pais,lectura_acumulada, model,any_fabricacio, data_instalacio, data_ultima_revisio,
								data_ultima_lectura, consum_mitja_anual, alta_disponibilitat, data_status)
							VALUES (t_numero_contracte, v_id_client, t_pot_contractada, t_adreca, t_cp, t_localitat,
								t_pais,0, t_model, t_any_fabricacio, t_data_instalacio, t_data_ultima_revisio,
								NULL, 0, t_alta_disponibilitat, NULL);
							pr_alta_log_aplicacio(procediment, entrada,sortida);
	END IF;

EXCEPTION
        WHEN client_not_exist THEN
	    ROLLBACK;
	    sortida := '-20082 - El client '||t_DNI_NIF||' no existeix o est inactiu.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN model_not_exist THEN
	    ROLLBACK;
	    sortida := '-20084 - El model '||t_model||' no existeix.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN pais_not_exist THEN
	    ROLLBACK;
	    sortida := '-20086 - El pas '||t_pais||' no existeix.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN comptador_exist THEN
	    ROLLBACK;
	    sortida := '-20080 - El comptador '||t_numero_contracte||' ja existeix i est actiu.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una alta de comptador';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

END;


/* --------------------------------------------------- Modificaci de comptador
Si no existeix o est inactiu ens dona un error -20088
Si el client no existeix o est inactiu ens dona l'error -20082
Si el nou model de comptador no existeix ens dona l'error -20084
Si el nou pas no existeix ens dona l'error -20086
Si existeix i est actiu l'actualitza
*/

PROCEDURE pr_modifica_comptador
	(t_numero_contracte IN COMPTADORS.numero_contracte%TYPE,
	t_DNI_NIF IN CLIENTS.DNI_NIF%TYPE,
	t_pot_contractada IN COMPTADORS.pot_contractada%TYPE,
	t_adreca IN COMPTADORS.adreca%TYPE,
	t_cp IN COMPTADORS.cp%TYPE,
	t_localitat IN COMPTADORS.localitat%TYPE,
	t_pais IN COMPTADORS.pais%TYPE,
	t_model IN COMPTADORS.model%TYPE,
	t_any_fabricacio IN COMPTADORS.any_fabricacio%TYPE,
	t_data_instalacio IN COMPTADORS.data_instalacio%TYPE,
	t_data_ultima_revisio IN COMPTADORS.data_ultima_revisio%TYPE,
	t_alta_disponibilitat IN COMPTADORS.alta_disponibilitat%TYPE
	)
IS

	v_exist BOOLEAN;
	v_exist_client BOOLEAN;
	V_exist_model BOOLEAN;
	V_exist_pais BOOLEAN;
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_modifica_comptador';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	comptador_not_exist EXCEPTION;
	client_not_exist EXCEPTION;
	model_not_exist EXCEPTION;
	pais_not_exist EXCEPTION;
	v_id_client NUMBER;
BEGIN

	v_exist := FN_COMPTADOR_EXIST_X_NC(t_numero_contracte);

 	IF (general.fn_client_exist_x_DNI_NIF(t_DNI_NIF))
		THEN v_id_client := general.fn_cerca_id_client_x_DNI_NIF(t_DNI_NIF);
		 	v_exist_client := general.FN_ID_EXIST_DINAMIC('CLIENTS', v_id_client);
		ELSE v_exist_client := FALSE;
	END IF;

	v_exist_model := FN_MODEL_COMPTADOR_EXIST(t_model);
	v_exist_pais := FN_PAIS_EXIST(t_pais);
        entrada := '-'||t_numero_contracte||'-'||t_DNI_NIF||'-'||t_pot_contractada||'-'||t_adreca||'-'||t_cp||'-'
			||t_localitat||'-'||t_pais||'-'||t_model||'-'||t_any_fabricacio||'-'||t_data_instalacio||'-'
			||t_data_ultima_revisio||'-'||t_alta_disponibilitat||'-';

	IF (NOT v_exist_client)
		THEN RAISE client_not_exist;
		ELSIF (NOT v_exist_model)
			THEN RAISE model_not_exist;
			ELSIF (NOT v_exist_pais)
				THEN RAISE pais_not_exist;
				ELSIF (NOT v_exist)
					THEN RAISE comptador_not_exist;
					ELSE UPDATE COMPTADORS SET
						numero_contracte = t_numero_contracte,
						client = v_id_client,
						pot_contractada = t_pot_contractada,
						adreca = t_adreca,
						cp = t_cp,
						localitat = t_localitat,
						pais = t_pais,
						lectura_acumulada = 0,
						model = t_model,
						any_fabricacio = t_any_fabricacio,
						data_instalacio = t_data_instalacio,
						data_ultima_revisio = t_data_ultima_revisio,
						data_ultima_lectura = NULL,
						consum_mitja_anual = 0,
						alta_disponibilitat = t_alta_disponibilitat,
						data_status = NULL
 			               		WHERE numero_contracte = t_numero_contracte;
						pr_alta_log_aplicacio(procediment, entrada,sortida);
	END IF;

EXCEPTION
        WHEN client_not_exist THEN
	    ROLLBACK;
	    sortida := '-20082 - El client '||t_DNI_NIF||' no existeix o est inactiu.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN model_not_exist THEN
	    ROLLBACK;
	    sortida := '-20084 - El model '||t_model||' no existeix.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN pais_not_exist THEN
	    ROLLBACK;
	    sortida := '-20086 - El pas '||t_pais||' no existeix.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN comptador_not_exist THEN
	    ROLLBACK;
	    sortida := '-20088 - El comptador '||t_numero_contracte||' no existeix o est inacti.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una modificaci de comptador';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

END;


/* --------------------------------------------------- Baixa de comptador
Si no existeix o est inactiu ens dona un error -20088
Si existeix i est actiu el dona de baixa lgica
*/

PROCEDURE pr_baixa_comptador
	(t_numero_contracte IN COMPTADORS.numero_contracte%TYPE
	)
IS
	v_exist BOOLEAN;
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_baixa_comptador';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	comptador_not_exist EXCEPTION;
BEGIN

	v_exist := FN_COMPTADOR_EXIST_X_NC(t_numero_contracte);
        entrada := '-'||t_numero_contracte||'-';

	IF (NOT v_exist)
		THEN RAISE comptador_not_exist;
-- <---------------------------------- Dona de baixa lgica la relaci centraletes_comptadors.
		ELSE pr_baixa_rel_centraletes_comp(fn_cerca_id_comptador_per_nc(t_numero_contracte));
			UPDATE COMPTADORS SET
			data_status = SYSDATE
                        WHERE numero_contracte = t_numero_contracte;
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	END IF;

EXCEPTION
        WHEN comptador_not_exist THEN
	    ROLLBACK;
	    sortida := '-20088 - El comptador '||t_numero_contracte||' no existeix o est inacti.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una baixa de comptador';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

END;



/* --------------------------------------------------------------------------- TRACTAMENT DE CENTRALETES

   -----------------------------------------------------------------------------------------------------

   --------------------------------------------------- Alta de centraleta
Si la lnia a la que es vol connectar la centraleta no existeig o est inactiva ens dona un error -20062
Si el pas no existeix ens dona l'error -20064
Si existeix i est activa ens dona un error -20060
Si existeix, per no est actiu l'actualitza
Si no existeix el dona d'alta
*/

PROCEDURE pr_alta_centraleta
	(t_linia IN CENTRALETES.linia%TYPE,
	t_adreca IN CENTRALETES.adreca%TYPE,
	t_cp IN CENTRALETES.cp%TYPE,
	t_localitat IN CENTRALETES.localitat%TYPE,
	t_pais IN CENTRALETES.pais%TYPE,
	t_capacitat_maxima IN CENTRALETES.capacitat_maxima%TYPE,
	t_alta_disponibilitat IN CENTRALETES.alta_disponibilitat%TYPE
	)
IS
	v_exist BOOLEAN;
	v_inactiu BOOLEAN;

	v_exist_linia BOOLEAN;
	v_exist_pais BOOLEAN;
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_alta_centraleta';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	linia_not_exist EXCEPTION;
	pais_not_exist EXCEPTION;
	centraleta_exist EXCEPTION;
BEGIN

	v_exist := fn_centraleta_exist_x_adr_cp(t_adreca,t_cp);
	v_inactiu := FN_CENTRALETA_INACTIU_X_ADR_CP(t_adreca,t_cp);
 	v_exist_linia := FN_ID_EXIST_DINAMIC('LINIES', t_linia);
	v_exist_pais := FN_PAIS_EXIST(t_pais);
        entrada := '-'||t_linia||'-'||t_adreca||'-'||t_cp||'-'||t_localitat||'-'||t_pais||'-'||
			t_capacitat_maxima||'-'||t_alta_disponibilitat||'-';

	IF (NOT v_exist_linia)
		THEN RAISE linia_not_exist;
		ELSIF (NOT v_exist_pais)
			THEN RAISE pais_not_exist;
			ELSIF (v_exist)
				THEN RAISE centraleta_exist;
				ELSIF (v_inactiu)
					THEN UPDATE CENTRALETES SET
						linia = t_linia,
						adreca = t_adreca,
						cp = t_cp,
						localitat = t_localitat,
						pais = t_pais,
						capacitat_maxima = t_capacitat_maxima,
						alta_disponibilitat = t_alta_disponibilitat,
						data_status = NULL
 				       		WHERE adreca = t_adreca AND cp = t_cp;
						pr_alta_log_aplicacio(procediment, entrada,sortida);
					ELSE INSERT INTO CENTRALETES (linia, adreca, cp, localitat, pais, capacitat_maxima,
							alta_disponibilitat, data_status)
						VALUES (t_linia, t_adreca, t_cp, t_localitat, t_pais, t_capacitat_maxima,
							t_alta_disponibilitat,NULL);
						pr_alta_log_aplicacio(procediment, entrada,sortida);
	END IF;

EXCEPTION
        WHEN linia_not_exist THEN
	    ROLLBACK;
	    sortida := '-20062 - La linia '||t_linia||' no existeix o est inactiva.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN pais_not_exist THEN
	    ROLLBACK;
	    sortida := '-20064 - El pas '||t_pais||' no existeix.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN centraleta_exist THEN
	    ROLLBACK;
	    sortida := '-20060 - La centraleta '||t_adreca||' - '||t_cp||' ja existeix i est activa.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una alta de centraleta';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

END;



/* --------------------------------------------------- Modificacio de centraleta
Si la lnia a la que es vol connectar la centraleta no existeig o est inactiva ens dona un error -20062
Si el pas no existeix ens dona l'error -20064
Si no existeix o est inactiva ens dona un error -20066
Si existeix el dona l'actualitza
*/

PROCEDURE pr_modifica_centraleta
	(t_linia IN CENTRALETES.linia%TYPE,
	t_adreca IN CENTRALETES.adreca%TYPE,
	t_cp IN CENTRALETES.cp%TYPE,
	t_localitat IN CENTRALETES.localitat%TYPE,
	t_pais IN CENTRALETES.pais%TYPE,
	t_capacitat_maxima IN CENTRALETES.capacitat_maxima%TYPE,
	t_alta_disponibilitat IN CENTRALETES.alta_disponibilitat%TYPE
	)
IS
	v_exist BOOLEAN;
	v_id_ctrl CENTRALETES.id%TYPE;
	v_exist_linia BOOLEAN;
	v_exist_pais BOOLEAN;
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_modifica_centraleta';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	linia_not_exist EXCEPTION;
	pais_not_exist EXCEPTION;
	centraleta_not_exist EXCEPTION;
BEGIN

	IF (fn_centraleta_exist_x_adr_cp(t_adreca,t_cp))
		THEN v_id_ctrl := fn_cerca_id_ctrl_per_adre_cp(t_adreca,t_cp);
		 	v_exist := FN_ID_EXIST_DINAMIC('CENTRALETES', v_id_ctrl);
		ELSE v_exist := FALSE;
	END IF;

        v_exist_linia := FN_ID_EXIST_DINAMIC('LINIES', t_linia);
	v_exist_pais := FN_PAIS_EXIST(t_pais);
        entrada := '-'||t_linia||'-'||t_adreca||'-'||t_cp||'-'||t_localitat||'-'||t_pais||'-'||
			t_capacitat_maxima||'-'||t_alta_disponibilitat||'-';

	IF (NOT v_exist_linia)
		THEN RAISE linia_not_exist;
		ELSIF (NOT v_exist_pais)
			THEN RAISE pais_not_exist;
			ELSIF (NOT v_exist)
				THEN RAISE centraleta_not_exist;
				ELSE UPDATE CENTRALETES SET
					linia = t_linia,
					adreca = t_adreca,
					cp = t_cp,
					localitat = t_localitat,
					pais = t_pais,
					capacitat_maxima = t_capacitat_maxima,
					alta_disponibilitat = t_alta_disponibilitat
	 			       	WHERE adreca = t_adreca	AND cp = t_cp;
					pr_alta_log_aplicacio(procediment, entrada,sortida);
	END IF;

EXCEPTION
        WHEN linia_not_exist THEN
	    ROLLBACK;
	    sortida := '-20062 - La linia '||t_linia||' no existeix o est inactiva.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN pais_not_exist THEN
	    ROLLBACK;
	    sortida := '-20064 - El pas '||t_pais||' no existeix.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN centraleta_not_exist THEN
	    ROLLBACK;
	    sortida := '-20066 - La centraleta '||t_adreca||' - '||t_cp||' no existeix o est inactiva.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una modificaci de centraleta';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

END;



/* --------------------------------------------------- Baixa de centraleta
Si no existeix o est inactiva ens dona un error -20066
Si existeix i est activa es dona de baixa lgica
*/

PROCEDURE pr_baixa_centraleta
	(t_adreca IN CENTRALETES.adreca%TYPE,
	t_cp IN CENTRALETES.cp%TYPE
	)
IS

	v_exist BOOLEAN;
        v_id_ctrl CENTRALETES.id%TYPE;
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_baixa_centraleta';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	centraleta_not_exist EXCEPTION;
BEGIN

	IF (fn_centraleta_exist_x_adr_cp(t_adreca,t_cp))
		THEN v_id_ctrl := fn_cerca_id_ctrl_per_adre_cp(t_adreca,t_cp);
		 	v_exist := FN_ID_EXIST_DINAMIC('CENTRALETES', v_id_ctrl);
		ELSE v_exist := FALSE;
	END IF;

        entrada := '-'||t_adreca||'-'||t_cp||'-';

	IF (NOT v_exist)
		THEN RAISE centraleta_not_exist;
		ELSE 
/*
		Donar de baixa a la relaci centraleta_comptador tots el registres que tinguin aquesta centraleta
*/
                        pr_baixa_centr_comp_x_linies(v_id_ctrl);
                        UPDATE CENTRALETES SET
			data_status = SYSDATE
                        WHERE adreca = t_adreca AND cp = t_cp;
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	END IF;

EXCEPTION
        WHEN centraleta_not_exist THEN
	    ROLLBACK;
	    sortida := '-20066 - La centraleta '||t_adreca||' - '||t_cp||' no existeix o est inactiva.';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una baixa de centraleta';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

END;



/* -------------------------------------------------------------------------------- TRACTAMENT DE LINIES

   -----------------------------------------------------------------------------------------------------

   --------------------------------------------------- Alta de linia
L'error 20050 no el donar mai ja que el id_linia es autmatic per el
	que en el cas d'alta no cal mirar si existeix o est activa,
	tampoc es podr modificar, noms donar d'alta.
-- Si existeix i est activa ens dona un error -20050

Si la central de producci no existex o est inactiva ens dona l'error -20052
Si existeix, per no est actiu l'actualitza
Si no existeix el dona d'alta
*/

PROCEDURE pr_alta_linia
	(t_id_central IN LINIES.central%TYPE,
	t_capacitat IN LINIES.capacitat%TYPE,
	t_alta_disponibilitat IN LINIES.alta_disponibilitat%TYPE
	)
IS
	v_exist_central BOOLEAN;
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_alta_linia';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	central_not_exist EXCEPTION;
BEGIN
 	v_exist_central := FN_ID_EXIST_DINAMIC('CENTRALS_PRODUCCIO', t_id_central);
	entrada := '-'||t_id_central||'-'||t_capacitat||'-'||t_alta_disponibilitat||'-';

	IF (NOT v_exist_central)
		THEN RAISE central_not_exist;
		ELSE INSERT INTO LINIES (central,capacitat, carrega_acumulada, alta_disponibilitat,data_status)
			VALUES (t_id_central, t_capacitat, 0, t_alta_disponibilitat, NULL);
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	END IF;

EXCEPTION
        WHEN central_not_exist THEN
	    ROLLBACK;
	    sortida := '-20052 - La central '||t_id_central||' no existeix o est inactiva.';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una alta de lnia';
			pr_alta_log_aplicacio(procediment, entrada,sortida);
	    COMMIT;

END;



/* --------------------------------------------------- Modificaci de lnies
Si no existeix o est inactiva ens dona un error -20054
Si la central de producci no existex o est inactiva ens dona l'error -20052
Si existeix i est actiu l'actualitza
*/

PROCEDURE pr_modifica_linia
	(t_id_linia IN LINIES.id%TYPE,
	t_id_central IN LINIES.central%TYPE,
	t_capacitat IN LINIES.capacitat%TYPE,
	t_alta_disponibilitat IN LINIES.alta_disponibilitat%TYPE
	)
IS

	v_exist BOOLEAN;
	v_exist_central BOOLEAN;
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_modifica_linia';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';


	linia_not_exist EXCEPTION;
	central_not_exist EXCEPTION;
BEGIN
 	v_exist := FN_ID_EXIST_DINAMIC('LINIES', t_id_linia);
 	v_exist_central := FN_ID_EXIST_DINAMIC('CENTRALS_PRODUCCIO', t_id_central);
	entrada := '-'||t_id_linia||'-'||t_id_central||'-'||t_capacitat||'-'||t_alta_disponibilitat||'-';

	IF (NOT v_exist_central)
		THEN RAISE central_not_exist;
		ELSIF (NOT v_exist)
			THEN RAISE linia_not_exist;
			ELSE UPDATE LINIES SET
				id = t_id_linia,
				central = t_id_central,
				capacitat = t_capacitat,
				alta_disponibilitat = t_alta_disponibilitat
		               	WHERE id = t_id_linia;
				pr_alta_log_aplicacio(procediment,entrada,sortida);
	END IF;

EXCEPTION
        WHEN central_not_exist THEN
	    ROLLBACK;
	    sortida := '-20052 - La central '||t_id_central||' no existeix o est inactiva.';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

        WHEN linia_not_exist THEN
	    ROLLBACK;
	    sortida := '-20054 - La lnia '||t_id_linia||' no existeix o est inactiva.';
				pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una modificaci de lnia';
				pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;
END;



/* -------------- Donar de baixa els comptadors que noms s'alimentten d'aquesta centraleta
	si estan connectats a un altre centraleta no donar-los de baixa
	Mirar si el comptador est connectat a d'altres centraletes
	Si no ho est donar de baixa el comptador
		si ho est no es dona de baixa el comptador
*/

PROCEDURE pr_baixa_comp_sense_centr
	(t_id_centraleta IN CENTRALETES_COMPTADORS.centraleta%TYPE,
	t_id_comptador IN CENTRALETES_COMPTADORS.comptador%TYPE
	)
IS
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_baixa_comp_sense_centr';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	CURSOR compt_per_centraleta IS
		select * FROM COMPTADORS
			WHERE id = t_id_comptador
				AND data_status IS NULL
				AND not exists(select * FROM CENTRALETES_COMPTADORS
						WHERE comptador = t_id_comptador
						AND data_status IS NULL
						AND centraleta <> t_id_centraleta)
			FOR UPDATE OF data_status;
	reg COMPTADORS%ROWTYPE;

BEGIN
	entrada := '-'||t_id_centraleta||'-'||t_id_comptador||'-';

	OPEN compt_per_centraleta;
	FETCH compt_per_centraleta INTO reg;
	WHILE compt_per_centraleta%FOUND LOOP
		UPDATE COMPTADORS
		SET data_status = SYSDATE
		WHERE CURRENT OF compt_per_centraleta;
		pr_alta_log_aplicacio(procediment,entrada,sortida);
		FETCH compt_per_centraleta INTO reg;
	END LOOP;

EXCEPTION
        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una baixa de comptador';
				pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

END;



/* -------------- Dona de baixa a la relaci centraleta_comptador tots el registres que tinguin aquesta centraleta
	MIrar si exiteix i donar de baixa
	Mirar si el comptador est connectat a d'altres centraletes
		Si no ho est donar de baixa el comptador
		si ho est no es dona de baixa el comptador
*/

PROCEDURE pr_baixa_centr_comp_x_linies
	(t_id_centraleta IN CENTRALETES_COMPTADORS.centraleta%TYPE
	)
IS
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_baixa_centr_comp_x_linies';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	CURSOR centr_compt_per_linia IS
		select * FROM CENTRALETES_COMPTADORS
			WHERE centraleta = t_id_centraleta AND data_status IS NULL
			FOR UPDATE OF data_status;
	reg CENTRALETES_COMPTADORS%ROWTYPE;
BEGIN
	entrada := '-'||t_id_centraleta||'-';

	OPEN centr_compt_per_linia;
	FETCH centr_compt_per_linia INTO reg;
	WHILE centr_compt_per_linia%FOUND LOOP
/*
		Donar de baixa els comptadors que noms s'alimentten d'aquesta centraleta
		si estan connectats a un altre centraleta no donar-los de baixa
*/
                pr_baixa_comp_sense_centr(reg.centraleta, reg.comptador);
		UPDATE CENTRALETES_COMPTADORS
		SET data_status = SYSDATE
		WHERE CURRENT OF centr_compt_per_linia;
		pr_alta_log_aplicacio(procediment,entrada,sortida);
		FETCH centr_compt_per_linia INTO reg;
	END LOOP;

EXCEPTION
        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una baixa de la relac centraletes_comptadors';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

END;



/* ------------------------------------ Dona de baixa lgica les centraletes que estan connectades a aquesta lnia
		Una centraleta noms pot tenir una lnia, per una lnia pot tenir moltes centraletes
*/


PROCEDURE pr_baixa_centraletes_linia
	(t_id_linia IN LINIES.id%TYPE
	)
IS
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_baixa_centraletes_linia';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	CURSOR centraletes_linia IS
		select * FROM CENTRALETES
			WHERE linia = t_id_linia
			FOR UPDATE OF data_status;
	reg CENTRALETES%ROWTYPE;
BEGIN
	entrada := '-'||t_id_linia||'-';

	OPEN centraletes_linia;
	FETCH centraletes_linia INTO reg;
	WHILE centraletes_linia%FOUND LOOP
/*
		Donar de baixa a la relaci centraleta_comptador tots el registres que tinguin aquesta centraleta
*/
                pr_baixa_centr_comp_x_linies(reg.id);
		UPDATE CENTRALETES
		SET data_status = SYSDATE
		WHERE CURRENT OF centraletes_linia;
		pr_alta_log_aplicacio(procediment,entrada,sortida);
		FETCH centraletes_linia INTO reg;
	END LOOP;

EXCEPTION
        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una baixa de centraleta';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

END;



/* --------------------------------------------------- Baixa de lnies
Si no existeix o es inactiva ens dona un error -20054
Si existeix i est activa es dona de baixa lgica
*/

PROCEDURE pr_baixa_linia
	(t_id_linia IN LINIES.id%TYPE
	)
IS

	v_exist BOOLEAN;
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_baixa_linia';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	linia_not_exist EXCEPTION;
BEGIN
 	v_exist := FN_ID_EXIST_DINAMIC('LINIES', t_id_linia);
	entrada := '-'||t_id_linia||'-';

	IF (NOT v_exist)
		THEN RAISE linia_not_exist;
-- <---------------------------------- Dona de baixa lgica la centraletes que estan connectades a aquesta lnia
		ELSE pr_baixa_centraletes_linia(t_id_linia);
			UPDATE LINIES SET
			data_status = SYSDATE
                        WHERE id = t_id_linia;
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	END IF;

EXCEPTION

        WHEN linia_not_exist THEN
	    ROLLBACK;
	    sortida := '-20054 - La lnia '||t_id_linia||' no existeix o est inactiva.';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una baixa de lnia';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

END;


/* -------------------------------------------------------------- ASSIGNACI DE CENTRALETES - COMPTADORS

   -----------------------------------------------------------------------------------------------------

   ----------------------------------------------------- Alta de relaci centraleta_comptador
Si la centraleta no existeix o est inactiva dona l'error -20092
Si el comptador no existeix o est inactiva dona l'error -20094
Si la relaci centraleta_coptador ja existeix i est activa dona l'error -20090
Si la relaci existeix per est inactiva, l'activa
Si la relaci no existeix la dona d'alta.
*/


PROCEDURE pr_alta_centraleta_comptador
	(t_adreca IN CENTRALETES.adreca%type,
        t_cp IN CENTRALETES.cp%TYPE,
        t_numero_contracte IN COMPTADORS.numero_contracte%TYPE
        )
        
IS

	v_exist BOOLEAN;
	v_actiu BOOLEAN;

	v_id_comptador COMPTADORS.id%TYPE;
	v_id_centraleta CENTRALETES.id%TYPE;

	v_exist_centraleta BOOLEAN;
	v_exist_comptador BOOLEAN;

	procediment LOG_APLICACIO.procediment%TYPE := 'pr_alta_centraleta_comptador';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	centraleta_not_exist EXCEPTION;
	comptador_not_exist EXCEPTION;
	rel_cent_compt_exist EXCEPTION;

BEGIN
      entrada := '-'||t_adreca||'-'||t_cp||'-'||t_numero_contracte||'-';


      v_exist_centraleta := fn_centraleta_exist_x_adr_cp(t_adreca, t_cp);
      IF (v_exist_centraleta)
          THEN v_id_centraleta := fn_cerca_id_ctrl_per_adre_cp(t_adreca, t_cp);
          ELSE v_exist_centraleta := FALSE;
      END IF;

      v_exist_comptador := fn_comptador_exist_x_NC(t_numero_contracte);
      IF (v_exist_comptador)
          THEN v_id_comptador := fn_cerca_id_comptador_per_NC(t_numero_contracte);
          ELSE v_exist_comptador := FALSE;
      END IF;
      
      v_exist := FN_CENTR_COMP_EXIST(v_id_centraleta, v_id_comptador);
      v_actiu := FN_CENTR_COMP_ACTIU(v_id_centraleta, v_id_comptador);

	IF (NOT v_exist_centraleta)
		THEN RAISE centraleta_not_exist;
		ELSIF (NOT v_exist_comptador)
			THEN RAISE comptador_not_exist;
			ELSIF (v_exist)
				THEN RAISE rel_cent_compt_exist;
				ELSIF (NOT v_actiu)
					THEN UPDATE CENTRALETES_COMPTADORS SET
						centraleta = v_id_centraleta,
						comptador = v_id_comptador,
						data_status = NULL
			               		WHERE centraleta = v_id_centraleta AND comptador = v_id_comptador;
						pr_alta_log_aplicacio(procediment,entrada,sortida);
						ELSE INSERT INTO CENTRALETES_COMPTADORS(centraleta, comptador, data_status)
							VALUES (v_id_centraleta, v_id_comptador, NULL);
                                                        pr_alta_log_aplicacio(procediment,entrada,sortida);
	END IF;

EXCEPTION

        WHEN centraleta_not_exist THEN
	    ROLLBACK;
	    sortida := '-20092 - La centraleta '||t_adreca||'-'||t_cp||' no existeix o est inactiva.';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

        WHEN comptador_not_exist THEN
	    ROLLBACK;
	    sortida := '-20094 - El comptador '||t_numero_contracte||' no existeix o est inactiva.';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;


        WHEN rel_cent_compt_exist THEN
	    ROLLBACK;
	    sortida := '-20090 - La relaci '||t_adreca||'-'||t_cp||' <-> '||t_numero_contracte||' ja existeix i est activa.';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una alta de relaci centraleta-comptador';
            DBMS_OUTPUT.PUT_LINE('entrada'|| entrada|| '-'|| sortida ||'sortida');
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

END;



/* ----------------------------------------------------- Baixa de relaci centraleta_comptador
Si la centraleta no existeix o est inactiva dona l'error -20092
Si el comptador no existeix o est inactiva dona l'error -20094
Si la relaci centraleta_coptador no existeix o est inactiva dona l'error -20096
Si la relaci existeix i est activa la dona de baixa lgica.
*/

PROCEDURE pr_baixa_centraleta_comptador
	(t_adreca IN CENTRALETES.adreca%type,
        t_cp IN CENTRALETES.cp%TYPE,
        t_numero_contracte IN COMPTADORS.numero_contracte%TYPE
        )
 
IS

	v_exist BOOLEAN;
	v_actiu BOOLEAN;

	v_id_comptador COMPTADORS.id%TYPE;
	v_id_centraleta CENTRALETES.id%TYPE;

	v_exist_centraleta BOOLEAN;
	v_exist_comptador BOOLEAN;

	procediment LOG_APLICACIO.procediment%TYPE := 'pr_baixa_centraleta_comptador';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	centraleta_not_exist EXCEPTION;
	comptador_not_exist EXCEPTION;
	rel_cent_comp_not_exist EXCEPTION;
BEGIN

      entrada := '-'||t_adreca||'-'||t_cp||'-'||t_numero_contracte||'-';


      v_exist_centraleta := fn_centraleta_exist_x_adr_cp(t_adreca, t_cp);
      IF (v_exist_centraleta)
          THEN v_id_centraleta := fn_cerca_id_ctrl_per_adre_cp(t_adreca, t_cp);
          ELSE v_exist_centraleta := FALSE;
      END IF;

      v_exist_comptador := fn_comptador_exist_x_NC(t_numero_contracte);
      IF (v_exist_comptador)
          THEN v_id_comptador := fn_cerca_id_comptador_per_NC(t_numero_contracte);
          ELSE v_exist_comptador := FALSE;
      END IF;

	v_exist := FN_CENTR_COMP_EXIST(v_id_centraleta, v_id_comptador);
	v_actiu := FN_CENTR_COMP_ACTIU(v_id_centraleta, v_id_comptador);

	IF (NOT v_exist_centraleta)
		THEN RAISE centraleta_not_exist;
		ELSIF (NOT v_exist_comptador)
			THEN RAISE comptador_not_exist;
			ELSIF (NOT v_exist)
				THEN RAISE rel_cent_comp_not_exist;
				ELSE UPDATE CENTRALETES_COMPTADORS SET
					data_status = SYSDATE
			               	WHERE centraleta = v_id_centraleta AND comptador = v_id_comptador;
					pr_alta_log_aplicacio(procediment,entrada,sortida);

	END IF;

EXCEPTION

        WHEN centraleta_not_exist THEN
	    ROLLBACK;
	    sortida := '-20092 - La centraleta '||t_adreca||'-'||t_cp||' no existeix o est inactiva.';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

        WHEN comptador_not_exist THEN
	    ROLLBACK;
	    sortida := '-20094 - El comptador '||t_numero_contracte||' no existeix o est inactiva.';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

        WHEN rel_cent_comp_not_exist THEN
	    ROLLBACK;
	    sortida := '-20096 - La relaci '||t_adreca||'-'||t_cp||' <-> '||t_numero_contracte||' no existeix o est inactiva.';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una baixa de relaci centraleta-comptador';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

END;


/* ----------------------------------------------------------------- TRACTAMENT DE CENTRALS DE PRODUCCIO

--------------------------------------------------------------------------------------------------------
*/


/* --------------------------------------------------- Alta de central de producci
Si existeix i est activa ens dona l'error -20040
Si el tipus de central no existeix ens dona l'error 20042
Si el pas no existeix ens dona l'error -20044
Si no existeix el dona d'alta
*/

PROCEDURE pr_alta_central
	(t_nom IN CENTRALS_PRODUCCIO.nom%TYPE,
 	t_tipus IN CENTRALS_PRODUCCIO.tipus%TYPE,
	t_adreca IN CENTRALS_PRODUCCIO.adreca%TYPE,
	t_cp IN CENTRALS_PRODUCCIO.cp%TYPE,
	t_localitat IN CENTRALS_PRODUCCIO.localitat%TYPE,
	t_pais IN CENTRALS_PRODUCCIO.pais%TYPE,
	t_telefon IN CENTRALS_PRODUCCIO.telefon%TYPE,
	t_e_mail IN CENTRALS_PRODUCCIO.e_mail%TYPE,
	t_energia_maxima IN CENTRALS_PRODUCCIO.energia_maxima%TYPE,
	t_data_ult_revisio IN CENTRALS_PRODUCCIO.data_ult_revisio%TYPE,
	t_energia_minima_NU IN CENTRALS_PRODUCCIO.energia_minima_NU%TYPE,
	t_rebuig_radioactiu IN CENTRALS_PRODUCCIO.rebuig_radioactiu%TYPE,
	t_kg_emisio_CO2 IN CENTRALS_PRODUCCIO.kg_emisio_CO2%TYPE,
 	t_unitats_produccio IN CENTRALS_PRODUCCIO.unitats_produccio%TYPE,
	t_alta_disponibilitat IN CENTRALS_PRODUCCIO.alta_disponibilitat%TYPE
	)
IS

	v_exist BOOLEAN;
	v_actiu BOOLEAN;
	v_exist_pais BOOLEAN;
	v_exist_tipus BOOLEAN;


	procediment LOG_APLICACIO.procediment%TYPE := 'pr_alta_central';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	tipus_not_exist EXCEPTION;
	pais_not_exist EXCEPTION;
	central_exist EXCEPTION;
BEGIN

	v_exist := FN_CENTRAL_EXIST_X_NOM(t_nom);
	v_actiu := FN_CENTRAL_ACTIU_X_NOM(t_nom);
	v_exist_pais := FN_PAIS_EXIST(t_pais);
	v_exist_tipus := FN_TIPUS_CENTRAL_EXIST(t_tipus);

	entrada := '-'||t_nom||'-'||t_tipus||'-'||t_adreca||'-'||t_cp||'-'||t_localitat||'-'||t_pais||'-'||
		t_telefon||'-'||t_e_mail||'-'||t_energia_maxima||'-'||t_data_ult_revisio||'-'||t_energia_minima_NU||'-'||
		t_rebuig_radioactiu||'-'||t_kg_emisio_CO2||'-'||t_unitats_produccio||'-'||t_alta_disponibilitat||'-';


	IF (NOT v_exist_tipus)
		THEN RAISE tipus_not_exist;
		ELSIF (NOT v_exist_pais)
			THEN RAISE pais_not_exist;
		ELSIF (v_exist)
			THEN RAISE central_exist;
			ELSIF (NOT v_actiu)
				THEN UPDATE CENTRALS_PRODUCCIO SET
					nom = t_nom,
 					tipus = t_tipus,
					adreca = t_adreca,
					cp = t_cp,
					localitat = t_localitat,
					pais = t_pais,
					telefon = t_telefon,
					e_mail = t_e_mail,
					energia_maxima = t_energia_maxima,
					data_ult_revisio = t_data_ult_revisio,
					energia_minima_NU = t_energia_minima_NU,
					rebuig_radioactiu = t_rebuig_radioactiu,
					kg_emisio_CO2 = t_kg_emisio_CO2,
 					unitats_produccio = t_unitats_produccio,
					alta_disponibilitat = t_alta_disponibilitat,
					data_status = NULL
	     				WHERE nom = t_nom;
					pr_alta_log_aplicacio(procediment,entrada,sortida);
				ELSE INSERT INTO CENTRALS_PRODUCCIO (nom,tipus,adreca,cp,localitat,pais,telefon,e_mail,
						energia_maxima,data_ult_revisio,energia_minima_NU,rebuig_radioactiu,Kg_emisio_CO2,
						unitats_produccio,alta_disponibilitat,data_status)
					VALUES (t_nom, t_tipus, t_adreca, t_cp, t_localitat, t_pais, t_telefon, t_e_mail,
						t_energia_maxima, t_data_ult_revisio, t_energia_minima_NU, t_rebuig_radioactiu, t_Kg_emisio_CO2,
						t_unitats_produccio,t_alta_disponibilitat,NULL);
					pr_alta_log_aplicacio(procediment,entrada,sortida);
	END IF;

EXCEPTION
        WHEN tipus_not_exist THEN
	    ROLLBACK;
	    sortida := '-20042 - El tipus de central '||t_tipus||' no existeix.';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

        WHEN pais_not_exist THEN
	    ROLLBACK;
	    sortida := '-20044 - El pais '||t_pais||' no existeix.';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

        WHEN central_exist THEN
	    ROLLBACK;
	    sortida := '-20040 - La central '||t_nom||' ja existeix i est activa.';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una alta de central de producci';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

END;


/* --------------------------------------------------- Modificaci de centrals de producci
Si el tipus de central no existeix ens dona un error -20042
Si el pas no existeix ens dona un error -20044
Si la central de producci no existex o est inactiva ens dona l'error -20046
Si existeix i est activa l'actualitza
*/

PROCEDURE pr_modifica_centrals
	(t_nom IN CENTRALS_PRODUCCIO.nom%TYPE,
 	t_tipus IN CENTRALS_PRODUCCIO.tipus%TYPE,
	t_adreca IN CENTRALS_PRODUCCIO.adreca%TYPE,
	t_cp IN CENTRALS_PRODUCCIO.cp%TYPE,
	t_localitat IN CENTRALS_PRODUCCIO.localitat%TYPE,
	t_pais IN CENTRALS_PRODUCCIO.pais%TYPE,
	t_telefon IN CENTRALS_PRODUCCIO.telefon%TYPE,
	t_e_mail IN CENTRALS_PRODUCCIO.e_mail%TYPE,
	t_energia_maxima IN CENTRALS_PRODUCCIO.energia_maxima%TYPE,
	t_data_ult_revisio IN CENTRALS_PRODUCCIO.data_ult_revisio%TYPE,
	t_energia_minima_NU IN CENTRALS_PRODUCCIO.energia_minima_NU%TYPE,
	t_rebuig_radioactiu IN CENTRALS_PRODUCCIO.rebuig_radioactiu%TYPE,
	t_kg_emisio_CO2 IN CENTRALS_PRODUCCIO.kg_emisio_CO2%TYPE,
 	t_unitats_produccio IN CENTRALS_PRODUCCIO.unitats_produccio%TYPE,
	t_alta_disponibilitat IN CENTRALS_PRODUCCIO.alta_disponibilitat%TYPE
	)

IS

	v_exist BOOLEAN;
	v_actiu BOOLEAN;
	v_exist_nom_central BOOLEAN;
	v_exist_pais BOOLEAN;
	v_exist_tipus BOOLEAN;
	v_id_central NUMBER;
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_modifica_centralS';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	tipus_not_exist EXCEPTION;
	pais_not_exist EXCEPTION;
	central_not_exist EXCEPTION;
BEGIN

	IF (fn_exist_central_x_nom(t_nom))
		THEN v_id_central := FN_CERCA_ID_CENTRAL_X_NOM(t_nom);
		 	v_exist := FN_ID_EXIST_DINAMIC('CENTRALS_PRODUCCIO', v_id_central);
		ELSE v_exist := FALSE;
	END IF;

	v_exist_pais := FN_PAIS_EXIST(t_pais);
	v_exist_tipus := FN_TIPUS_CENTRAL_EXIST(t_tipus);
	entrada := '-'||t_nom||'-'||t_tipus||'-'||t_adreca||'-'||t_cp||'-'||t_localitat||'-'||t_pais||'-'||
		t_telefon||'-'||t_e_mail||'-'||t_energia_maxima||'-'||t_data_ult_revisio||'-'||t_energia_minima_NU||'-'||
		t_rebuig_radioactiu||'-'||t_kg_emisio_CO2||'-'||t_unitats_produccio||'-'||t_alta_disponibilitat||'-';


	IF (NOT v_exist_tipus)
		THEN RAISE tipus_not_exist;
		ELSIF (NOT v_exist_pais)
			THEN RAISE pais_not_exist;
			ELSIF (NOT v_exist)
				THEN RAISE central_not_exist;
				ELSE UPDATE CENTRALS_PRODUCCIO SET
					nom = t_nom,
 					tipus = t_tipus,
					adreca = t_adreca,
					cp = t_cp,
					localitat = t_localitat,
					pais = t_pais,
					telefon = t_telefon,
					e_mail = t_e_mail,
					energia_maxima = t_energia_maxima,
					data_ult_revisio = t_data_ult_revisio,
					energia_minima_NU = t_energia_minima_NU,
					rebuig_radioactiu = t_rebuig_radioactiu,
					kg_emisio_CO2 = t_kg_emisio_CO2,
 					unitats_produccio = t_unitats_produccio,
					alta_disponibilitat = t_alta_disponibilitat,
					data_status = NULL
					WHERE nom = t_nom;
					pr_alta_log_aplicacio(procediment,entrada,sortida);
	END IF;

EXCEPTION
        WHEN tipus_not_exist THEN
	    ROLLBACK;
	    sortida := '-20042 - El tipus de central '||t_tipus||' no existeix.';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

        WHEN pais_not_exist THEN
	    ROLLBACK;
	    sortida := '-20044 - El pais '||t_pais||' no existeix.';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

        WHEN central_not_exist THEN
	    ROLLBACK;
	    sortida := '-20046 - La central '||t_nom||' no existeix o est inactiva.';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una modificaci de central de producci';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

END;



/* --------------------- Dona de baixa lgica les lnies que estan connectades a aquesta central de producci
		Una lnia noms pot estar connectada a una central, per una central pot tenir moltes lnies
*/

PROCEDURE pr_baixa_linia_central
	(t_id_central IN LINIES.central%TYPE
	)
IS
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_baixa_linia_central';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	CURSOR linies_central IS
		select * FROM LINIES
			WHERE central = t_id_central
			FOR UPDATE OF data_status;
	reg LINIES%ROWTYPE;
BEGIN
	entrada := '-'||t_id_central||'-';

	OPEN linies_central;
	FETCH linies_central INTO reg;
	WHILE linies_central%FOUND LOOP
/*
		Donar de baixa a les lnies que tinguin aquesta central
*/
                pr_baixa_centraletes_linia(reg.id);
		UPDATE LINIES
		SET data_status = SYSDATE
		WHERE CURRENT OF linies_central;
		pr_alta_log_aplicacio(procediment,entrada,sortida);
		FETCH linies_central INTO reg;
	END LOOP;

EXCEPTION
        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una baixa de lnia';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

END;



/* --------------------------------------------------- Baixa de centrals de producci
Si no existeix o est inactiva ens dona un error -20046
Si existeix i est actiu el dona de baixa lgica
*/

PROCEDURE pr_baixa_central
	(t_nom IN CENTRALS_PRODUCCIO.nom%TYPE
	)
IS

v_exist BOOLEAN;
v_id_central NUMBER;

	procediment LOG_APLICACIO.procediment%TYPE := 'pr_baixa_central';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	central_not_exist EXCEPTION;
BEGIN

	IF (fn_exist_central_x_nom(t_nom))
		THEN v_id_central := FN_CERCA_ID_CENTRAL_X_NOM(t_nom);
		 	v_exist := FN_ID_EXIST_DINAMIC('CENTRALS_PRODUCCIO', v_id_central);
		ELSE v_exist := FALSE;
	END IF;

	entrada := '-'||t_nom||'-';

	IF (NOT v_exist)
		THEN RAISE central_not_exist;
		ELSE pr_baixa_linia_central(v_id_central);
			UPDATE CENTRALS_PRODUCCIO SET
			data_status = SYSDATE
                        WHERE nom = t_nom;
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	END IF;

EXCEPTION
        WHEN central_not_exist THEN
	    ROLLBACK;
	    sortida := '-20046 - La central '||t_nom||' no existeix o est inactiva.';
				pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

        WHEN OTHERS THEN
	    ROLLBACK;
            sortida := 'Error'||SQLCODE||SQLERRM||'- Al donar una baixa de central de producci';
			pr_alta_log_aplicacio(procediment,entrada,sortida);
	    COMMIT;

END;

END;
/
disconnect;
