connect Administrador/a12345

CREATE OR REPLACE
PACKAGE GENERAL IS

/* -------------------------------------------------------------------------- FUNCIONS GENERALS

-----------------------------------------------------------------------------------------------
*/

/* ------------------------------------------ Comprova que el id existeix 

Totes les taules que utilitzen aquesta funci han de tenir el camp identificador amb nom id
Funci generica que comprova l'existncia a una taula d'un ndex

Parmetres d'entrada:
t_id - nmero d'ndex a cercar
taula - nom de la taula a cercar

Parmetre de sortida:
Retorna TRUE si existeix.
Retorna FALSE si no existeix.
*/

FUNCTION fn_id_exist_dinamic
	(t_taula IN VARCHAR2,
	t_id IN NUMBER
	)
	RETURN BOOLEAN;


/* -------------------------------------------- Comprova que el status s inactiu per id
Totes les taules que utilitzen aquesta funci han de tenir el camp identificador amb nom id
Funci generica que comprova di un id existent est actiu a una taula per ndex

Parmetres d'entrada:
t_id - nmero d'ndex a cercar
taula - nom de la taula a cercar

Parmetre de sortida:
Retorna TRUE si el client est inactiu
Retorna FALSE si el client est actiu	
*/

FUNCTION fn_id_inactiu_dinamic
	(t_taula IN VARCHAR2,
	t_id IN NUMBER
	)
	RETURN BOOLEAN;


/* ------------------------------------------------------------------------ FUNCIONS DE PAISOS

-----------------------------------------------------------------------------------------------
*/


/* ------------------------------------------ Comprova que el pais existeix 
Retorna TRUE si existeix.
Retorna FALSE si no existeix.
*/

FUNCTION fn_pais_exist
	(t_id_pais IN PAISOS.id_pais%TYPE)
	RETURN BOOLEAN;

/* ------------------------------------------ Comprova que el pais est inactiu
Retorna TRUE si existeix i est inactiu.
Retorna FALSE si no existeix i est actiu.
*/

FUNCTION fn_pais_inactiu
	(t_id_pais IN PAISOS.id_pais%TYPE)
	RETURN BOOLEAN;


/* ------------------------------ Comprova que el pais no te cap relaci amb d'altres taules
Noms cal comprovar l'existncia de centrals de producci i de clients
Les lnies no tenen l'atribut pas.
Si no hi ha centrals de producci no hi haur centraletes connectades a aquesta central actives.
Si no hi ha centraletes actives tampoc hi haur comptadors actius connectats a aquestes centraletes

 
Retorna 'central' si existeix una central d'aquest pas.
Retorna 'client' si existeix una client d'aquest pas.
Retorna NULL si no existeix cao relaci d'aquest pas.

*/
FUNCTION fn_pais_relacio
	(t_id_pais IN PAISOS.id_pais%TYPE)
	RETURN VARCHAR2;

/* --------------------------------------------------------------- FUNCIONS DE TIPUS DE CENTRAL

-----------------------------------------------------------------------------------------------
*/

/* ------------------------------------------ Comprova que el tipus de central existeix 
Retorna TRUE si existeix.
Retorna FALSE si no existeix.
*/

FUNCTION fn_tipus_central_exist
	(t_id_tipus IN TIPUS_CENTRALS.id_tipus%TYPE)
	RETURN BOOLEAN;

/* ------------------------------------------ Comprova que el tipus de central est inactiu
Retorna TRUE si existeix i est inactiu.
Retorna FALSE si no existeix i est actiu.
*/

FUNCTION fn_tipus_central_inactiu
	(t_id_tipus IN TIPUS_CENTRALS.id_tipus%TYPE)
	RETURN BOOLEAN;

/* ----------------------- Comprova que el tipus de central no te centrals de producci assignades 
Retorna TRUE si existeix una central d'aquest tipus.
Retorna FALSE si no existeix cap central d'aquest tipus

*/
FUNCTION fn_tipus_centrals_relacio
	(t_id_tipus_central IN CENTRALS_PRODUCCIO.tipus%TYPE)
	RETURN BOOLEAN;

/* ----------------------------------------------------------- FUNCIONS DE MODELS DE COMPTADORS

-----------------------------------------------------------------------------------------------
*/

/* ------------------------------------------ Comprova que el model de comptador existeix 
Retorna TRUE si existeix.
Retorna FALSE si no existeix.
*/

FUNCTION fn_model_comptador_exist
	(t_id_model IN MODELS_COMPTADOR.id_model%TYPE)
	RETURN BOOLEAN;

/* ------------------------------------------ Comprova que el model de comptador est inactiu
Retorna TRUE si existeix i est inactiu.
Retorna FALSE si no existeix i est actiu.
*/

FUNCTION fn_model_comptador_inactiu
	(t_id_model IN MODELS_COMPTADOR.id_model%TYPE)
	RETURN BOOLEAN;

/* --------------------------------------- Comprova que el model de comptador no te comptadors assignades 
Retorna TRUE si existeix un comptador d'aquest model.
Retorna FALSE si no existeix algun ccomptador d'aquest model.

*/
FUNCTION fn_models_comptador_relacio
	(t_id_model IN MODELS_COMPTADOR.id_model%TYPE)
	RETURN BOOLEAN;

/* ------------------------------------------------------------------------ FUNCIONS DE CLIENTS

-----------------------------------------------------------------------------------------------
*/

/* ------------------------------------------ Comprova que el client existeix i est actiu per DNI_NIF

Retorna TRUE si el client existeix i est actiu.
Retorna FALSE si no existeix o si existeix per est inactiu.
*/

FUNCTION fn_client_exist_x_DNI_NIF
	(t_DNI_NIF IN CLIENTS.DNI_NIF%TYPE)
	RETURN BOOLEAN;

/* -------------------------------------------- Comprova que el client est inactiu per DNI_NIF
Retorna TRUE si el client est inactiu
Retorna FALSE si el client est actiu	
*/

FUNCTION fn_client_inactiu_x_DNI_NIF
	(t_DNI_NIF IN CLIENTS.DNI_NIF%TYPE)
	RETURN BOOLEAN;

/* -------------------------------------------- Cerca id_client per DNI_NIF
Entrada DNI_NIF 
Retorna id_client
*/

FUNCTION fn_cerca_id_client_x_DNI_NIF
	(t_DNI_NIF IN CLIENTS.DNI_NIF%TYPE)
	RETURN NUMBER;

/* -------------------------------------------- Cerca DNI_NIF per id_client
Entrada DNI_NIF 
Retorna id_client
*/

FUNCTION fn_cerca_DNI_NIF_x_id_client
	(t_id_client IN CLIENTS.id%TYPE)
	RETURN VARCHAR2;

/* -------------------------------------------- Comprova si el client t comptadors actius
Retorna TRUE si el client t comptadors
Retorna FALSE si el client no t comptadors	
*/

FUNCTION fn_client_comptadors_actius
	(t_DNI_NIF IN CLIENTS.DNI_NIF%TYPE)
	RETURN BOOLEAN;

/* --------------------------------------------------------------------- FUNCIONS DE COMPTADORS

-----------------------------------------------------------------------------------------------
*/

/* ------------------------------------------ Comprova que el comptador existeix i est actiu x numero de contracte
Retorna TRUE si existeix i est actiu
Retorna FALSE si no existeix o est inactiu
*/

FUNCTION fn_comptador_exist_x_NC
	(t_numero_contracte IN COMPTADORS.numero_contracte%TYPE)
	RETURN BOOLEAN;

/* -------------------------------------------- Comprova que el comptador est inactiu per NC
Retorna TRUE si est inactiu
Retorna FALSE si no existeix o est actiu
actiu
*/

FUNCTION fn_comptador_inactiu_x_nc
	(t_numero_contracte IN COMPTADORS.numero_contracte%TYPE)
	RETURN BOOLEAN;

/* -------------------------------------------- Cerca el id_comptador per el numero contracta
Entra el numero de contracte
Retorna id_comptador
*/

FUNCTION fn_cerca_id_comptador_per_NC
	(t_numero_contracte IN COMPTADORS.numero_contracte%TYPE)
	RETURN NUMBER;

/* -------------------------------------------- Cerca el numero contracta per el id_comptador
Entra id_comptador
Retorna el numero de contracte
*/

FUNCTION fn_cerca_num_contracte_per_id
	(t_id_comptador IN COMPTADORS.id%TYPE)
	RETURN NUMBER;


/* -------------------------------------------------------------------- FUNCIONS DE CENTRELETES

-----------------------------------------------------------------------------------------------
*/

/* ------------------------------------------ Comprova que la centraleta existeix i est activa per adrea i cp
Retorna TRUE si existeix i est actiu
Retorna FALSE si no existeix o est inactiu
*/

FUNCTION fn_centraleta_exist_x_adr_cp
	(t_adreca IN CENTRALETES.adreca%TYPE,
	t_cp IN CENTRALETES.cp%TYPE
	)
	RETURN BOOLEAN;

/* -------------------------------------------- Comprova que la centraleta est activa per adrea i cp
Retorna TRUE si no est actiu
Retorna FALSE si est actiu	
*/

FUNCTION fn_centraleta_inactiu_x_adr_cp
	(t_adreca IN CENTRALETES.adreca%TYPE,
	t_cp IN CENTRALETES.cp%TYPE
	)
	RETURN BOOLEAN;

/* --------------------------- Obtenir l'adrea de la centraleta a partir del id
Entra el id de la centraleta  
Retorna adrea de la centraleta	
*/

FUNCTION fn_cerca_adreca_centr_per_id
	(t_id_centraleta IN CENTRALETES.id%TYPE
	)
	RETURN VARCHAR2;

/* --------------------------- Obtenir cp de la centraleta a partir del id
	
Entra el id de la centraleta  
Retorna cp de la centraleta	
*/

FUNCTION fn_cerca_cp_centr_per_id
	(t_id_centraleta IN CENTRALETES.id%TYPE
	)
	RETURN VARCHAR2;

/* --------------------------- Obtenir l'adrea de la centraleta a partir del id
Entra l'adrea i el cp de la centraleta  
Retorna id de la centraleta	
*/

FUNCTION fn_cerca_id_ctrl_per_adre_cp
	(t_adreca IN CENTRALETES.adreca%TYPE,
	t_cp IN  CENTRALETES.cp%TYPE
	)
	RETURN NUMBER;

/* ---------------------------------------------------------- FUNCIONS DE CENTRALS DE PRODUCCIO

-----------------------------------------------------------------------------------------------
*/

/* ------------------------------------------ Comprova que la central existeix i est activa per nom
Retorna TREU si existeix i est activa
Retorna FAlSE si no existeix o est inactiva
*/

FUNCTION fn_central_exist_x_nom
      (t_nom IN CENTRALS_PRODUCCIO.nom%TYPE)
	RETURN BOOLEAN;

/* -------------------------------------------- Comprova que la central est activa per nom
Retorna TRUE si est activa
Retorna FALS si est innactiva	
*/

FUNCTION fn_central_actiu_x_nom
	(t_nom IN CENTRALS_PRODUCCIO.nom%TYPE)
	RETURN BOOLEAN;

/* -------------------------------------------- Cerca el id_central per nom
Entra el nom de la central de producci
Retorna id_central
*/

FUNCTION fn_cerca_id_central_x_nom
	(t_nom IN CENTRALS_PRODUCCIO.nom%TYPE)
	RETURN NUMBER;

/* ------------------------------------------ Comprova que la central existeix per nom
Retorna TRUE si existeixen la central.
Retorna FALSE si no existeixen.
*/

FUNCTION fn_exist_central_x_nom
	(t_nom IN CENTRALS_PRODUCCIO.nom%TYPE)
	RETURN BOOLEAN;

/* -------------------------------------------- Cerca el nom per id_central 
Entra el nom de la central de producci
Retorna id_central
*/

FUNCTION fn_cerca_nom_x_id_central
	(t_id_central IN CENTRALS_PRODUCCIO.id%TYPE)
	RETURN VARCHAR2;

/* --------------------------------------------------------- FUNCIONS DE CENTRELETES-COMPTADORS

-----------------------------------------------------------------------------------------------
*/

/* --------------------------- Comprova que la relaci centraleta-comptador existeix i est activa
Retorna TRUE si existeix i est actiu
Retorna FALSE si no existeix o est inactiu
*/
FUNCTION fn_centr_comp_exist
	(t_id_centraleta IN CENTRALETES_COMPTADORS.centraleta%TYPE,
	t_id_comptador IN CENTRALETES_COMPTADORS.comptador%TYPE
	)
	RETURN BOOLEAN;

/* ------------------------------------- Comprova que la relaci centraleta-comptador est activa
Retorna TRUE si est actiu
Retorna FALSE si est innactiu	
*/
FUNCTION fn_centr_comp_actiu
	(t_id_centraleta IN CENTRALETES_COMPTADORS.centraleta%TYPE,
	t_id_comptador IN CENTRALETES_COMPTADORS.comptador%TYPE
	)
	RETURN BOOLEAN;

/* -------------------------------------------- Cerca la central per centraleta
Entra l'adrea i el cp de la centraleta
Cerca a linies el id_central
Retorna el id_central
*/

FUNCTION fn_cerca_central_x_centraleta
	(t_adreca IN CENTRALETES.adreca%TYPE,
	t_CP IN CENTRALETES.cp%TYPE)
	RETURN NUMBER;


/* --------------------------------------------------------------------------------------- PROCEDIMENTS 
   ------------------------------------------------------------------------- TRACTAMENT DE DIETARI

   -----------------------------------------------------------------------------------------------------


   --------------------------------------------------- Alta de log_aplicacio
*/

PROCEDURE pr_alta_log_aplicacio
	(t_procediment IN LOG_APLICACIO.procediment%TYPE,
	 t_entrada IN LOG_APLICACIO.entrada%TYPE,
	 t_sortida IN LOG_APLICACIO.sortida%TYPE
	);

/* -------------------------------------------------------------------------------- TRACTAMENT DE PAISOS

--------------------------------------------------------------------------------------------------------
*/


/* --------------------------------------------------- Alta de pais
Si existeix ens dona un error -20010
Si no existeix el dona d'alta
*/

PROCEDURE pr_alta_pais
	(t_id_pais IN PAISOS.id_pais%TYPE,
	t_nom IN PAISOS.nom%TYPE,
	t_poblacio IN PAISOS.poblacio%TYPE,
	t_extensio IN PAISOS.extensio%TYPE
	);

/* --------------------------------------------------- Modificaci de pais
Si no existeix ens dona un error -20061
Si existeix fa la modificaci
*/

PROCEDURE pr_modifica_pais
	(t_id_pais IN PAISOS.id_pais%TYPE,
	t_nom IN PAISOS.nom%TYPE,
	t_poblacio IN PAISOS.poblacio%TYPE,
	t_extensio IN PAISOS.extensio%TYPE
	);

/* --------------------------------------------------- Baixa de pais
Si no existeix ens dona un error -20061
Si t dades relacionades a CENTRALS_PRODUCCIO dona l'error -20062
Si t dades relacionades a CLIENTS dona l'error -20063
Si existeix i no t relacions dona de baixa
*/

PROCEDURE pr_baixa_pais
	(t_id_pais IN PAISOS.id_pais%TYPE
	);

/* ----------------------------------------------------------------------- TRACTAMENT DE TIPUS de CENTRAL

--------------------------------------------------------------------------------------------------------
*/

/* --------------------------------------------------- Alta de tipus de central
Si existeix ens dona un error -20070
Si no existeix el dona d'alta
*/

PROCEDURE pr_alta_tipus_centrals
	(t_id_tipus_central IN TIPUS_CENTRALS.id_tipus%TYPE,
	t_descripcio IN TIPUS_CENTRALS.descripcio%TYPE
	);

/* --------------------------------------- Modificaci de tipus de centrals de produccio
Si no existeix ens dona un error -20071
Si existeix fa la modificaci
*/

PROCEDURE pr_modifica_tipus_centrals
	(t_id_tipus_central IN TIPUS_CENTRALS.id_tipus%TYPE,
	t_descripcio IN TIPUS_CENTRALS.descripcio%TYPE
	);

/* --------------------------------------------------- Baixa de tipus de central de producci
Si no existeix ens dona un error -20071
Si t dades relacionades a CENTRALS_PRODUCCIO dona l'error -20072
Si existeix i no t relacions dona de baixa
*/

PROCEDURE pr_baixa_tipus_centrals
	(t_id_tipus_central IN TIPUS_CENTRALS.id_tipus%TYPE
	);

/* ----------------------------------------------------------------------- TRACTAMENT DE MODEL COMPTADOR

--------------------------------------------------------------------------------------------------------
*/

/* ------------------------------------------------------------------- Alta de models de comptador
Si existeix ens dona un error -20080
Si no existeix el dona d'alta
*/

PROCEDURE pr_alta_models_comptador
	(t_id_model IN MODELS_COMPTADOR.id_model%TYPE,
	t_fabricant IN MODELS_COMPTADOR.fabricant%TYPE,
	t_descripcio IN MODELS_COMPTADOR.descripcio%TYPE
	);

/* ------------------------------------------------------------------- Modificaci de models de comptador
Si no existeix ens dona un error -20081
Si existeix fa la modificaci
*/

PROCEDURE pr_modifica_models_comptador
	(t_id_model IN MODELS_COMPTADOR.id_model%TYPE,
	t_fabricant IN MODELS_COMPTADOR.fabricant%TYPE,
	t_descripcio IN MODELS_COMPTADOR.descripcio%TYPE
	);

/* --------------------------------------------------------------------------- Baixa de models de comptador
Si no existeix ens dona un error -20081
Si t dades relacionades a COMPTADORS dona l'error -20082
Si existeix i no t relacions dona de baixa
*/

PROCEDURE pr_baixa_models_comptador
	(t_id_model IN MODELS_COMPTADOR.id_model%TYPE
	);

/* ------------------------------------------------------------------------------- TRACTAMENT DE CLIENTS

--------------------------------------------------------------------------------------------------------
*/

/* --------------------------------------------------- Alta de client
Si existeix i est actiu ens dona un error -20070
Si el pas no existeix ens dona un error -20072
Si existeix, per no est actiu l'actualitza
Si no existeix el dona d'alta
*/

PROCEDURE pr_alta_client
	(t_DNI_NIF IN CLIENTS.DNI_NIF%TYPE,
	t_nom_rao_social IN CLIENTS.nom_rao_social%TYPE,
	t_adreca IN CLIENTS.adreca%TYPE,
	t_cp IN CLIENTS.cp%TYPE,
	t_localitat IN CLIENTS.localitat%TYPE,
	t_pais IN CLIENTS.pais%TYPE,
	t_telefon IN CLIENTS.telefon%TYPE,
	t_e_mail IN CLIENTS.e_mail%TYPE,
	t_companyia IN CLIENTS.companyia%TYPE
	);

/* --------------------------------------------------- Modificaci de client
Si no existeix ens dona un error -20074
Si el pas no existeix ens dona un error -20072
Si existeix i est actiu l'actualitza
*/
PROCEDURE pr_modifica_client
	(t_DNI_NIF IN CLIENTS.DNI_NIF%TYPE,
	t_nom_rao_social IN CLIENTS.nom_rao_social%TYPE,
	t_adreca IN CLIENTS.adreca%TYPE,
	t_cp IN CLIENTS.cp%TYPE,
	t_localitat IN CLIENTS.localitat%TYPE,
	t_pais IN CLIENTS.pais%TYPE,
	t_telefon IN CLIENTS.telefon%TYPE,
	t_e_mail IN CLIENTS.e_mail%TYPE,
	t_companyia IN CLIENTS.companyia%TYPE
	);

/* --------------------------- Procediment que dona de baixa lgica a la relaci centraletes_comptadors

*/
PROCEDURE pr_baixa_rel_centraletes_comp
	(t_id_comptador IN CENTRALETES_COMPTADORS.comptador%TYPE
	);

/* --------------------------- Procediment que dona de baixa lgica a la relaci centraletes_comptadors per centraleta

*/
PROCEDURE pr_baixa_centr_comp_x_centr
	(t_id_centraleta IN CENTRALETES_COMPTADORS.centraleta%TYPE
	);

/* --------------------------- Procediment que dona de baixa lgica tots els comptadors d'un client
	Abans de de donar de baixa el comptador dona de baixa la relaci centraletes_comptadors
*/
PROCEDURE pr_baixa_comptadors_client
	(t_id_client IN COMPTADORS.client%TYPE
	);

/* --------------------------------------------------- baixa de client
Si no existeix ens dona un error -20074
Si existeix i est actiu es dona de baixa lgica, 
	dona de baixa lgica els comptadors del client i la relaci comptadors_centraletes 
*/

PROCEDURE pr_baixa_client
	(t_DNI_NIF IN CLIENTS.DNI_NIF%TYPE	
	);

/* ---------------------------------------------------------------------------- TRACTAMENT DE COMPTADORS

--------------------------------------------------------------------------------------------------------
*/

/* --------------------------------------------------- Alta de comptador
Si existeix i est actiu ens dona un error -20080
Si existeix, per no est actiu l'actualitza
Si el client no existeix o est desactivat dona un error -20082
Si el model no existeix dona el missatge -20084
Si el pas no existeix dona el missatge -20086
Si no existeix el dona d'alta
*/

PROCEDURE pr_alta_comptador
	(t_numero_contracte IN COMPTADORS.numero_contracte%TYPE,
	t_DNI_NIF IN CLIENTS.DNI_NIF%TYPE,
	t_pot_contractada IN COMPTADORS.pot_contractada%TYPE,
	t_adreca IN COMPTADORS.adreca%TYPE,
	t_cp IN COMPTADORS.cp%TYPE,
	t_localitat IN COMPTADORS.localitat%TYPE,
	t_pais IN COMPTADORS.pais%TYPE,
	t_model IN COMPTADORS.model%TYPE,
	t_any_fabricacio IN COMPTADORS.any_fabricacio%TYPE,
	t_data_instalacio IN COMPTADORS.data_instalacio%TYPE,
	t_data_ultima_revisio IN COMPTADORS.data_ultima_revisio%TYPE,
	t_alta_disponibilitat IN COMPTADORS.alta_disponibilitat%TYPE
	);

/* --------------------------------------------------- Modificaci de comptador
Si no existeix o est inactiu ens dona un error -20088
Si el client no existeix o est inactiu ens dona l'error -20082
Si el nou model de comptador no existeix ens dona l'error -20084
Si el nou pas no existeix ens dona l'error -20086
Si existeix i est actiu l'actualitza
*/

PROCEDURE pr_modifica_comptador
	(t_numero_contracte IN COMPTADORS.numero_contracte%TYPE,
	t_DNI_NIF IN CLIENTS.DNI_NIF%TYPE,
	t_pot_contractada IN COMPTADORS.pot_contractada%TYPE,
	t_adreca IN COMPTADORS.adreca%TYPE,
	t_cp IN COMPTADORS.cp%TYPE,
	t_localitat IN COMPTADORS.localitat%TYPE,
	t_pais IN COMPTADORS.pais%TYPE,
	t_model IN COMPTADORS.model%TYPE,
	t_any_fabricacio IN COMPTADORS.any_fabricacio%TYPE,
	t_data_instalacio IN COMPTADORS.data_instalacio%TYPE,
	t_data_ultima_revisio IN COMPTADORS.data_ultima_revisio%TYPE,
	t_alta_disponibilitat IN COMPTADORS.alta_disponibilitat%TYPE
	);

/* --------------------------------------------------- Baixa de comptador
Si no existeix o est inactiu ens dona un error -20088
Si existeix i est actiu el dona de baixa lgica
*/

PROCEDURE pr_baixa_comptador
	(t_numero_contracte IN COMPTADORS.numero_contracte%TYPE
	);

/* --------------------------------------------------------------------------- TRACTAMENT DE CENTRALETES

--------------------------------------------------------------------------------------------------------
*/


/* --------------------------------------------------- Alta de centraleta
Si la lnia a la que es vol connectar la centraleta no existeig o est inactiva ens dona un error -20062
Si el pas no existeix ens dona l'error -20064
Si existeix i est activa ens dona un error -20060
Si existeix, per no est actiu l'actualitza
Si no existeix el dona d'alta
*/
PROCEDURE pr_alta_centraleta
	(t_linia IN CENTRALETES.linia%TYPE,
	t_adreca IN CENTRALETES.adreca%TYPE,
	t_cp IN CENTRALETES.cp%TYPE,
	t_localitat IN CENTRALETES.localitat%TYPE,
	t_pais IN CENTRALETES.pais%TYPE,
	t_capacitat_maxima IN CENTRALETES.capacitat_maxima%TYPE,
	t_alta_disponibilitat IN CENTRALETES.alta_disponibilitat%TYPE
	);

/* --------------------------------------------------- Modificacio de centraleta
Si la lnia a la que es vol connectar la centraleta no existeig o est inactiva ens dona un error -20062
Si el pas no existeix ens dona l'error -20064
Si no existeix o est inactiva ens dona un error -20066
Si existeix el dona l'actualitza
*/

PROCEDURE pr_modifica_centraleta
	(t_linia IN CENTRALETES.linia%TYPE,
	t_adreca IN CENTRALETES.adreca%TYPE,
	t_cp IN CENTRALETES.cp%TYPE,
	t_localitat IN CENTRALETES.localitat%TYPE,
	t_pais IN CENTRALETES.pais%TYPE,
	t_capacitat_maxima IN CENTRALETES.capacitat_maxima%TYPE,
	t_alta_disponibilitat IN CENTRALETES.alta_disponibilitat%TYPE
	);

/* --------------------------------------------------- Baixa de centraleta
Si no existeix o est inactiva ens dona un error -20066
Si existeix i est activa es dona de baixa lgica
*/

PROCEDURE pr_baixa_centraleta
	(t_adreca IN CENTRALETES.adreca%TYPE,
	t_cp IN CENTRALETES.cp%TYPE
	);

/* -------------------------------------------------------------------------------- TRACTAMENT DE LINIES

--------------------------------------------------------------------------------------------------------
*/


/* --------------------------------------------------- Alta de linia
L'error 20050 no el donar mai ja que el id_linia es autmatic per el 
	que en el cas d'alta no cal mirar si existeix o est activa,
	tampoc es podr modificar, noms donar d'alta.
-- Si existeix i est activa ens dona un error -20050

Si la central de producci no existex o est inactiva ens dona l'error -20052
Si existeix, per no est actiu l'actualitza
Si no existeix el dona d'alta
*/

PROCEDURE pr_alta_linia
	(t_id_central IN LINIES.central%TYPE,
	t_capacitat IN LINIES.capacitat%TYPE,
	t_alta_disponibilitat IN LINIES.alta_disponibilitat%TYPE
	);

/* --------------------------------------------------- Modificaci de lnies
Si no existeix o est inactiva ens dona un error -20054
Si la central de producci no existex o est inactiva ens dona l'error -20052
Si existeix i est actiu l'actualitza
*/

PROCEDURE pr_modifica_linia
	(t_id_linia IN LINIES.id%TYPE,
	t_id_central IN LINIES.central%TYPE,
	t_capacitat IN LINIES.capacitat%TYPE,
	t_alta_disponibilitat IN LINIES.alta_disponibilitat%TYPE
	);

/* -------------- Donar de baixa els comptadors que noms s'alimentten d'aquesta centraleta 
	si estan connectats a un altre centraleta no donar-los de baixa
	Mirar si el comptador est connectat a d'altres centraletes
	Si no ho est donar de baixa el comptador
		si ho est no es dona de baixa el comptador
*/

PROCEDURE pr_baixa_comp_sense_centr
	(t_id_centraleta IN CENTRALETES_COMPTADORS.centraleta%TYPE,
	t_id_comptador IN CENTRALETES_COMPTADORS.comptador%TYPE
	);

/* -------------- Dona de baixa a la relaci centraleta_comptador tots el registres que tinguin aquesta centraleta
	MIrar si exiteix i donar de baixa
	Mirar si el comptador est connectat a d'altres centraletes
		Si no ho est donar de baixa el comptador
		si ho est no es dona de baixa el comptador
*/

PROCEDURE pr_baixa_centr_comp_x_linies
	(t_id_centraleta IN CENTRALETES_COMPTADORS.centraleta%TYPE
	);

/* ------------------------------------ Dona de baixa lgica les centraletes que estan connectades a aquesta lnia
		Una centraleta noms pot tenir una lnia, per una lnia pot tenir moltes centraletes
*/

PROCEDURE pr_baixa_centraletes_linia
	(t_id_linia IN LINIES.id%TYPE
	);

/* --------------------------------------------------- Baixa de lnies
Si no existeix o es inactiva ens dona un error -20054
Si existeix i est activa es dona de baixa lgica
*/

PROCEDURE pr_baixa_linia
	(t_id_linia IN LINIES.id%TYPE
	);

/* -------------------------------------------------------------- ASSIGNACI DE CENTRALETES - COMPTADORS 

   -----------------------------------------------------------------------------------------------------

/* ----------------------------------------------------- Alta de relaci centraleta_comptador
Si la centraleta no existeix o est inactiva dona l'error -20092
Si el comptador no existeix o est inactiva dona l'error -20094
Si la relaci centraleta_coptador ja existeix i est activa dona l'error -20090
Si la relaci existeix per est inactiva, l'activa
Si la relaci no existeix la dona d'alta.
*/

PROCEDURE pr_alta_centraleta_comptador
	(t_adreca IN CENTRALETES.adreca%type,
        t_cp IN CENTRALETES.cp%TYPE,
        t_numero_contracte IN COMPTADORS.numero_contracte%TYPE
        );

/* ----------------------------------------------------- Baixa de relaci centraleta_comptador
Si la centraleta no existeix o est inactiva dona l'error -20092
Si el comptador no existeix o est inactiva dona l'error -20094
Si la relaci centraleta_coptador no existeix o est inactiva dona l'error -20096
Si la relaci existeix i est activa la dona de baixa lgica.
*/

PROCEDURE pr_baixa_centraleta_comptador
	(t_adreca IN CENTRALETES.adreca%type,
        t_cp IN CENTRALETES.cp%TYPE,
        t_numero_contracte IN COMPTADORS.numero_contracte%TYPE
        )
 ;

/* ----------------------------------------------------------------- TRACTAMENT DE CENTRALS DE PRODUCCIO

--------------------------------------------------------------------------------------------------------
*/

/* --------------------------------------------------- Alta de central de producci
Si existeix i est activa ens dona l'error -20040
Si el tipus de central no existeix ens dona l'error 20042
Si el pas no existeix ens dona l'error -20044
Si no existeix el dona d'alta
*/

PROCEDURE pr_alta_central
	(t_nom IN CENTRALS_PRODUCCIO.nom%TYPE,
 	t_tipus IN CENTRALS_PRODUCCIO.tipus%TYPE,
	t_adreca IN CENTRALS_PRODUCCIO.adreca%TYPE,
	t_cp IN CENTRALS_PRODUCCIO.cp%TYPE,
	t_localitat IN CENTRALS_PRODUCCIO.localitat%TYPE,
	t_pais IN CENTRALS_PRODUCCIO.pais%TYPE,
	t_telefon IN CENTRALS_PRODUCCIO.telefon%TYPE,
	t_e_mail IN CENTRALS_PRODUCCIO.e_mail%TYPE,
	t_energia_maxima IN CENTRALS_PRODUCCIO.energia_maxima%TYPE,
	t_data_ult_revisio IN CENTRALS_PRODUCCIO.data_ult_revisio%TYPE,
	t_energia_minima_NU IN CENTRALS_PRODUCCIO.energia_minima_NU%TYPE,
	t_rebuig_radioactiu IN CENTRALS_PRODUCCIO.rebuig_radioactiu%TYPE,
	t_kg_emisio_CO2 IN CENTRALS_PRODUCCIO.kg_emisio_CO2%TYPE,
 	t_unitats_produccio IN CENTRALS_PRODUCCIO.unitats_produccio%TYPE,
	t_alta_disponibilitat IN CENTRALS_PRODUCCIO.alta_disponibilitat%TYPE
	);

/* --------------------------------------------------- Modificaci de centrals de producci
Si el tipus de central no existeix ens dona un error -20042
Si el pas no existeix ens dona un error -20044
Si la central de producci no existex o est inactiva ens dona l'error -20046
Si existeix i est activa l'actualitza
*/

PROCEDURE pr_modifica_centrals
	(t_nom IN CENTRALS_PRODUCCIO.nom%TYPE,
 	t_tipus IN CENTRALS_PRODUCCIO.tipus%TYPE,
	t_adreca IN CENTRALS_PRODUCCIO.adreca%TYPE,
	t_cp IN CENTRALS_PRODUCCIO.cp%TYPE,
	t_localitat IN CENTRALS_PRODUCCIO.localitat%TYPE,
	t_pais IN CENTRALS_PRODUCCIO.pais%TYPE,
	t_telefon IN CENTRALS_PRODUCCIO.telefon%TYPE,
	t_e_mail IN CENTRALS_PRODUCCIO.e_mail%TYPE,
	t_energia_maxima IN CENTRALS_PRODUCCIO.energia_maxima%TYPE,
	t_data_ult_revisio IN CENTRALS_PRODUCCIO.data_ult_revisio%TYPE,
	t_energia_minima_NU IN CENTRALS_PRODUCCIO.energia_minima_NU%TYPE,
	t_rebuig_radioactiu IN CENTRALS_PRODUCCIO.rebuig_radioactiu%TYPE,
	t_kg_emisio_CO2 IN CENTRALS_PRODUCCIO.kg_emisio_CO2%TYPE,
 	t_unitats_produccio IN CENTRALS_PRODUCCIO.unitats_produccio%TYPE,
	t_alta_disponibilitat IN CENTRALS_PRODUCCIO.alta_disponibilitat%TYPE
	);

/* --------------------- Dona de baixa lgica les lnies que estan connectades a aquesta central de producci
		Una lnia noms pot estar connectada a una central, per una central pot tenir moltes lnies
*/

PROCEDURE pr_baixa_linia_central
	(t_id_central IN LINIES.central%TYPE
	);

/* --------------------------------------------------- Baixa de centrals de producci
Si no existeix o est inactiva ens dona un error -20046
Si existeix i est actiu el dona de baixa lgica
*/

PROCEDURE pr_baixa_central
	(t_nom IN CENTRALS_PRODUCCIO.nom%TYPE
	);

END;
/

disconnect;