connect Administrador/a12345

CREATE OR REPLACE
PACKAGE BODY PR_CON_PLUS AS


/* Donat un DNI o NIF de client i un any concret el concum per mesos d'aquest client i any
*/

PROCEDURE pr_con_client_any
	(t_DNI_NIF IN CLIENTS.DNI_NIF%TYPE,
	t_any IN NUMBER)

IS
        v_exist_client BOOLEAN;


        linia_cap CHAR(40) := '---------------------------------------';
        l_mes NUMBER(2);
        l_consum NUMBER(9,2);


        CURSOR c_consulta IS
                SELECT hlc.mes, SUM(hlc.consum)
                    FROM (CLIENTS cc INNER JOIN COMPTADORS co
				ON cc.id = co.client and DNI_NIF=t_DNI_NIF)
          		INNER JOIN HISTORIC_LECTURES_COMPTADORS hlc
				ON co.id = hlc.comptador and hlc.anyhis=t_any
                    GROUP BY hlc.mes
                    ORDER BY hlc.mes;

        procediment LOG_APLICACIO.procediment%TYPE := 'pr_con_client_any';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';


         DNI_NIF_not_exist EXCEPTION;
         entrada_any_is_null EXCEPTION;
         entrada_DNI_NIF_is_null EXCEPTION;
BEGIN

        entrada := '-'||t_DNI_NIF||'-'||t_any||'-';


        IF (t_DNI_NIF IS NULL)
                THEN RAISE entrada_DNI_NIF_is_null;
        END IF;

       IF (t_any IS NULL)
                THEN RAISE entrada_any_is_null;
        END IF;

        v_exist_client := general.fn_client_exist_x_DNI_NIF(t_DNI_NIF);

	IF (NOT v_exist_client)
		THEN RAISE DNI_NIF_not_exist;
        END IF;


    DBMS_OUTPUT.PUT_LINE ('-----------------------------------------------------------------------------------------------');
    DBMS_OUTPUT.PUT_LINE ('Consum del client :'||t_DNI_NIF||'  a l''any :'||t_any);
    DBMS_OUTPUT.PUT_LINE ('-----------------------------------------------------------------------------------------------');


      DBMS_OUTPUT.PUT_LINE (rpad('Mes',10)
                            ||' '||rpad('Consum',20));

      DBMS_OUTPUT.PUT_LINE (rpad(linia_cap,10)
                            ||' '||rpad(linia_cap,20));

        OPEN c_consulta;
        LOOP
            FETCH c_consulta
            INTO l_mes, l_consum;
            EXIT WHEN c_consulta%NOTFOUND;

	    DBMS_OUTPUT.PUT_LINE (
            rpad(TO_CHAR(TO_DATE(TO_CHAR(l_mes),'mm'),'Month','NLS_DATE_LANGUAGE=CATALAN'),10)
                            ||' '||lpad(TO_CHAR(l_consum,'999G999D99'),20));
        END LOOP;
        CLOSE c_consulta;
	GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);

EXCEPTION
        WHEN DNI_NIF_not_exist THEN
            ROLLBACK;
            sortida := '-20110 - El client '||t_DNI_NIF||' no existeix.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

         WHEN entrada_DNI_NIF_is_null THEN
            ROLLBACK;
            sortida := '-20111 - El client entrat es un valor nul.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

        WHEN entrada_any_is_null THEN
            ROLLBACK;
            sortida := '-20112 - Any entrat es un valor nul.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

        WHEN OTHERS THEN
            ROLLBACK;
            sortida := SQLERRM||' En la consulta de consums de client per mesos';
            		GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

END;


/* Relaci del 10 mesos de ms consum dels ltims 5 anys
*/

PROCEDURE pr_con_consum_5_any

IS

        linia_cap CHAR(40) := '---------------------------------------';
        l_any NUMBER(4);
        l_mes NUMBER(2);
        l_consum NUMBER(9,2);


        CURSOR c_consulta IS
               SELECT anyhis, mes, total_mes
                      FROM (SELECT anyhis, mes, sum(consum) as total_mes
                                FROM (SELECT anyhis, mes, consum 
                                          FROM HISTORIC_LECTURES_COMPTADORS
                                          WHERE anyhis >= extract(year from SYSDATE)-5)
                                GROUP BY anyhis, mes
                                ORDER BY total_mes DESC)
                      WHERE rownum <= 10;

        procediment LOG_APLICACIO.procediment%TYPE := 'pr_con_consum_5_any';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

BEGIN

        entrada := ' ';

    DBMS_OUTPUT.PUT_LINE ('-----------------------------------------------------------------------------------------------');
    DBMS_OUTPUT.PUT_LINE ('Mesos de ms consum dels ltims 5 anys');
    DBMS_OUTPUT.PUT_LINE ('-----------------------------------------------------------------------------------------------');


      DBMS_OUTPUT.PUT_LINE (rpad('Any',5)
                            ||' ' ||rpad('Mes',10)
                            ||' '||rpad('Consum',20));

      DBMS_OUTPUT.PUT_LINE (rpad(linia_cap,5)
                            ||' '||rpad(linia_cap,10)
                            ||' '||rpad(linia_cap,20));

        OPEN c_consulta;
        LOOP
            FETCH c_consulta
            INTO l_any, l_mes, l_consum;
            EXIT WHEN c_consulta%NOTFOUND;

	    DBMS_OUTPUT.PUT_LINE (lpad(TO_CHAR(l_any),5)
                            ||' '||rpad(TO_CHAR(TO_DATE(TO_CHAR(l_mes),'mm'),'Month','NLS_DATE_LANGUAGE=CATALAN'),10)
                            ||' '||lpad(TO_CHAR(l_consum,'999G999D99'),20));
        END LOOP;
        CLOSE c_consulta;
	GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);

EXCEPTION
        WHEN OTHERS THEN
            ROLLBACK;
            sortida := SQLERRM||' En la consulta de mesos de ms consum';
            		GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

END;


/* Crrega mensual donada una lnia i un any
*/

PROCEDURE pr_con_carrega_linies_any
	( t_linia IN HISTORIC_CARREGA_LINIA_MES.linia%TYPE,
        t_any IN HISTORIC_CARREGA_LINIA_MES.anyhis%TYPE)

IS
 	v_exist_linia BOOLEAN;

        linia_cap CHAR(40) := '---------------------------------------';
        l_linia NUMBER(10);
        l_mes NUMBER(2);
        l_carrega NUMBER(9,2);


        CURSOR c_consulta IS
                SELECT linia, mes, carrega 
                       FROM HISTORIC_CARREGA_LINIA_MES
                       WHERE linia= t_linia AND anyhis = t_any
                ORDER BY linia, mes;
                
                
        procediment LOG_APLICACIO.procediment%TYPE := 'pr_con_carrega_linies_any';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	entrada_linia_is_null EXCEPTION;
	linia_not_exist EXCEPTION;
        entrada_any_is_null EXCEPTION;
BEGIN

        entrada := '-'||t_linia||'-'||t_any||'-';


       IF (t_linia IS NULL)
                THEN RAISE entrada_linia_is_null;
        END IF;


       IF (t_any IS NULL)
                THEN RAISE entrada_any_is_null;
        END IF;

        v_exist_linia := general.fn_id_exist_dinamic('LINIES',t_linia);

	IF (NOT v_exist_linia)
		THEN RAISE linia_not_exist;
        END IF;
 
    DBMS_OUTPUT.PUT_LINE ('-----------------------------------------------------------------------------------------------');
    DBMS_OUTPUT.PUT_LINE ('Crrega de la lnia :'||t_linia ||' a  l''any :'||t_any);
    DBMS_OUTPUT.PUT_LINE ('-----------------------------------------------------------------------------------------------');


      DBMS_OUTPUT.PUT_LINE (rpad('Mes',10)
                            ||' '||rpad('Crrega',20));

      DBMS_OUTPUT.PUT_LINE (rpad(linia_cap,10)
                            ||' '||rpad(linia_cap,20));

        OPEN c_consulta;
        LOOP
            FETCH c_consulta
            INTO l_linia, l_mes, l_carrega;
            EXIT WHEN c_consulta%NOTFOUND;

	    DBMS_OUTPUT.PUT_LINE (rpad(TO_CHAR(TO_DATE(TO_CHAR(l_mes),'mm'),'Month','NLS_DATE_LANGUAGE=CATALAN'),10)
                            ||' '||lpad(TO_CHAR(l_carrega,'999G999D99'),20));
        END LOOP;
        CLOSE c_consulta;
	GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);

EXCEPTION

        WHEN linia_not_exist THEN
            ROLLBACK;
            sortida := '-20115 - La lnia '||t_linia||' no existeix.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

         WHEN entrada_linia_is_null THEN
            ROLLBACK;
            sortida := '-20116 - la lnia entrada es un valor nul.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

        WHEN entrada_any_is_null THEN
            ROLLBACK;
            sortida := '-20112 - Any entrat es un valor nul.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

        WHEN OTHERS THEN
            ROLLBACK;
            sortida := SQLERRM||' En la consulta de crregues de linies per any';
            		GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

END;


/* Producci mensual donada una central i un any
*/
PROCEDURE pr_con_produccio_central_any
	( t_central IN CENTRALS_PRODUCCIO.nom%TYPE,
        t_any IN HISTORIC_PRODUIDA_MES.anyhis%TYPE)

IS

        linia_cap CHAR(40) := '---------------------------------------';
        l_central VARCHAR(20);
        l_mes NUMBER(2);
        l_potencia NUMBER(9,2);
        v_exist BOOLEAN;

        CURSOR c_consulta IS
                SELECT central, mes, potencia
                       FROM CENTRALS_PRODUCCIO, HISTORIC_PRODUIDA_MES
                       WHERE nom = t_central AND id=central AND anyhis = t_any
                ORDER BY central, nom, mes;


        procediment LOG_APLICACIO.procediment%TYPE := 'pr_con_produccio_central_any';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

        entrada_central_is_null EXCEPTION;
        central_not_exist EXCEPTION;
        entrada_any_is_null EXCEPTION;
BEGIN

        entrada := '-'||t_central||'-'||t_any||'-';

       IF (t_central IS NULL)
                THEN RAISE entrada_central_is_null;
        END IF;

       IF (t_any IS NULL)
                THEN RAISE entrada_any_is_null;
        END IF;

      v_exist := general.fn_central_exist_x_nom(t_central);
      
      IF (NOT v_exist)
          THEN RAISE central_not_exist;
      END IF;

    DBMS_OUTPUT.PUT_LINE ('-----------------------------------------------------------------------------------------------');
    DBMS_OUTPUT.PUT_LINE ('Producci de la central :'||t_central ||' a  l''any :'||t_any);
    DBMS_OUTPUT.PUT_LINE ('-----------------------------------------------------------------------------------------------');


      DBMS_OUTPUT.PUT_LINE (rpad('Mes',10)
                            ||' '||rpad('Producci',20));

      DBMS_OUTPUT.PUT_LINE (rpad(linia_cap,10)
                            ||' '||rpad(linia_cap,20));

        OPEN c_consulta;
        LOOP
            FETCH c_consulta
            INTO l_central, l_mes, l_potencia;
            EXIT WHEN c_consulta%NOTFOUND;

	    DBMS_OUTPUT.PUT_LINE (rpad(TO_CHAR(TO_DATE(TO_CHAR(l_mes),'mm'),'Month','NLS_DATE_LANGUAGE=CATALAN'),10)
                            ||' '||lpad(TO_CHAR(l_potencia,'999G999D99'),20));
        END LOOP;
        CLOSE c_consulta;
	GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);

EXCEPTION
        WHEN entrada_central_is_null THEN
            ROLLBACK;
            sortida := '-20113 - Central entrada es un valor nul.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

        WHEN central_not_exist THEN
            ROLLBACK;
            sortida := '-20114 - La central '||t_central||' no existeix.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

        WHEN entrada_any_is_null THEN
            ROLLBACK;
            sortida := '-20112 - Any entrat es un valor nul.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

        WHEN OTHERS THEN
            ROLLBACK;
            sortida := SQLERRM||' En la consulta de producci de centrals per any';
            		GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

END;


/* Donat un any quota de mercat de les companyies distribuidores
*/
PROCEDURE pr_quota_mercat_companyia
          (t_any IN HISTORIC_PRODUIDA_MES.anyhis%TYPE)

IS

        linia_cap CHAR(40) := '---------------------------------------';
        total_consum NUMBER(12,2);
        l_companyia VARCHAR(30);
        l_lectures NUMBER;
        l_quota NUMBER(9,2);
        t_total_consum NUMBER(12,2);

  CURSOR c_total_consum IS
        SELECT SUM(consum)
            FROM LECTURES_COMPTADORS 
            WHERE extract(year from data_lectura) = t_any;

        CURSOR c_consulta IS
                SELECT companyia, COUNT(companyia), SUM(consum) 
                       FROM LECTURES_COMPTADORS
                       WHERE extract(year from data_lectura) = t_any
                GROUP BY companyia
                ORDER BY companyia;

        procediment LOG_APLICACIO.procediment%TYPE := 'pr_quota_mercat_companyia';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

         entrada_any_is_null EXCEPTION;
BEGIN

        entrada := '-';

       IF (t_any IS NULL)
                THEN RAISE entrada_any_is_null;
        END IF;

        OPEN c_total_consum;
           FETCH c_total_consum
           INTO t_total_consum;
        CLOSE c_total_consum;


    DBMS_OUTPUT.PUT_LINE ('-----------------------------------------------------------------------------------------------');
    DBMS_OUTPUT.PUT_LINE ('Quotes de mercat per companyies distribuidores per l''any :'||t_any);
    DBMS_OUTPUT.PUT_LINE ( 'De un consum total de :'||lpad(TO_CHAR(t_total_consum,'999G999G999G999D99'),20));
    DBMS_OUTPUT.PUT_LINE ('-----------------------------------------------------------------------------------------------');


      DBMS_OUTPUT.PUT_LINE (rpad('Companyia',20)
                            ||' '||rpad('Lectures',12)
                            ||' '||rpad('Consum',20)
                            ||' '||rpad('Quota',10));

      DBMS_OUTPUT.PUT_LINE (rpad(linia_cap,20)
                            ||' '||rpad(linia_cap,12)
                            ||' '||rpad(linia_cap,20)
                            ||' '||rpad(linia_cap,10));

        OPEN c_consulta;
        LOOP
            FETCH c_consulta
            INTO l_companyia, l_lectures, l_quota;
            EXIT WHEN c_consulta%NOTFOUND;

            DBMS_OUTPUT.PUT_LINE (rpad(TO_CHAR(l_companyia),20)
                            ||' '||lpad(TO_CHAR(l_lectures,'999G999G999'),12)
                            ||' '||lpad(TO_CHAR(l_quota,'999G999G999G999D99'),20)
                            ||' '||lpad(TO_CHAR((l_quota*100/t_total_consum),'999D99')||'%',10));
        END LOOP;
        CLOSE c_consulta;
	GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);

EXCEPTION
        WHEN entrada_any_is_null THEN
            ROLLBACK;
            sortida := '-20112 - Any entrat es un valor nul.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

        WHEN OTHERS THEN
            ROLLBACK;
            sortida := SQLERRM||' En la consulta de quota de mercat de companyies distribuidores';
            		GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

END;


END;
/
disconnect;