connect Administrador/a12345

CREATE OR REPLACE
PACKAGE BODY PR_CONSULTES AS


/* ------------------------------------- Funci de test de dates introduides per l'usuari
*/
/* Retorna false si la data no es correcte
 * Retorna true si la data es correcte.
 */

FUNCTION check_date (
        date_test varchar2
	)
	RETURN BOOLEAN
IS
	dateVar date;
	mascara VARCHAR2(20) := 'DD/MM/YYYY';
BEGIN
	dateVar:= to_date(date_test, mascara);
	return TRUE;
EXCEPTION
	when others then
	return FALSE;
END;


/* --------------------------------------------- Procediments de consulta
*/

/* Donada una ciutat i una data com a parmetres, el llistat de
	tots els comptadors on el consum mensual de la data
	indicada ha superat el 80% del consum mitj de tots els
	comptadors de la ciutat en aquell mateix perode de temps.
	En aquest llistat caldr retornar la informaci bsica segent:
		- Codi de contracte.
		- La potncia mxima contractada.
		- El tant per cent de consum elctric consumit en relaci
			al consum mitj
	Tot aix ordenat de forma ascendent pel tant per cent de
	consum elctric consumit.
*/

PROCEDURE pr_con_ciutat_data_80
	(t_localitat IN COMPTADORS.localitat%TYPE,
	t_data IN VARCHAR2)
IS
        v_exist_localitat BOOLEAN;
        consum_mitja_comptador NUMBER;
        v_date DATE;
        mascara VARCHAR2(20) := 'DD/MM/YYYY';
        t_mes NUMBER;
        t_any NUMBER;
        linia_cap CHAR(40) := '---------------------------------------';

        l_numero_contracte COMPTADORS.numero_contracte%TYPE;
        l_pot_contractada COMPTADORS.pot_contractada%TYPE;
        l_consum_mitja NUMBER(9,2);
        t_mitja_total NUMBER(9,2);

        CURSOR c_mitja_total IS
                SELECT AVG(consum)
                    FROM COMPTADORS , HISTORIC_LECTURES_COMPTADORS
                        WHERE localitat = t_localitat
                              AND mes = t_mes
	                      AND anyhis = t_any;

        procediment LOG_APLICACIO.procediment%TYPE := 'pr_con_ciutat_data_80';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

        CURSOR c_consulta IS
            SELECT co.numero_contracte, co.pot_contractada,
                    AVG(hlc.consum) AS consum_mitja
                  FROM COMPTADORS co INNER JOIN HISTORIC_LECTURES_COMPTADORS hlc
                        ON (co.id = hlc.comptador)
                  WHERE co.localitat = t_localitat
        		AND hlc.mes = t_mes
                	AND hlc.anyhis = t_any
	           GROUP BY co.id, co.numero_contracte, co.pot_contractada
                   ORDER BY AVG(hlc.consum);

         localitat_without_comptador EXCEPTION;
         entrada_localitat_is_null EXCEPTION;
         entrada_data_is_null EXCEPTION;
         invalit_date EXCEPTION;
BEGIN

        entrada := '-'||t_localitat||'-'||t_data||'-';

        IF check_date(t_data)
            THEN v_date := to_date(t_data, mascara);
                  t_mes := extract(month from v_date);
                  t_any := extract(year from v_date);
            ELSE RAISE invalit_date;
        END IF;

        IF (t_localitat IS NULL)
                THEN RAISE entrada_localitat_is_null;
        END IF;

       IF (t_data IS NULL)
                THEN RAISE entrada_data_is_null;
        END IF;

        v_exist_localitat := pr_entrades.fn_exist_localitat(t_localitat);
	IF (NOT v_exist_localitat)
		THEN RAISE localitat_without_comptador;
        END IF;

        OPEN c_mitja_total;
           FETCH c_mitja_total
           INTO t_mitja_total;
        CLOSE c_mitja_total;

    DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');
    DBMS_OUTPUT.PUT_LINE ('Comptadors que superen el 80% del consum mitja dels comptadors de la ciutat en el mes de la data');
    DBMS_OUTPUT.PUT_LINE ('Ciutat :'||t_localitat||'  Data :'||v_date||'  80% del consum mitja de tots els comptadors :'||(t_mitja_total*0.8));
    DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');

      DBMS_OUTPUT.PUT_LINE (rpad('Contracte',10)
                            ||' '||rpad('Potencia contractada',20)
                            ||' '||rpad('Consum mitj',12));

      DBMS_OUTPUT.PUT_LINE (rpad(linia_cap,10)
                            ||' '||rpad(linia_cap,20)
                            ||' '||rpad(linia_cap,12));

        OPEN c_consulta;
        LOOP
            FETCH c_consulta
            INTO l_numero_contracte, l_pot_contractada, l_consum_mitja;
            EXIT WHEN c_consulta%NOTFOUND;
            IF (l_consum_mitja > (t_mitja_total*0.8))
                THEN DBMS_OUTPUT.PUT_LINE (lpad(TO_CHAR(l_numero_contracte),10)
                            ||' '||lpad(TO_CHAR(l_pot_contractada,'999G999D99'),20)
                            ||' '||lpad(TO_CHAR(l_consum_mitja,'999G999D99'),12));
            END IF;
        END LOOP;
        CLOSE c_consulta;
	GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);

EXCEPTION
        WHEN localitat_without_comptador THEN
            ROLLBACK;
            sortida := '-20100 - La localitat '||t_localitat||' no t comptadors.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

         WHEN entrada_localitat_is_null THEN
            ROLLBACK;
            sortida := '-20101 - La localitat entrada es un valor nul.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

        WHEN entrada_data_is_null THEN
            ROLLBACK;
            sortida := '-20102 - La data entrada es un valor nul.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;
 
        WHEN invalit_date THEN
            ROLLBACK;
            sortida := SQLERRM||'- Error en la data '||t_data ||' entrada.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

        WHEN OTHERS THEN
            ROLLBACK;
            sortida := SQLERRM||' En la consulta de comptadors amb consum major del 80%';
            		GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

END;




/* Llistat de les 10 centraletes de distribuci que distribueixen
	ms energia, cal mostrar les dades segents:
		- Ladrea de la centraleta de distribuci.
		- Energia emesa per la centraleta de distribuci.
		- Mxima energia que permet la centraleta de distribuci.
	Ordenat de forma descendent per la energia emesa en valor absolut.
*/

PROCEDURE pr_con_top10_centraletes

IS
        l_adreca COMPTADORS.adreca%TYPE;
        l_cp COMPTADORS.cp%TYPE;
        l_energia_emesa NUMBER(9,2);
        l_capacitat_maxima CENTRALETES.capacitat_maxima%TYPE;
        linia_cap CHAR(40) := '---------------------------------------';

        procediment LOG_APLICACIO.procediment%TYPE := 'pr_con_top10_centraletes';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

        CURSOR c_consulta IS
		SELECT cc.adreca, cc.cp, SUM(lectura_acumulada), cc.capacitat_maxima
	        	FROM CENTRALETES_COMPTADORS ceco INNER JOIN CENTRALETES cc
				ON (ceco.centraleta = cc.id)
				INNER JOIN COMPTADORS co
				ON (ceco.comptador = co.id)
			WHERE ROWNUM <= 10
			GROUP BY cc.adreca, cc.cp, cc.capacitat_maxima
			ORDER BY SUM(lectura_acumulada) DESC;

 BEGIN
        entrada := ' ';

        DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');
        DBMS_OUTPUT.PUT_LINE ('Les 10 Centraletes de distribuci que distribueixen ms energia');
        DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');

      DBMS_OUTPUT.PUT_LINE (rpad('Adrea',25)
                            ||' '||rpad('CP',5)
                            ||' '||rpad('Energia Emesa',14)
			    ||' '||rpad('Energia permesa',16));

      DBMS_OUTPUT.PUT_LINE (rpad(linia_cap,25)
                            ||' '||rpad(linia_cap,5)
                            ||' '||rpad(linia_cap,14)
                            ||' '||rpad(linia_cap,16));


        OPEN c_consulta;
        LOOP
            FETCH c_consulta
            INTO l_adreca, l_cp, l_energia_emesa, l_capacitat_maxima;
            EXIT WHEN c_consulta%NOTFOUND;
		DBMS_OUTPUT.PUT_LINE (rpad(l_adreca,25)
                            ||' '||rpad((l_cp),5,'0')
                            ||' '||lpad(TO_CHAR(l_energia_emesa,'99G999G999D99'),14)
                            ||' '||lpad(TO_CHAR(l_capacitat_maxima,'99G999GD99'),16));
        END LOOP;
        CLOSE c_consulta;
	GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);

EXCEPTION
        WHEN OTHERS THEN
            	ROLLBACK;
            	sortida := SQLERRM||' En la consulta de comptadors amb consum major del 80%';
            		GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            	COMMIT;

END;


/* Llistat de les 10 lnies de comunicaci ms carregades en relaci 
	a la seva prpia capacitat mxima, cal mostrar les dades segents:
		- Codi didentificaci
		- Crrega de la lnia, entesa com la potncia que suporta
			la lnia entre la capacitat mxima de la lnia.
		- Energia a la que es pot ampliar la lnia tenint en
			compte les centrals de producci a les que est connectada.
	Ordenat de forma descendent per la crrega de la lnia en valor absolut.
*/

PROCEDURE pr_con_top10_linies

IS   
        l_id_linia LINIES.id%TYPE;
        l_carrega_linia LINIES.carrega_acumulada%TYPE;
        l_energia_ampliable NUMBER(9,2);
        linia_cap CHAR(40) := '---------------------------------------';
        
        procediment LOG_APLICACIO.procediment%TYPE := 'pr_con_top10_linies';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

        CURSOR c_consulta IS
		SELECT li.id, (li.carrega_acumulada/li.capacitat), (ce.energia_maxima - li.capacitat)
                      	FROM linies li INNER JOIN centrals_produccio ce
                            ON (li.central = ce.id)
			WHERE ROWNUM <= 10
                      	ORDER BY li.carrega_acumulada DESC;


 BEGIN
        entrada := ' ';

        DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');
        DBMS_OUTPUT.PUT_LINE ('Les 10 lnies de comunicaci ms carregades en relaci a la seva capacitat');
        DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');

      DBMS_OUTPUT.PUT_LINE (rpad('Lnia',12)
                            ||' '||rpad('Crrega suportada',17)
                            ||' '||rpad('Energia ampliable',17));

      DBMS_OUTPUT.PUT_LINE (rpad(linia_cap,12)
                            ||' '||rpad(linia_cap,17)
                            ||' '||rpad(linia_cap,17));


        OPEN c_consulta;
        LOOP
            FETCH c_consulta
            INTO l_id_linia, l_carrega_linia, l_energia_ampliable;
            EXIT WHEN c_consulta%NOTFOUND;
		DBMS_OUTPUT.PUT_LINE (lpad(TO_CHAR(l_id_linia),12)
                            ||' '||lpad(TO_CHAR(l_carrega_linia,'999G999G999D99'),17)
                            ||' '||lpad(TO_CHAR(l_energia_ampliable,'999G999G999D99'),17));
        END LOOP;
        CLOSE c_consulta;
	GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
    
EXCEPTION
        WHEN OTHERS THEN
            ROLLBACK;
            sortida := SQLERRM||' En la consulta de lnies ms carregades';
            		GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

END;


/* El llistat dels clients que disposen de comptadors amb servei
	en alta disponibilitat tant de centrals de distribuci, com de
	lnies de comunicaci i de centrals de producci. En concret
	cal mostrar les dades segents:
	- El DNI del client.
	- El codi de contracte.
	- El model del comptador.
*/

PROCEDURE pr_con_clients_alta_disp

IS   
        l_DNI_NIF CLIENTS.DNI_NIF%TYPE;
        l_numero_contracte COMPTADORS.numero_contracte%TYPE;
        l_model COMPTADORS.model%TYPE;
        linia_cap CHAR(40) := '---------------------------------------';
        
        procediment LOG_APLICACIO.procediment%TYPE := 'pr_con_clients_alta_disp';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

        CURSOR c_consulta IS
		SELECT cl.DNI_NIF, co.numero_contracte, co.model
                    FROM clients cl INNER JOIN comptadors co
                        ON (cl.id = co.client AND co.alta_disponibilitat='S')
                    INNER JOIN centraletes_comptadors cc
                        ON (co.id = cc.comptador)
                    INNER JOIN centraletes ce
                        ON (cc.centraleta = ce.id AND ce.alta_disponibilitat='S')
                    INNER JOIN linies li
                        ON (ce.linia = li.id AND li.alta_disponibilitat='S')
                    INNER JOIN centrals_produccio cp
                        ON (li.central = cp.id AND cp.alta_disponibilitat='S');

 BEGIN
        entrada := ' ';

        DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');
        DBMS_OUTPUT.PUT_LINE ('Els clients que poden disposar de tota la xarxa amb alta disponibilitat');
        DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');


      DBMS_OUTPUT.PUT_LINE (rpad('DNI_NIF',10)
                            ||' '||rpad('Nmero contracte',20)
                            ||' '||rpad('Model de comptador',12));

      DBMS_OUTPUT.PUT_LINE (rpad(linia_cap,10)
                            ||' '||rpad(linia_cap,20)
                            ||' '||rpad(linia_cap,12));


        OPEN c_consulta;
        LOOP
            FETCH c_consulta
            INTO l_DNI_NIF, l_numero_contracte, l_model;
            EXIT WHEN c_consulta%NOTFOUND;
		DBMS_OUTPUT.PUT_LINE (lpad(l_DNI_NIF,10)
                            ||' '||lpad(TO_CHAR(l_numero_contracte),20)
                            ||' '||rpad(TO_CHAR(l_model),12));
        END LOOP;
        CLOSE c_consulta;
	GENERAL.pr_alta_log_aplicacio(procediment, entrada, sortida);
    
EXCEPTION
        WHEN OTHERS THEN
            ROLLBACK;
            sortida := SQLERRM||' En la consulta de clients amb alta disponibilitat';
            		GENERAL.pr_alta_log_aplicacio(procediment, entrada, sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

END;


/* Donada una central de producci i un interval de temps,
	volem conixer el consum produt pels comptadors que
	depenen daquesta central i lenergia produda per la central
	en aquest mateix perode.
*/

PROCEDURE pr_con_centrals_produccio
	(t_nom IN CENTRALS_PRODUCCIO.nom%TYPE,
	t_data IN VARCHAR2
        )

IS

        v_exist_central BOOLEAN;
        consum_comptadors HISTORIC_LECTURES_COMPTADORS.consum%TYPE;
        produccio_central HISTORIC_PRODUIDA_MES.potencia%TYPE;
        v_date DATE;
        mascara VARCHAR2(20) := 'DD/MM/YYYY';
        t_mes NUMBER;
        t_any NUMBER;
        
        l_nom VARCHAR2(40);
        l_consum_comptadors HISTORIC_LECTURES_COMPTADORS.consum%TYPE;
        l_produccio_central HISTORIC_PRODUIDA_MES.potencia%TYPE;
        linia_cap CHAR(40) := '---------------------------------------';

        procediment LOG_APLICACIO.procediment%TYPE := 'pr_con_centrals_produccio';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

        CURSOR c_consulta IS
            SELECT cp.nom, SUM(hlc.consum), SUM(hpm.potencia) 
                  FROM CENTRALS_PRODUCCIO cp INNER JOIN LINIES li
                        ON (cp.id = li.central)
                  INNER JOIN centraletes ce
                        ON (li.id = ce.linia)
                  INNER JOIN centraletes_comptadors cc
                        ON (ce.id = cc.centraleta)
                  INNER JOIN historic_lectures_comptadors hlc
                        ON (cc.comptador = hlc.comptador)
                  INNER JOIN HISTORIC_PRODUIDA_MES hpm
                        ON (cp.id = hpm.central)
                  WHERE cp.nom = t_nom
                        AND hlc.mes <= t_mes
                	AND hlc.anyhis <= t_any
	           GROUP BY cp.nom;

         central_not_exist EXCEPTION;  
         entrada_nom_is_null EXCEPTION; 
         entrada_data_is_null EXCEPTION;
         invalit_date EXCEPTION;
BEGIN
        entrada := '-'||t_nom||'-'||t_data||'-';

        IF check_date(t_data)
            THEN v_date := to_date(t_data, mascara);
                  t_mes := extract(month from v_date);
                  t_any := extract(year from v_date);
            ELSE RAISE invalit_date;
        END IF;

        IF (t_nom IS NULL)
                THEN RAISE entrada_nom_is_null;
        END IF;
        
       IF (t_data IS NULL)
                THEN RAISE entrada_data_is_null;
        END IF;

        v_exist_central := GENERAL.fn_exist_central_x_nom(t_nom);
	IF (NOT v_exist_central)
		THEN RAISE central_not_exist;
        END IF;

    DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');
    DBMS_OUTPUT.PUT_LINE ('Producci de central i comsum de comptadors associats fins a una data.');
    DBMS_OUTPUT.PUT_LINE ('Central :'||t_nom||'  Data :'||v_date);
    DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');

      DBMS_OUTPUT.PUT_LINE (rpad('Nom de la central',25)
                            ||' '||rpad('Consum comptadors',18)
                            ||' '||rpad('Producci central',18));

      DBMS_OUTPUT.PUT_LINE (rpad(linia_cap,25)
                            ||' '||rpad(linia_cap,18)
                            ||' '||rpad(linia_cap,18));

    
        OPEN c_consulta;
        LOOP
            FETCH c_consulta
            INTO l_nom, l_consum_comptadors, l_produccio_central;
            EXIT WHEN c_consulta%NOTFOUND;

		DBMS_OUTPUT.PUT_LINE (rpad(l_nom,25)
                            ||' '||lpad(TO_CHAR(l_consum_comptadors,'999G999G999D99'),18)
                            ||' '||lpad(TO_CHAR(l_produccio_central,'999G999G999D99'),18));

        END LOOP;
        CLOSE c_consulta;
	GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);


EXCEPTION
        WHEN central_not_exist THEN
            ROLLBACK;
            sortida := '-20105 - La central '||t_nom||' no existeix.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;
         WHEN entrada_nom_is_null THEN
            ROLLBACK;
            sortida := '-20106 - El nom de central entrada es un valor nul.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;
        WHEN entrada_data_is_null THEN
            ROLLBACK;
            sortida := '-20107 - La data entrada es un valor nul.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;
        WHEN invalit_date THEN
            ROLLBACK;
            sortida := SQLERRM||'- Error en la data '||t_data ||' entrada.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;
        WHEN OTHERS THEN
            ROLLBACK;
            sortida := SQLERRM||' En la consulta de producci de centrals';
            		GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;
END;



/* Percentatge de lectures de comptadors efectuades de forma
	presencial i de forma telemtica en un perode de temps.
*/

PROCEDURE pr_con_modalitat_lectures
	(t_data IN VARCHAR2
        )
IS
        t_total NUMBER;
        t_presencial NUMBER;
        t_telematica NUMBER;
        v_date DATE;
        mascara VARCHAR2(20) := 'DD/MM/YYYY';
        t_mes NUMBER;
        t_any NUMBER;

        l_lectures_totals NUMBER;        
        l_lectures_presencials NUMBER(5,2);
        l_lectures_telematiques NUMBER(5,2);

        procediment LOG_APLICACIO.procediment%TYPE := 'pr_con_modalitat_lectures';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

        entrada_data_is_null EXCEPTION;
        invalit_date EXCEPTION;
BEGIN
        entrada := '-'||t_data||'-';


        IF check_date(t_data)
            THEN v_date := to_date(t_data, mascara);
                  t_mes := extract(month from v_date);
                  t_any := extract(year from v_date);
            ELSE RAISE invalit_date;
        END IF;

       IF (t_data IS NULL)
                THEN RAISE entrada_data_is_null;
        END IF;

        SELECT COUNT(*) INTO t_total
            FROM LECTURES_COMPTADORS
            WHERE EXTRACT(MONTH FROM  data_lectura) <= t_mes
                AND EXTRACT(YEAR FROM data_lectura) <= t_any;

        SELECT COUNT(*) INTO t_presencial
            FROM LECTURES_COMPTADORS
            WHERE modalitat_lectora = 'P'
                AND EXTRACT(MONTH FROM  data_lectura) <= t_mes
                AND EXTRACT(YEAR FROM data_lectura) <= t_any;
            
        SELECT COUNT(*) INTO t_telematica
            FROM LECTURES_COMPTADORS
            WHERE modalitat_lectora = 'T'
                AND EXTRACT(MONTH FROM  data_lectura) <= t_mes
               AND EXTRACT(YEAR FROM data_lectura) <= t_any;
 
        l_lectures_totals := t_total;
        l_lectures_presencials := ((t_presencial*100)/t_total);
        l_lectures_telematiques := ((t_telematica*100)/t_total);

    DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');
    DBMS_OUTPUT.PUT_LINE ('Percentatge de les modalitats lectores de comptadors fins a una data.');
    DBMS_OUTPUT.PUT_LINE ('total :'||l_lectures_totals||'  Presencial :'||l_lectures_presencials||'%  Telemtica :'||l_lectures_telematiques||'%' );
    DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');

	GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);

EXCEPTION
        WHEN entrada_data_is_null THEN
            ROLLBACK;
            sortida := '-20108 - La data entrada es un valor nul.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;
        WHEN OTHERS THEN
            ROLLBACK;
            sortida := SQLERRM||' En la consulta de producci de centrals';
            		GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;
END;


/* Llistat de comptadors que tinguin un determinat nombre
	danys dantiguitat.
*/

PROCEDURE pr_con_anys_comptadors
	(t_antiguitat IN NUMBER
        )
IS
        l_id_comptador COMPTADORS.id%TYPE;
        l_numero_contracte COMPTADORS.numero_contracte%TYPE;     
        l_model COMPTADORS.model%TYPE;
        l_any_fabricacio COMPTADORS.any_fabricacio%TYPE;
        linia_cap CHAR(40) := '---------------------------------------';

        procediment LOG_APLICACIO.procediment%TYPE := 'pr_con_modalitat_lectures';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

        CURSOR c_consulta IS
            SELECT co.id, co.numero_contracte, co.model, co.any_fabricacio 
                FROM COMPTADORS co
                WHERE (EXTRACT(YEAR FROM SYSDATE)- any_fabricacio) > t_antiguitat;
 
	entrada_antiguitat_null EXCEPTION;
BEGIN
        entrada := '-'||t_antiguitat||'-';

        IF (t_antiguitat IS NULL)
            THEN RAISE entrada_antiguitat_null;
        END IF;
        
    DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');
    DBMS_OUTPUT.PUT_LINE ('Comptadors amb '||t_antiguitat||' anys de antiguitat .');
    DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');

      DBMS_OUTPUT.PUT_LINE (rpad('ID comptador',13)
                            ||' '||rpad('Contracte',13)
                            ||' '||rpad('Model comptador',16)
                            ||' '||rpad('Anys antiguitat',16));

      DBMS_OUTPUT.PUT_LINE (rpad(linia_cap,13)
                            ||' '||rpad(linia_cap,13)
                            ||' '||rpad(linia_cap,16)
                            ||' '||rpad(linia_cap,16));

        OPEN c_consulta;
        LOOP
            FETCH c_consulta
            INTO l_id_comptador, l_numero_contracte, l_model, l_any_fabricacio;
            EXIT WHEN c_consulta%NOTFOUND;
 
		DBMS_OUTPUT.PUT_LINE (lpad(TO_CHAR(l_id_comptador),13)
                            ||' '||rpad(TO_CHAR(l_numero_contracte),13)
                            ||' '||rpad(l_model,16)
                            ||' '||lpad(l_any_fabricacio,16));

        END LOOP;
        CLOSE c_consulta;
	GENERAL.pr_alta_log_aplicacio(procediment, entrada, sortida);

EXCEPTION

        WHEN entrada_antiguitat_null THEN
            ROLLBACK;
            sortida := '-20109 Error en entrada de parmetres, no pot ser nul.';
            		GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;
        WHEN OTHERS THEN
            ROLLBACK;
            sortida := SQLERRM||' En la consulta de producci de centrals';
            		GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;
END;


END;
/

disconnect;