connect Administrador/a12345

CREATE OR REPLACE
PACKAGE BODY PR_ENTRADES AS

/* ------------------------------------------------------------------- Crrega de dades diaries

-----------------------------------------------------------------------------------------------

/* ------------------------------------------- FUncions de lectures_comptador
*/

/*------------------------------------------------ fn_client_x_NC
Aquesta funci cercar del comptador el id client,
i comprobar que existeixi
*/

FUNCTION fn_client_x_NC
	(t_numero_contracte IN LECTURES_COMPTADORS.numero_contracte%TYPE
	)

RETURN BOOLEAN
AS

V_id_client NUMBER;
v_client_exist BOOLEAN;

BEGIN
	select client INTO v_id_client FROM COMPTADORS
		WHERE numero_contracte = t_numero_contracte;

	v_client_exist := general.fn_id_exist_dinamic('CLIENTS', v_id_client);


	RETURN v_client_exist;
END;

/*------------------------------------------------ fn_companyia_client
Aquesta funci cercar del comptador el id client,
lleguir del client per id i retornar la companyia
*/

FUNCTION fn_companyia_client
	(t_numero_contracte IN LECTURES_COMPTADORS.numero_contracte%TYPE
	)

RETURN VARCHAR2
AS

V_id_client NUMBER;
v_companyia VARCHAR2(20);

BEGIN
	select client INTO v_id_client FROM COMPTADORS
		WHERE numero_contracte = t_numero_contracte;

	select companyia INTO v_companyia FROM CLIENTS
		WHERE id = v_id_client;

	RETURN v_companyia;
END;



/* ----------------------------------------------------- fn_exist_his_comp_mes_any
Aquesta funci cercar en l'histric de lectures si existeix algun acumulat
per aquest comptador, mes i any.
*/

FUNCTION fn_exist_his_comp_mes_any
	(t_id_comptador IN HISTORIC_LECTURES_COMPTADORS.comptador%TYPE,
	t_mes IN HISTORIC_LECTURES_COMPTADORS.mes%TYPE,
	t_anyhis IN HISTORIC_LECTURES_COMPTADORS.anyhis%TYPE
	)

RETURN BOOLEAN
AS

t_exist NUMBER;


BEGIN
	select COUNT(1) INTO t_exist FROM HISTORIC_LECTURES_COMPTADORS
		WHERE comptador = t_id_comptador AND mes = t_mes AND anyhis = t_anyhis;

	RETURN (t_exist > 0);
END;



/* ----------------------------------------------------- fn_calcul_con_mitja_anual_comp
Calcula el consum mitja anual del comptador
Accedeix a la taula HISTORIC_LECTURES_COMPTADORS extreu la dada i la retorna
*/

FUNCTION fn_calcul_con_mitja_anual_comp
	(t_numero_contracte IN LECTURES_COMPTADORS.numero_contracte%TYPE
	)

RETURN NUMBER
AS

v_calcul NUMBER;

BEGIN

	select AVG(total_consum) INTO v_calcul
        	from (select numero_contracte, extract(year from data_lectura) as anylec, sum(consum) as total_consum
		        from lectures_comptadors 
		        where numero_contracte = t_numero_contracte
		        group by numero_contracte, extract(year from data_lectura)
		        order by numero_contracte,extract(year from data_lectura));

	RETURN v_calcul;
END;



/* ----------------------------------------------------- fn_cerca_id_client_x_comp
Retorna el id_client de la taula COMPTADORS
*/

FUNCTION fn_cerca_id_client_x_comp
	(t_id_comptador IN COMPTADORS.id%TYPE
	)

RETURN NUMBER
AS

t_id_client NUMBER;


BEGIN
	select client INTO t_id_client FROM COMPTADORS
		WHERE id = t_id_comptador;

	RETURN t_id_client;
END;



/* ----------------------------------------------------- fn_calcul_con_mtj_anual_client
Calcula la mitjana del consum anual del client per a tots els comptador d'aquest
*/

FUNCTION fn_calcul_con_mtj_anual_client
	(t_id_client IN CLIENTS.id%TYPE
	)

RETURN NUMBER
AS

v_calcul NUMBER;


BEGIN

	select AVG(total_consum) INTO v_calcul
        	from (select cl.DNI_NIF, extract(year from lc.data_lectura) as anylec, sum(lc.consum) as total_consum
			from (clients cl INNER JOIN comptadors co
		        ON cl.id = co.client AND cl.id=t_id_client)
		        INNER JOIN lectures_comptadors lc
		                ON co.numero_contracte = lc.numero_contracte
			        group by cl.DNI_NIF, extract(year from lc.data_lectura)
		        	order by cl.DNI_NIF ,extract(year from lc.data_lectura));

	RETURN v_calcul;
END;



/* ----------------------------------------------------- fn_exist_his_linia_mes
Aquesta funci cercar en l'histric de linies si existeix algun acumulat
per aquest linia, mes i any.
*/

FUNCTION fn_exist_his_linia_mes
	(t_id_linia IN HISTORIC_CARREGA_LINIA_MES.linia%TYPE,
	t_mes IN HISTORIC_CARREGA_LINIA_MES.mes%TYPE,
	t_anyhis IN HISTORIC_CARREGA_LINIA_MES.anyhis%TYPE
	)

RETURN BOOLEAN
AS

t_exist NUMBER;


BEGIN
	select COUNT(1) INTO t_exist FROM HISTORIC_CARREGA_LINIA_MES
		WHERE linia = t_id_linia AND mes = t_mes AND anyhis = t_anyhis;

	RETURN (t_exist > 0);
END;



/* ----------------------------------------------------- fn_exist_his_linia_anual
Aquesta funci cercar en l'histric de linies si existeix algun acumulat
per aquest linia i any.
*/

FUNCTION fn_exist_his_linia_anual
	(t_id_linia IN HISTORIC_CARREGA_LINIA_ANUAL.linia%TYPE,
	t_anyhis IN HISTORIC_CARREGA_LINIA_ANUAL.anyhis%TYPE
	)

RETURN BOOLEAN
AS

t_exist NUMBER;


BEGIN
	select COUNT(1) INTO t_exist FROM HISTORIC_CARREGA_LINIA_ANUAL
		WHERE linia = t_id_linia AND anyhis = t_anyhis;

	RETURN (t_exist > 0);
END;



/* ----------------------------------------------------- fn_calcul_carrega_mitja_anual
Calcula la mitjana de la carrega anual de la linia
*/

FUNCTION fn_calcul_carrega_mitja_anual
	(t_id_linia IN HISTORIC_CARREGA_LINIA_ANUAL.linia%TYPE,
	t_anyhis IN HISTORIC_CARREGA_LINIA_ANUAL.anyhis%TYPE
	)

RETURN NUMBER
AS

v_calcul NUMBER;


BEGIN

	select AVG(carrega) INTO v_calcul FROM HISTORIC_CARREGA_LINIA_MES
		WHERE linia = t_id_linia AND anyhis = t_anyhis;

	RETURN v_calcul;
END;


/* ----------------------------------------------------- fn_exist_his_produccio_mes
Aquesta funci cercar en l'histric de centrals mensual si existeix algun acumulat
per aquesta central, mes i any.
*/

FUNCTION fn_exist_his_produccio_mes
	(t_id_central IN HISTORIC_PRODUIDA_MES.central%TYPE,
	t_mes IN HISTORIC_PRODUIDA_MES.mes%TYPE,
	t_anyhis IN HISTORIC_PRODUIDA_MES.anyhis%TYPE
	)

RETURN BOOLEAN
AS

t_exist NUMBER;


BEGIN
	select COUNT(1) INTO t_exist FROM HISTORIC_PRODUIDA_MES
		WHERE central = t_id_central AND mes = t_mes AND anyhis = t_anyhis;

	RETURN (t_exist > 0);
END;


/* ----------------------------------------------------- fn_exist_his_produccio_anual
Aquesta funci cercar en l'histric de centrals anuals si existeix algun acumulat
per aquesta central i any.
*/

FUNCTION fn_exist_his_produccio_anual
	(t_id_central IN HISTORIC_PRODUIDA_ANUAL.central%TYPE,
	t_anyhis IN HISTORIC_PRODUIDA_ANUAL.anyhis%TYPE
	)

RETURN BOOLEAN
AS

t_exist NUMBER;


BEGIN
	select COUNT(1) INTO t_exist FROM HISTORIC_PRODUIDA_ANUAL
		WHERE central = t_id_central AND anyhis = t_anyhis;

	RETURN (t_exist > 0);
END;


/* ----------------------------------------------------- fn_exist_his_lec_comp_mes_any
Aquesta funci cercar en l'histric de centrals anuals si existeix algun acumulat
per aquesta central i any.
*/

FUNCTION fn_exist_his_lec_comp_mes_any
	(t_id_comptador IN HISTORIC_LECTURES_COMPTADORS.comptador%TYPE,
	t_mes IN HISTORIC_LECTURES_COMPTADORS.mes%TYPE,
	t_anyhis IN HISTORIC_LECTURES_COMPTADORS.anyhis%TYPE
	)

RETURN BOOLEAN
AS

t_exist NUMBER;


BEGIN
	select COUNT(1) INTO t_exist FROM HISTORIC_LECTURES_COMPTADORS
		WHERE comptador = t_id_comptador AND mes = t_mes AND anyhis = t_anyhis;

	RETURN (t_exist > 0);
END;


/* -------------------------------------------- Cerca la central per comptador
Entra el id del comptador
Cerca a la relaci centraletes_comptadors el id de la centraleta
Cerca a la taula CENTALETES el id de la linia
Cerca a la taula linies per id_central
Retorna el id_central de la taula linies
*/

FUNCTION fn_cerca_central_x_comptador
	(t_id_comptador IN COMPTADORS.id%TYPE
	)

RETURN NUMBER
AS

t_id_centraleta NUMBER;
t_id_linia NUMBER;
t_id_central NUMBER;

BEGIN

	SELECT li.central INTO t_id_central
		FROM ((CENTRALETES_COMPTADORS cc JOIN CENTRALETES ce ON cc.comptador = t_id_comptador)
			JOIN LINIES li ON ce.linia = li.id);


    RETURN t_id_central;
END;



/* ------------------------------------------ Comprova que la ciutat t comptadors
Retorna TRUE si existeixen comptadors de la ciutat.
Retorna FALSE si no existeixen.
*/

FUNCTION fn_exist_localitat
      (t_localitat IN COMPTADORS.localitat%TYPE)

RETURN BOOLEAN
AS

t_exist NUMBER;

BEGIN

    SELECT COUNT(1) INTO t_exist FROM COMPTADORS
          WHERE localitat=t_localitat;

 RETURN (t_exist > 0);
END;
/* ---------------------------------------------------------------------------------- Procediments
*/

/* --------------------------------------------------- Altualitza taula HISTORIC_LECTURES_COMPTADORS
*/

PROCEDURE pr_actua_his_lec_comptadors
	(t_id_comptador IN HISTORIC_LECTURES_COMPTADORS.comptador%TYPE,
	t_anyhis HISTORIC_LECTURES_COMPTADORS.anyhis%TYPE,
	t_mes IN HISTORIC_LECTURES_COMPTADORS.mes%TYPE,
	t_consum IN HISTORIC_LECTURES_COMPTADORS.consum%TYPE
	)
IS
	v_exist_mes_any BOOLEAN;

	procediment LOG_APLICACIO.procediment%TYPE := 'pr_actua_his_lectures_comptadors';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

BEGIN
	v_exist_mes_any := FN_EXIST_HIS_LEC_COMP_MES_ANY(t_id_comptador, t_mes, t_anyhis);

        entrada := '-'||t_id_comptador||'-'||t_anyhis||'-'||t_mes||'-'||t_consum||'-';

	IF (v_exist_mes_any)
		THEN UPDATE HISTORIC_LECTURES_COMPTADORS SET
			consum = consum + t_consum
                        WHERE comptador = t_id_comptador AND 
                              mes = t_mes AND 
                              anyhis =t_anyhis;
                   	GENERAL.pr_alta_log_aplicacio(procediment, entrada, sortida);
		ELSE INSERT INTO HISTORIC_LECTURES_COMPTADORS(comptador, anyhis, mes, consum)
			VALUES (t_id_comptador, t_anyhis, t_mes, t_consum);
                    	GENERAL.pr_alta_log_aplicacio(procediment, entrada, sortida);

	END IF;

EXCEPTION
        WHEN OTHERS THEN
	    ROLLBACK;
            DBMS_OUTPUT.PUT_LINE
		('Error'||SQLCODE||SQLERRM||'- Al actualitzar histric de lectures de comptadors');
	    COMMIT;
END;



/* --------------------------------------------------- Altualitza taula COMPTADORS
*/

PROCEDURE pr_actua_comptador
	(t_numero_contracte IN COMPTADORS.numero_contracte%TYPE,
	t_consum IN COMPTADORS.lectura_acumulada%TYPE
	)
IS

	v_consum_mitja_anual NUMBER;

	procediment LOG_APLICACIO.procediment%TYPE := 'pr_actua_comptador';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

BEGIN

	v_consum_mitja_anual := FN_CALCUL_CON_MITJA_ANUAL_COMP(t_numero_contracte);
        entrada := '-'||t_numero_contracte||'-'||t_consum||'-';

	UPDATE COMPTADORS SET
		lectura_acumulada = lectura_acumulada + t_consum,
		consum_mitja_anual = v_consum_mitja_anual
                WHERE numero_contracte = t_numero_contracte;
        GENERAL.pr_alta_log_aplicacio(procediment, entrada, sortida);

EXCEPTION
        WHEN OTHERS THEN
	    ROLLBACK;
            DBMS_OUTPUT.PUT_LINE
		('Error'||SQLCODE||SQLERRM||'- Al actualitzar taula de comptadors');
	    COMMIT;
END;



/* --------------------------------------------------- Altualitza taula CLIENTS
*/

PROCEDURE pr_actua_client
	(t_id_comptador IN COMPTADORS.id%TYPE,
	t_consum IN COMPTADORS.lectura_acumulada%TYPE
	)
IS

	v_consum_mitja_anual NUMBER;
	v_id_client NUMBER;

	procediment LOG_APLICACIO.procediment%TYPE := 'pr_actua_client';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

BEGIN
	v_id_client := FN_CERCA_ID_CLIENT_x_COMP(t_id_comptador);
	v_consum_mitja_anual := FN_CALCUL_CON_MTJ_ANUAL_CLIENT(v_id_client);
        entrada := '-'||t_id_comptador||'-'||t_consum||'-';

	UPDATE CLIENTS SET
		consum_mitja = v_consum_mitja_anual
                WHERE id = v_id_client;
        GENERAL.pr_alta_log_aplicacio(procediment, entrada, sortida);

EXCEPTION
        WHEN OTHERS THEN
	    ROLLBACK;
            DBMS_OUTPUT.PUT_LINE
		('Error'||SQLCODE||SQLERRM||'- Al actualitzar taula de clients');
	    COMMIT;
END;



/* ------------------------------------------------------- Actualitza taula LINIA
	Un comptador pot estar connectat mitjanant 
		 diferents centraletes a diferenats lnies
*/

PROCEDURE pr_actua_linia
	(t_id_comptador IN COMPTADORS.id%TYPE,
	t_consum IN COMPTADORS.lectura_acumulada%TYPE
	)
IS

	procediment LOG_APLICACIO.procediment%TYPE := 'pr_actua_linia';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	CURSOR linies_comptador IS 
		select ce.linia FROM CENTRALETES_COMPTADORS cc JOIN CENTRALETES ce
         	       ON cc.comptador = t_id_comptador
              		  AND cc.centraleta = ce.id;
        v_id_linia CENTRALETES.linia%TYPE;

BEGIN
        entrada := '-'||t_id_comptador||'-'||t_consum||'-';

	OPEN linies_comptador;
	FETCH linies_comptador INTO v_id_linia;
	WHILE linies_comptador%FOUND LOOP
		UPDATE LINIES SET
			carrega_acumulada = carrega_acumulada + t_consum
		WHERE id = v_id_linia;
		FETCH linies_comptador INTO v_id_linia;
      		GENERAL.pr_alta_log_aplicacio(procediment, entrada, sortida);  
	END LOOP;

EXCEPTION
        WHEN OTHERS THEN
	    ROLLBACK;
            DBMS_OUTPUT.PUT_LINE
		('Error'||SQLCODE||SQLERRM||'- Al actualitzar taula de lnies');
	    COMMIT;
END;



/* ---------------------------------------------- Altualitza histric de carrega de lnies anual
*/


PROCEDURE pr_his_carrega_linia_anual
	(t_id_linia IN HISTORIC_CARREGA_LINIA_ANUAL.linia%TYPE,
	t_anyhis IN HISTORIC_CARREGA_LINIA_MES.mes%TYPE,
	t_consum IN LECTURES_COMPTADORS.consum%TYPE
	)
IS

	v_exist BOOLEAN;
	v_carrega_mitja_anual NUMBER;

	procediment LOG_APLICACIO.procediment%TYPE := 'pr_his_carrega__linia_anual';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

BEGIN

        entrada := '-'||t_id_linia||'-'||t_anyhis||'-'||t_consum||'-';
	v_exist := fn_exist_his_linia_anual(t_id_linia, t_anyhis);
	v_carrega_mitja_anual := fn_calcul_carrega_mitja_anual(t_id_linia, t_anyhis);

	IF (v_exist)
		THEN UPDATE HISTORIC_CARREGA_LINIA_ANUAL SET
			carrega_mitja_anual = v_carrega_mitja_anual,
			carrega = carrega + t_consum
		               WHERE linia = t_id_linia AND anyhis = t_anyhis;
                        GENERAL.pr_alta_log_aplicacio(procediment, entrada, sortida);
		ELSE INSERT INTO HISTORIC_CARREGA_LINIA_ANUAL(linia, anyhis, carrega_mitja_anual, carrega)
			VALUES (t_id_linia, t_anyhis, v_carrega_mitja_anual, t_consum);
                    	GENERAL.pr_alta_log_aplicacio(procediment, entrada, sortida);
        END IF;

EXCEPTION
        WHEN OTHERS THEN
	    ROLLBACK;
            DBMS_OUTPUT.PUT_LINE
		('Error'||SQLCODE||SQLERRM||'- Al actualitzar histtric de carrega de lnia anual');
	    COMMIT;
END;



/* ---------------------------------------------- Altualitza histric de carrega de lnies mensual
*/

PROCEDURE pr_his_carrega_linia_mes
	(t_id_comptador IN COMPTADORS.id%TYPE,
	t_mes IN HISTORIC_CARREGA_LINIA_MES.mes%TYPE,
	t_anyhis IN HISTORIC_CARREGA_LINIA_MES.mes%TYPE,
	t_consum IN COMPTADORS.lectura_acumulada%TYPE
	)
IS

	v_exist BOOLEAN;
	procediment LOG_APLICACIO.procediment%TYPE := 'pr_his_carrega__linia_mes';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	CURSOR linies_comptador IS 
		select ce.linia FROM CENTRALETES_COMPTADORS cc JOIN CENTRALETES ce
         	       ON cc.comptador = t_id_comptador
              		  AND cc.centraleta = ce.id;
                          
	v_id_linia CENTRALETES.linia%TYPE;

BEGIN

        entrada := '-'||t_id_comptador||'-'||t_mes||'-'||t_anyhis||'-'||t_consum||'-';

	OPEN linies_comptador;
	FETCH linies_comptador INTO v_id_linia;
	WHILE linies_comptador%FOUND LOOP
		v_exist := fn_exist_his_linia_mes(v_id_linia, t_mes, t_anyhis);

		IF (v_exist)
			THEN UPDATE HISTORIC_CARREGA_LINIA_MES SET
				carrega = carrega + t_consum
		               	WHERE linia = v_id_linia AND mes = t_mes AND anyhis = t_anyhis;
        	               	GENERAL.pr_alta_log_aplicacio(procediment, entrada, sortida);
                	       	pr_his_carrega_linia_anual(v_id_linia, t_anyhis, t_consum);  
			ELSE INSERT INTO HISTORIC_CARREGA_LINIA_MES(linia, mes, anyhis, carrega)
				VALUES (v_id_linia, t_mes, t_anyhis, t_consum);
                	    	GENERAL.pr_alta_log_aplicacio(procediment, entrada, sortida);
                        	pr_his_carrega_linia_anual(v_id_linia, t_anyhis, t_consum);  
	        END IF;

		FETCH linies_comptador INTO v_id_linia;
	END LOOP;

EXCEPTION
        WHEN OTHERS THEN
	    ROLLBACK;
            DBMS_OUTPUT.PUT_LINE
		('Error'||SQLCODE||SQLERRM||'- Al actualitzar histric de carrega de lnia mensual');
	    COMMIT;
END;



/* ---------------------------------------------- Altualitza histric de produccio de centrals anual
*/

PROCEDURE pr_his_produccio_anual
	(t_id_central IN HISTORIC_PRODUIDA_ANUAL.central%TYPE,
	t_anyhis IN HISTORIC_PRODUIDA_MES.mes%TYPE,
	t_consum IN LECTURES_COMPTADORS.consum%TYPE
	)
IS

	v_exist BOOLEAN;

	procediment LOG_APLICACIO.procediment%TYPE := 'pr_his_produccio_anual';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

BEGIN

        entrada := '-'||t_id_central||'-'||t_anyhis||'-'||t_consum||'-';
	v_exist := fn_exist_his_produccio_anual(t_id_central, t_anyhis);

	IF (v_exist)
		THEN UPDATE HISTORIC_PRODUIDA_ANUAL SET
			potencia = potencia + t_consum
		               WHERE central = t_id_central AND anyhis = t_anyhis;
	        GENERAL.pr_alta_log_aplicacio(procediment, entrada, sortida);
		ELSE INSERT INTO HISTORIC_PRODUIDA_ANUAL(central, anyhis, potencia)
			VALUES (t_id_central, t_anyhis, t_consum);
                    	GENERAL.pr_alta_log_aplicacio(procediment, entrada, sortida);
        END IF;
 
EXCEPTION
        WHEN OTHERS THEN
	    ROLLBACK;
            DBMS_OUTPUT.PUT_LINE
		('Error'||SQLCODE||SQLERRM||'- Al actualitzar histric de producci de centrals anual');
	    COMMIT;
END;




/* ---------------------------------------------- Altualitza histric de produccio de centrals mensual
*/

PROCEDURE pr_his_produccio_mes
	(t_id_comptador IN COMPTADORS.id%TYPE,
	t_mes IN HISTORIC_CARREGA_LINIA_MES.mes%TYPE,
	t_anyhis IN HISTORIC_CARREGA_LINIA_MES.mes%TYPE,
	t_consum IN COMPTADORS.lectura_acumulada%TYPE
	)
IS

	v_exist BOOLEAN;

	procediment LOG_APLICACIO.procediment%TYPE := 'pr_his_produccio_mes';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	CURSOR centrals_comptador IS 
		select distinct li.central FROM (CENTRALETES_COMPTADORS cc JOIN CENTRALETES ce
         	       ON cc.comptador = t_id_comptador
              		  AND cc.centraleta = ce.id)
                          JOIN LINIES li ON ce.linia = li.id;
                          
	v_id_central LINIES.central%TYPE;


BEGIN

        entrada := '-'||t_id_comptador||'-'||t_mes||'-'||t_anyhis||'-'||t_consum||'-';


	OPEN centrals_comptador;
	FETCH centrals_comptador INTO v_id_central;
	WHILE centrals_comptador%FOUND LOOP

		v_exist := fn_exist_his_produccio_mes(v_id_central, t_mes, t_anyhis);

		IF (v_exist)
			THEN UPDATE HISTORIC_PRODUIDA_MES SET
				potencia = potencia + t_consum
			               WHERE central = v_id_central AND mes = t_mes AND anyhis = t_anyhis;
		        	GENERAL.pr_alta_log_aplicacio(procediment, entrada, sortida);
				pr_his_produccio_anual(v_id_central, t_anyhis, t_consum);  
			ELSE INSERT INTO HISTORIC_PRODUIDA_MES(central, mes, anyhis, potencia)
				VALUES (v_id_central, t_mes, t_anyhis, t_consum);
                    		GENERAL.pr_alta_log_aplicacio(procediment, entrada, sortida);
				pr_his_produccio_anual(v_id_central, t_anyhis, t_consum); 
        	END IF;

		FETCH centrals_comptador INTO v_id_central;
	END LOOP;

EXCEPTION
        WHEN OTHERS THEN
	    ROLLBACK;
            DBMS_OUTPUT.PUT_LINE
		('Error'||SQLCODE||SQLERRM||'- Al actualitzar histric de producci de centrals mensual');
	    COMMIT;
END;



/*----------------------------------------------------- Alta de lectures de comptadors
*/

PROCEDURE pr_alta_lectura_comptador
	(t_numero_contracte IN LECTURES_COMPTADORS.numero_contracte%TYPE,
	t_data_lectura IN LECTURES_COMPTADORS.data_lectura%TYPE,
	t_consum IN LECTURES_COMPTADORS.consum%TYPE,
	t_modalitat IN LECTURES_COMPTADORS.modalitat_lectora%TYPE
	)
IS

	v_companyia VARCHAR2(20);
	v_exist_comptador BOOLEAN;
	v_client_exist BOOLEAN;
	t_mes NUMBER;
	t_anyhis NUMBER;
	t_id_comptador NUMBER;

	procediment LOG_APLICACIO.procediment%TYPE := 'pr_alta_lectura_comptador';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

	comptador_not_exist EXCEPTION;
	client_not_exist EXCEPTION;
BEGIN

 	v_exist_comptador := general.FN_COMPTADOR_EXIST_X_NC(t_numero_contracte);

	v_client_exist := fn_client_x_NC(t_numero_contracte);
	IF (v_client_exist)
		THEN v_companyia := FN_COMPANYIA_CLIENT(t_numero_contracte);
	END IF;

	t_mes := extract(month from t_data_lectura);
	t_anyhis := extract(year from t_data_lectura);

        entrada := '-'||t_numero_contracte||'-'||t_data_lectura||'-'||t_consum||'-'||t_modalitat||'-';

	IF (NOT v_exist_comptador)
        	THEN RAISE comptador_not_exist;
		ELSIF ( NOT v_client_exist)
			THEN RAISE client_not_exist;
			ELSE INSERT INTO LECTURES_COMPTADORS(numero_contracte, data_lectura, consum, modalitat_lectora, companyia)
					VALUES (t_numero_contracte, t_data_lectura, t_consum, t_modalitat, v_companyia);
	                    	GENERAL.pr_alta_log_aplicacio
        	                        (procediment, entrada, sortida);
				t_id_comptador := general.fn_cerca_id_comptador_per_NC  
                                              (t_numero_contracte);
				pr_actua_his_lec_comptadors
					(t_id_comptador, t_anyhis, t_mes, t_consum);
				pr_actua_comptador
					(t_numero_contracte, t_consum);
				pr_actua_client
					(t_id_comptador, t_consum);
				pr_actua_linia
					(t_id_comptador, t_consum);
				pr_his_carrega_linia_mes
					(t_id_comptador, t_mes, t_anyhis, t_consum);
				pr_his_produccio_mes
					(t_id_comptador, t_mes, t_anyhis, t_consum);
	END IF;

EXCEPTION
	WHEN comptador_not_exist THEN
		ROLLBACK;
		sortida := '-20088 - El comptador '||t_numero_contracte||' no existeix o est inactiu.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
		COMMIT;

	WHEN client_not_exist THEN
		ROLLBACK;
		sortida := '-20089 - El client del contracte '||t_numero_contracte||' no existeix o est inactiu.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
		COMMIT;

        WHEN OTHERS THEN
	    	ROLLBACK;
        	sortida :='Error'||SQLCODE||SQLERRM||'- Al actualitzar la lectura dels comptadors';
			GENERAL.pr_alta_log_aplicacio(procediment,entrada,sortida);
		COMMIT;
END;

END;
/

disconnect;


