connect Administrador/a12345

CREATE OR REPLACE
PACKAGE BODY "PR_ESTADISTIQUES" AS

/* --------------------------------------------- Procediments d'estadstiques
*/

/* Donada una central de producci, el consum dels comptadors que depenen de la central.
*/

PROCEDURE pr_est_con_comptadors_central
	(t_nom IN CENTRALS_PRODUCCIO.nom%TYPE
        )
IS
        v_exist_central BOOLEAN;
        
        l_nom CENTRALS_PRODUCCIO.nom%TYPE;
        l_id_comptador COMPTADORS.id%TYPE;
        l_numero_contracte COMPTADORS.numero_contracte%TYPE;
        l_lectura_acumulada COMPTADORS.lectura_acumulada%TYPE;
        linia_cap CHAR(40) := '---------------------------------------';

        procediment LOG_APLICACIO.procediment%TYPE := 'pr_est_consum_comptadors_central';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

        CURSOR c_consulta IS
              SELECT co.id, co.numero_contracte, co.lectura_acumulada 
                  FROM centrals_produccio cp INNER JOIN LINIES li 
                        ON (cp.nom = t_nom AND cp.id = li.central)
                  INNER JOIN centraletes ce
                        ON (li.id = ce.linia)
                  INNER JOIN centraletes_comptadors cc
                        ON (ce.id = cc.centraleta)
                  INNER JOIN comptadors co
                        ON (cc.comptador = co.id);

         central_not_exist EXCEPTION;  
         entrada_nom_is_null EXCEPTION; 
BEGIN
        entrada := '-'||t_nom||'-';

        IF (t_nom IS NULL)
                THEN RAISE entrada_nom_is_null;
        END IF;
        
        v_exist_central := GENERAL.fn_exist_central_x_nom(t_nom);
	IF (NOT v_exist_central)
		THEN RAISE central_not_exist;
        END IF;

    	DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');
    	DBMS_OUTPUT.PUT_LINE ('Consum acumulat dels comptadors connectats a una central de producci donada.');
    	DBMS_OUTPUT.PUT_LINE ('Central :'||t_nom);
    	DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');

    	DBMS_OUTPUT.PUT_LINE (rpad('ID comptador',13)
                            ||' '||rpad('Nmero de contracte',20)
                            ||' '||rpad('Consum acumulat',16));

    	DBMS_OUTPUT.PUT_LINE (rpad(linia_cap,13)
                            ||' '||rpad(linia_cap,20)
                            ||' '||rpad(linia_cap,16));

        OPEN c_consulta;
        LOOP
            FETCH c_consulta
            INTO l_id_comptador, l_numero_contracte, l_lectura_acumulada;
            EXIT WHEN c_consulta%NOTFOUND;
                
	    DBMS_OUTPUT.PUT_LINE (lpad(TO_CHAR(l_id_comptador),13)
                            ||' '||lpad(TO_CHAR(l_numero_contracte),20)
                            ||' '||lpad(TO_CHAR(l_lectura_acumulada,'999G999D99'),16));
        END LOOP;
        CLOSE c_consulta;
	GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);

EXCEPTION
        WHEN central_not_exist THEN
            ROLLBACK;
            sortida := '-20201 - La central '||t_nom||' no existeix.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;
         WHEN entrada_nom_is_null THEN
            ROLLBACK;
            sortida := '-20202 - El nom de central entrada es un valor nul.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;
        WHEN OTHERS THEN
            ROLLBACK;
            sortida := SQLERRM||' A l''estadstica de consum de comptadors per central';
            		GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;
END;


/* Donada una lnia de comunicaci i un any concret, el valor
	mitj de lenergia consumida, tenint en compte que aquest
	consum depn dels comptadors que salimenten mitjanant
	aquesta lnia. Si un comptador pot fer servir dues lnies,
	suposeu que les dues lnies computen el consum per
	garantir que les lnies estan ben dimensionades en cas de
	caiguda dalguna delles.
*/

PROCEDURE pr_est_consum_linia_any
	(t_linia IN LINIES.id%TYPE,
        t_any IN HISTORIC_CARREGA_LINIA_ANUAL.anyhis%TYPE
        )
IS
        v_exist_linia BOOLEAN;
        t_carrega_mitja_anual HISTORIC_CARREGA_LINIA_ANUAL.carrega_mitja_anual%TYPE;
       
        procediment LOG_APLICACIO.procediment%TYPE := 'pr_est_consum_linia_any';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

         linia_not_exist EXCEPTION;  
         entrada_linia_is_null EXCEPTION; 
         entrada_any_is_null EXCEPTION;
         
BEGIN
        entrada := '-'||t_linia||'-'||t_any;

        IF (t_linia IS NULL)
                THEN RAISE entrada_linia_is_null;
        END IF;

        IF (t_any IS NULL)
                THEN RAISE entrada_any_is_null;
        END IF;

        v_exist_linia := GENERAL.fn_id_exist_dinamic('LINIES',t_linia);
	IF (NOT v_exist_linia)
		THEN RAISE linia_not_exist;
        END IF;
        
        SELECT carrega_mitja_anual INTO t_carrega_mitja_anual
               FROM HISTORIC_CARREGA_LINIA_ANUAL
               WHERE linia = t_linia 
               AND anyhis = t_any;

    DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');
    DBMS_OUTPUT.PUT_LINE ('valor mitjar d''energia consumida per una lnia en un any');
    DBMS_OUTPUT.PUT_LINE ('Linia :'||t_linia||'      Any :'||t_any||'        energia mitja consumida :'||TO_CHAR(t_carrega_mitja_anual,'999G999D99'));
    DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');
    
	GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);

EXCEPTION
        WHEN linia_not_exist THEN
            ROLLBACK;
            sortida := '-20203 - La lnia '||t_linia||' no existeix.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;
            
         WHEN entrada_linia_is_null THEN
            ROLLBACK;
            sortida := '-20204 - La lnia entrada es un valor nul.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;
            
         WHEN entrada_any_is_null THEN
            ROLLBACK;
            sortida := '-20205 - Any entrat es un valor nul.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT; 
            
        WHEN OTHERS THEN
            ROLLBACK;
            sortida := SQLERRM||' A l''estadstica cel valor nitja d''energia consumida per lnia i any';
            		GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;
END;



/* Lnia que ha estat ms carregada a nivell denergia consumida.
*/

PROCEDURE pr_est_carrega_linia
	
IS
        procediment LOG_APLICACIO.procediment%TYPE := 'pr_est_carrega_linia';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

        t_id_linia LINIES.id%TYPE;
        t_carrega LINIES.carrega_acumulada%TYPE;
        
        l_id_linia CHAR(9);
        l_carrega CHAR(10);

        CURSOR c_consulta IS
              SELECT lia.id, lia.carrega_acumulada INTO t_id_linia, t_carrega
                      FROM LINIES lia
                      WHERE lia.id 
                            NOT IN (SELECT DISTINCT lib.id
                                           FROM LINIES lib INNER JOIN LINIES lic
                                           ON (lib.carrega_acumulada < lic.carrega_acumulada));

BEGIN
        entrada := '-';


    DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');
    DBMS_OUTPUT.PUT_LINE ('Lnia que histricament ha estat ms carregada.');

        OPEN c_consulta;
        LOOP
            FETCH c_consulta
            INTO l_id_linia, l_carrega;
            EXIT WHEN c_consulta%NOTFOUND;

            DBMS_OUTPUT.PUT_LINE ('Linia :'||l_id_linia||'      carrega acumulada :'||TO_CHAR(l_carrega,'999G999D99'));

        END LOOP;
        CLOSE c_consulta;

    DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');

	GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);

EXCEPTION
        WHEN OTHERS THEN
            ROLLBACK;
            sortida := SQLERRM||' A l''estadstica de lnia histricament ms carregada';
            		GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;
END;


/* Donat un any concret: percentatge de lnies que superen el
	50% denergia consumida.
*/

PROCEDURE pr_est_per_linies_carrega
        (t_any IN HISTORIC_CARREGA_LINIA_ANUAL.anyhis%TYPE
        )
	
IS
        procediment LOG_APLICACIO.procediment%TYPE := 'pr_est_percent_linia_carregades';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

        t_numero_linies NUMBER;
        t_linies NUMBER;
        t_percent_linies NUMBER DEFAULT 0;        
        t_id_linia HISTORIC_CARREGA_LINIA_ANUAL.linia%TYPE;

        l_carrega CHAR(10);

        CURSOR c_consulta IS
              SELECT li.id INTO t_id_linia
                      FROM LINIES li, HISTORIC_CARREGA_LINIA_ANUAL hcla
                      WHERE li.id = hcla.linia 
                            AND anyhis = t_any
                            AND hcla.carrega > (li.capacitat*0.5);
                      
         entrada_any_is_null EXCEPTION;
BEGIN
        entrada := '-';

        IF (t_any IS NULL)
                THEN RAISE entrada_any_is_null;
        END IF;

-- lleguim de la taula d'estadistiques per no utilitzar la funci agregada COUNT
        DBMS_STATS.GATHER_TABLE_STATS (ownname => 'Administrador', tabname => 'LINIES', estimate_percent => 100);

         SELECT NUM_ROWS INTO t_numero_linies
                FROM ALL_TABLES 
                WHERE table_name='LINIES';

-- lleguim el nombre de files del cursor sense utilitzar la funci agragada COUNT
        OPEN c_consulta;
        LOOP  
            FETCH c_consulta INTO t_id_linia;
            t_linies :=c_consulta%ROWCOUNT;
            EXIT WHEN c_consulta%NOTFOUND;
        END LOOP;
        CLOSE c_consulta;
        
        t_percent_linies := (t_linies*100)/t_numero_linies;
 
    
    DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');
    DBMS_OUTPUT.PUT_LINE ('Percentatges de lnies que en un any donat han superat el 50% de capacitat.');
    DBMS_OUTPUT.PUT_LINE ('Nombre total de lnies :'|| TO_CHAR(t_numero_linies, '999G999')
                          ||'      carrega acumulada :'||TO_CHAR(t_percent_linies, '999D99')||'%');
    DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');

 	GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);

EXCEPTION
         WHEN entrada_any_is_null THEN
            ROLLBACK;
            sortida := '-20206 - Any entrat es un valor nul.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;
        WHEN OTHERS THEN
            ROLLBACK;
            sortida := SQLERRM||' a estadstiques de lnies que superen el 50% de capacitat';
            		GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;
END;


/* Donat un any concret: el nombre de centrals de producci
	que generen menys del 30% de producci.
*/

PROCEDURE pr_est_centrals_menys_30
        (t_any IN HISTORIC_PRODUIDA_ANUAL.anyhis%TYPE
        )
	
IS
        procediment LOG_APLICACIO.procediment%TYPE := 'pr_est_centrals_menys_30';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

        t_numero_centrals NUMBER;
        t_id_central CENTRALS_PRODUCCIO.id%TYPE;

/* Les centrals nuclears necessiten un mnim d'energia per funcionar que no es pot
      comptabilitazar ja que no surt de la central a l'exterior.
*/
        CURSOR c_consulta IS
              SELECT ce.id INTO t_id_central
                      FROM CENTRALS_PRODUCCIO ce, HISTORIC_PRODUIDA_ANUAL hpa
                      WHERE ce.id = hpa.central 
                            AND anyhis = t_any
                            AND hpa.potencia < ((ce.energia_maxima-ce.energia_minima_NU)*0.3);
                      
         entrada_any_is_null EXCEPTION;
BEGIN
        entrada := '-';

        IF (t_any IS NULL)
                THEN RAISE entrada_any_is_null;
        END IF;

-- lleguim el nombre de files del cursor sense utilitzar la funci agragada COUNT
        OPEN c_consulta;
        LOOP 
            FETCH c_consulta INTO t_id_central;
            t_numero_centrals :=c_consulta%ROWCOUNT;
            EXIT WHEN c_consulta%NOTFOUND;
        END LOOP;
        CLOSE c_consulta;
    
    DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');
    DBMS_OUTPUT.PUT_LINE ('Nombre de centrals que generen menys del 30 % de la seva producci.');
    DBMS_OUTPUT.PUT_LINE ('Nombre de centrals :'|| TO_CHAR(t_numero_centrals, '999G999'));
    DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');

 	GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);

EXCEPTION

         WHEN entrada_any_is_null THEN
            ROLLBACK;
            sortida := '-20207 - Any entrat es un valor nul.';
			GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

        WHEN OTHERS THEN
            ROLLBACK;
            sortida := SQLERRM||' A estadstiques de quantitat de centrals que generen menys del 30% de producci';
            		GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;
END;


/* Top-10 de comptadors que histricament han tingut ms consum.
*/

PROCEDURE pr_est_top10_comptadors

IS
        linia_cap CHAR(40) := '---------------------------------------';
        l_id_comptador COMPTADORS.id%TYPE;
        l_numero_contracte COMPTADORS.numero_contracte%TYPE;
        l_lectura_acumulada COMPTADORS.lectura_acumulada%TYPE;

        procediment LOG_APLICACIO.procediment%TYPE := 'pr_est_top10_comptadors';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

        CURSOR c_consulta IS
		SELECT id, numero_contracte, lectura_acumulada INTO l_id_comptador, l_numero_contracte, l_lectura_acumulada
                      	FROM COMPTADORS
			WHERE ROWNUM <= 10
                      	ORDER BY lectura_acumulada DESC;


 BEGIN
        entrada := ' ';

        DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');
        DBMS_OUTPUT.PUT_LINE ('Els 10 comptadors que histricament han tingut ms consum');
        DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');

      	DBMS_OUTPUT.PUT_LINE (rpad('ID comptador',10)
                            ||' '||rpad('Nmero contracte',20)
                            ||' '||rpad('Consum acumulat',20));

      	DBMS_OUTPUT.PUT_LINE (rpad(linia_cap,10)
                            ||' '||rpad(linia_cap,20)
                            ||' '||rpad(linia_cap,20));

        OPEN c_consulta;
        LOOP
            FETCH c_consulta
            INTO l_id_comptador, l_numero_contracte, l_lectura_acumulada;
            EXIT WHEN c_consulta%NOTFOUND;
 
	    DBMS_OUTPUT.PUT_LINE (lpad(TO_CHAR(l_id_comptador),10)
                            ||' '||lpad(TO_CHAR(l_numero_contracte),20)
                            ||' '||lpad(TO_CHAR(l_lectura_acumulada,'999G999D99'),20));

        END LOOP;
        CLOSE c_consulta;
	GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);

EXCEPTION
        WHEN OTHERS THEN
            ROLLBACK;
            sortida := SQLERRM||' A la consulta dels 10 comptadors amb ms consum histric';
            		GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

END;


/* Consum mig de tots dels clients.
*/

PROCEDURE pr_est_consum_mig_clients

IS
        linia_cap CHAR(40) := '---------------------------------------';
        procediment LOG_APLICACIO.procediment%TYPE := 'pr_est_consum_mig_clients';
	entrada LOG_APLICACIO.entrada%TYPE;
	sortida LOG_APLICACIO.entrada%TYPE := 'Processat';

        l_id_client CLIENTS.id%TYPE;
        l_DNI_NIF CLIENTS.DNI_NIF%TYPE;
        l_consum_mitja CLIENTS.consum_mitja%TYPE;
        
        CURSOR c_consulta IS
          	SELECT id, DNI_NIF, consum_mitja INTO l_id_client, l_DNI_NIF, l_consum_mitja
		FROM CLIENTS;

        
 BEGIN
        entrada := ' ';

        DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');
        DBMS_OUTPUT.PUT_LINE ('Consum mig de tots els clients');
        DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');

      	DBMS_OUTPUT.PUT_LINE (rpad('ID client',10)
                            ||' '||rpad('DNI o NIF',20)
                            ||' '||rpad('Consum mitj',20));

      	DBMS_OUTPUT.PUT_LINE (rpad(linia_cap,10)
                            ||' '||rpad(linia_cap,20)
                            ||' '||rpad(linia_cap,20));



        OPEN c_consulta;
        LOOP
            FETCH c_consulta
            INTO l_id_client, l_DNI_NIF, l_consum_mitja;
            EXIT WHEN c_consulta%NOTFOUND;

	    DBMS_OUTPUT.PUT_LINE (lpad(TO_CHAR(l_id_client),10)
                            ||' '||lpad(l_DNI_NIF,20)
                            ||' '||lpad(TO_CHAR(l_consum_mitja,'999G999D99'),20));

        END LOOP;
        CLOSE c_consulta;

			
	GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);

EXCEPTION
        WHEN OTHERS THEN
            ROLLBACK;
            sortida := SQLERRM||' A la consulta de consum mig de tots els clients';
            		GENERAL.pr_alta_log_aplicacio(procediment, entrada,sortida);
			DBMS_OUTPUT.PUT_LINE(sortida);
            COMMIT;

END;

END;
/
disconnect;