/* -------------------------------------------- Connexio com administrador */

CONNECT Administrador/a12345
prompt <------------------------------- Connectat com Administrador.
prompt

prompt <------------------------------- Creacio de la taula PAISOS.

CREATE TABLE PAISOS
	(
	id_pais	 		CHAR(2 CHAR) 		CONSTRAINT PK_paisos PRIMARY KEY,
	nom 			VARCHAR2(20 CHAR) 	CONSTRAINT NN_paisos_nom NOT NULL,
	poblacio 		NUMBER 			CONSTRAINT NN_paisos_poblacio NOT NULL,
	extensio 		NUMBER 			CONSTRAINT NN_paisos_extensio NOT NULL,
	data_status		DATE			DEFAULT NULL
	);

prompt <-------------------------------  Creaci de la taula TIPUS_CENTRALS

CREATE TABLE TIPUS_CENTRALS
	(
	id_tipus		CHAR(2 CHAR) 		CONSTRAINT PK_tipus_centrals PRIMARY KEY,
	descripcio 		VARCHAR2(20 CHAR) 	CONSTRAINT NN_tipus_centrals_descripcio NOT NULL,
	data_status		DATE			DEFAULT NULL
	);

prompt <-------------------------------  Creaci de la taula MODELS_COMPTADOR 

CREATE TABLE MODELS_COMPTADOR
	(
	id_model 	CHAR(9 CHAR) 		CONSTRAINT PK_models_comptador PRIMARY KEY,
	fabricant 	VARCHAR2(20 CHAR) 	CONSTRAINT NN_models_comptador_fabricant NOT NULL,
	descripcio 	VARCHAR2(30 CHAR),
	data_status	DATE			DEFAULT NULL
	);

prompt <-------------------------------  Creaci de la taula CENTRALS_PRODUCCIO

CREATE SEQUENCE s_centrals_produccio
	INCREMENT BY 1
	START WITH 1;

CREATE TABLE CENTRALS_PRODUCCIO
	(
	id			NUMBER	 		CONSTRAINT PK_centrals_produccio PRIMARY KEY,
	nom 			VARCHAR2(40 CHAR) 	CONSTRAINT NN_centrals_produccio_nom NOT NULL,
	tipus	 					CONSTRAINT FK_centrals_produccio_tipus REFERENCES TIPUS_CENTRALS(id_tipus),
	adreca 			VARCHAR2(50 CHAR) 	CONSTRAINT NN_centrals_produccio_adreca NOT NULL,
	cp 			VARCHAR2(5 CHAR) 	CONSTRAINT NN_centrals_produccio_cp NOT NULL,
	localitat	 	VARCHAR2(30 CHAR) 	CONSTRAINT NN_centrals_produccio_loca NOT NULL,
	pais 						CONSTRAINT FK_centrals_produccio_pais REFERENCES PAISOS(id_pais),
	telefon		 	VARCHAR2(10 CHAR),
	e_mail 			VARCHAR2(30 CHAR),
	energia_maxima 		NUMBER (9,2) 		CONSTRAINT NN_centrals_produccio_e_max NOT NULL,
	data_ult_revisio 	DATE,
	energia_minima_NU 	NUMBER(9,2),
	rebuig_radioactiu	NUMBER(9,2), 
	kg_emisio_CO2	 	NUMBER(9,2),
 	unitats_produccio	NUMBER,
	alta_disponibilitat 	CHAR(1 CHAR)	 	DEFAULT 'N',
	data_status		DATE			DEFAULT NULL
	);

/* el nom de la central ser nic per la identificaci de la central de producci
*/
ALTER TABLE CENTRALS_PRODUCCIO ADD
	(
	CONSTRAINT UN_Centrals_nom UNIQUE(nom)
	);

/* 	Si es una central nuclear han d'estar informts els camps energia_mnima i rebuig_radioactiu
	si no ho es han d'estar a 0 o null
*/
ALTER TABLE CENTRALS_PRODUCCIO ADD
	(
	CONSTRAINT CK_Centrals_NU CHECK 
		((tipus='NU' AND energia_minima_NU >0 AND rebuig_radioactiu >0) OR  
		(tipus<>'NU' AND energia_minima_NU <=0 AND rebuig_radioactiu <=0))
	);

/*	Si es una central trmica o de carb ha d'estar informat el camp kg_emisio_CO2
	si no ho es ha d'estar a 0 o null
*/
ALTER TABLE CENTRALS_PRODUCCIO ADD
	(
	CONSTRAINT CK_Centrals_TE_CA CHECK 
		(((tipus='TE' OR tipus='CA' ) AND kg_emisio_CO2 >0) OR  
		((tipus<>'TE' AND tipus<>'CA' ) AND kg_emisio_CO2 <=0))
	);

/* 	Si es una central elica o solar ha d'estar informat el camp unitats_produccio
	si no ho es ha d'estar a 0 o null
*/
ALTER TABLE CENTRALS_PRODUCCIO ADD
	(
	CONSTRAINT CK_Centrals_EO_SO CHECK 
		(((tipus='EO' OR tipus='SO') AND unitats_produccio >0) OR  
		((tipus<>'EO' AND tipus<>'SO') AND unitats_produccio <=0))
	);

/* 	l'atribut alta disponibilitat pot estar 
	S => alta disponibilitat
	N => baixa disponibilitat
*/
ALTER TABLE CENTRALS_PRODUCCIO ADD
	(
	CONSTRAINT ck_centrals_alta_dispo CHECK (UPPER(alta_disponibilitat) IN('S','N'))
	);


/*	Creaci de la seqncia de les centrals de produccio
*/
CREATE OR REPLACE TRIGGER centrals_produccio_insert
	   BEFORE INSERT ON CENTRALS_PRODUCCIO
	   FOR EACH ROW
	BEGIN
	   SELECT TO_CHAR(s_centrals_produccio.NEXTVAL) INTO :NEW.id FROM DUAL;
END centrals_produccio_insert;
/


prompt <-------------------------------  Creaci de la taula LINIES 

CREATE SEQUENCE s_linies
	INCREMENT BY 1
	START WITH 1;

CREATE TABLE LINIES
	(
	id	 		NUMBER	 	CONSTRAINT PK_linies PRIMARY KEY,
	central	 				CONSTRAINT FK_linies_central_associada REFERENCES CENTRALS_PRODUCCIO(id),
	capacitat 		NUMBER(9,2)	CONSTRAINT NN_linies_capacitat NOT NULL,
	carrega_acumulada	NUMBER(9,2)	DEFAULT 0,
	alta_disponibilitat 	CHAR(1 CHAR) 	DEFAULT 'N',
	data_status		DATE		DEFAULT NULL
	);

/* 	l'atribut alta disponibilitat pot estar 
	S => alta disponibilitat
	N => baixa disponibilitat
*/
ALTER TABLE LINIES ADD
	(
	CONSTRAINT ck_linies_alta_disponibilitat CHECK (UPPER(alta_disponibilitat) IN('S','N'))
	);

/*	Creaci de la seqncia de les lnies
*/
CREATE OR REPLACE TRIGGER linies_insert
	   BEFORE INSERT ON LINIES
	   FOR EACH ROW
	BEGIN
	   SELECT TO_CHAR(s_linies.NEXTVAL) INTO :NEW.id FROM DUAL;
END linies_insert;
/


prompt <------------------------------- Creaci de la taula CENTRALETES

CREATE SEQUENCE s_centraletes
	INCREMENT BY 1
	START WITH 1;

CREATE TABLE CENTRALETES
	(
	id	 		NUMBER	 		CONSTRAINT PK_centraletes PRIMARY KEY,
	linia 						CONSTRAINT FK_centraletes_linia REFERENCES LINIES(id),
	adreca 			VARCHAR2(50 CHAR) 	CONSTRAINT NN_centraletes_adreca NOT NULL,
	cp 			VARCHAR2(5 CHAR) 	CONSTRAINT NN_centraletes_cp NOT NULL,
	localitat 		VARCHAR2(30 CHAR) 	CONSTRAINT NN_centraletes_localitat NOT NULL,
	pais 						CONSTRAINT FK_centraletes_pais REFERENCES PAISOS(id_pais),
	capacitat_maxima 	NUMBER(9,2) 		CONSTRAINT NN_centraletes_capacitat_max NOT NULL,
	alta_disponibilitat 	CHAR(1 CHAR)	 	DEFAULT 'N',
	data_status		DATE			DEFAULT NULL
	);

/* l'adrea i cp ser nica per la identificaci de la centraleta
*/
ALTER TABLE CENTRALETES ADD
	(
	CONSTRAINT UN_centraleta_adreca UNIQUE(adreca,cp)
	);

/* Cada centraleta noms pot ser alimentada per una lnia
*/

ALTER TABLE CENTRALETES ADD
	(
	CONSTRAINT UN_centraleta_cen_lin UNIQUE(id, linia)
	);

/* 	l'atribut alta disponibilitat pot estar 
	S => alta disponibilitat
	N => baixa disponibilitat
*/
ALTER TABLE CENTRALETES ADD
	(
	CONSTRAINT ck_centraletes_disponibilitat CHECK (UPPER(alta_disponibilitat) IN('S','N'))
	);

/*	Creaci de la seqncia de les centrals de produccio
*/
CREATE OR REPLACE TRIGGER centraletes_insert
	   BEFORE INSERT ON CENTRALETES
	   FOR EACH ROW
	BEGIN
	   SELECT TO_CHAR(s_centraletes.NEXTVAL) INTO :NEW.id FROM DUAL;
END centraletes_insert;
/


prompt <-------------------------------  Creaci de la taula CLIENTS

CREATE SEQUENCE s_clients
	INCREMENT BY 1
	START WITH 1;

CREATE TABLE CLIENTS
	(
	id	 		NUMBER	 		CONSTRAINT PK_clients PRIMARY KEY,
	DNI_NIF			VARCHAR2(10 CHAR) 	CONSTRAINT NN_clients_nif NOT NULL,
	nom_rao_social 		VARCHAR2(40 CHAR) 	CONSTRAINT NN_clients_nom_rao_social NOT NULL,
	adreca			VARCHAR2(50 CHAR) 	CONSTRAINT NN_clients_adreca NOT NULL,
	cp 			VARCHAR2(5 CHAR) 	CONSTRAINT NN_clients_cp NOT NULL,
	localitat 		VARCHAR2(30 CHAR) 	CONSTRAINT NN_clients_localitat NOT NULL,
	pais 						CONSTRAINT FK_clients_pais REFERENCES PAISOS(id_pais),
	telefon		 	VARCHAR2(10 CHAR),
	e_mail 			VARCHAR2(30 CHAR),
	companyia		VARCHAR2(20 CHAR),
	consum_mitja		NUMBER(9,2)		DEFAULT 0,
	data_status		DATE			DEFAULT NULL
	);

/* el DNI_NIF ser nic per la identificaci del client
*/
ALTER TABLE CLIENTS ADD
	(
	CONSTRAINT UN_clients_DNI_NIF UNIQUE(DNI_NIF)
	);

/*	Creaci de la seqncia de clients
*/
CREATE OR REPLACE TRIGGER clients_insert
	   BEFORE INSERT ON CLIENTS
	   FOR EACH ROW
	BEGIN
	   SELECT TO_CHAR(s_clients.NEXTVAL) INTO :NEW.id FROM DUAL;
END clients_insert;
/

prompt <-------------------------------  Creaci de la taula COMPTADORS 

CREATE SEQUENCE s_comptadors
	INCREMENT BY 1
	START WITH 1;

CREATE TABLE COMPTADORS
	(
	id	 		NUMBER	 		CONSTRAINT PK_comptadors PRIMARY KEY,
	numero_contracte 	VARCHAR2(12 CHAR) 	CONSTRAINT NN_comptadors_numero_contracte NOT NULL,
	client 				 		CONSTRAINT FK_comptadors_client REFERENCES CLIENTS(id),
	pot_contractada 	NUMBER(9,2) 		CONSTRAINT NN_comptadors_pot_contractada NOT NULL,
	adreca 			VARCHAR2(50 CHAR) 	CONSTRAINT NN_comptadors_adreca NOT NULL,
	cp 			VARCHAR2(5 CHAR) 	CONSTRAINT NN_comptadors_cp NOT NULL,
	localitat 		VARCHAR2(30 CHAR) 	CONSTRAINT NN_comptadors_localitat NOT NULL,
	pais 			CHAR(2 CHAR)	 	CONSTRAINT FK_comptadors_pais REFERENCES PAISOS(id_pais),
	lectura_acumulada 	NUMBER 			DEFAULT 0,
	model 						CONSTRAINT FK_comptadors_model REFERENCES MODELS_COMPTADOR(id_model),
	any_fabricacio 		NUMBER(4) 		CONSTRAINT NN_comptadors_any_fabricacio NOT NULL,
	data_instalacio 	DATE 			DEFAULT SYSDATE,
	data_ultima_revisio 	DATE 			DEFAULT SYSDATE,
	data_ultima_lectura	DATE			DEFAULT NULL,
	consum_mitja_anual	NUMBER(9,2)		DEFAULT 0,
	alta_disponibilitat 	CHAR(1 CHAR) 		DEFAULT 'N',
	data_status		DATE			DEFAULT NULL
	);

/* el numero de contracte ser nic per la identificaci de la lnia a la taula
*/
ALTER TABLE COMPTADORS ADD
	(
	CONSTRAINT UN_comptadors_num_contracte UNIQUE(numero_contracte)
	);

/* la data de l'ltima revisi ha de ser igual o superior a la data d'instal.laci
*/
ALTER TABLE COMPTADORS ADD
	(
	CONSTRAINT CK_comptadors_dat_ult_revisio 
		CHECK ( data_ultima_revisio >= data_instalacio )
	);

/* l'any de fabricaci no pot ser menor o igual a la data de instal.laci
*/
ALTER TABLE COMPTADORS ADD
(
  CONSTRAINT CH_comptadors_any_favricacio CHECK(any_fabricacio<=extract(YEAR from data_instalacio))
);


/* La data de l'ltima lectura ha de ser igual o superior a la data de l'ltima revisi
*/
ALTER TABLE COMPTADORS ADD
	(
	CONSTRAINT CK_comptadors_dat_ult_lectura 
		CHECK ( data_ultima_lectura >= data_ultima_revisio )
	);

/* 	l'atribut alta disponibilitat pot estar 
	S => alta disponibilitat
	N => baixa disponibilitat
*/
ALTER TABLE COMPTADORS ADD
	(
	CONSTRAINT ck_comptadors_alta_dispo CHECK (UPPER(alta_disponibilitat) IN('S','N'))
	);

/*	Creaci de la seqncia de les centrals de produccio
*/
CREATE OR REPLACE TRIGGER comptadors_insert
	   BEFORE INSERT ON COMPTADORS
	   FOR EACH ROW
	BEGIN
	   SELECT TO_CHAR(s_comptadors.NEXTVAL) INTO :NEW.id FROM DUAL;
END comptadors_insert;
/


prompt <-------------------------------   Creaci relaci CENTRALETES_COMPTADORS  
--			( Un comptador pot tenir ms d'una centraleta) 

CREATE TABLE CENTRALETES_COMPTADORS
	(
	centraleta  			CONSTRAINT FK_centr_compt_centr REFERENCES CENTRALETES(id),
	comptador  			CONSTRAINT FK_centr_compt_compt REFERENCES COMPTADORS(id),
	data_status		DATE	DEFAULT NULL
	);

/* 	clau primaria composta
*/
ALTER TABLE CENTRALETES_COMPTADORS ADD
	(
	CONSTRAINT PK_centr_compt PRIMARY KEY (centraleta, comptador)
	);

prompt <----------------------------- Creaci de la taula HISTORIC_LECTURES_COMPTADORS

CREATE TABLE HISTORIC_LECTURES_COMPTADORS
(
	comptador 			 	CONSTRAINT FK_his_lec_com_comp REFERENCES COMPTADORS(id),
	mes 			NUMBER(2) 	CONSTRAINT NN_his_lec_com_mes NOT NULL,
	anyhis	 		NUMBER(4)	CONSTRAINT NN_his_lec_com_any NOT NULL,
	consum 			NUMBER(9,2)	CONSTRAINT NN_his_lec_com_any_consum NOT NULL
); 


/*     Clau primaria composta
*/
ALTER TABLE HISTORIC_LECTURES_COMPTADORS ADD
	(
	CONSTRAINT PK_his_lec_com_mes PRIMARY KEY (comptador,anyhis,mes)
	);

/*     Control del nombre del mes
*/
ALTER TABLE HISTORIC_LECTURES_COMPTADORS ADD
	(
	CONSTRAINT CK_his_lec_com_mes CHECK (mes>=1 AND mes<=12)
	);

/* Control de l'any entre el 2000 i el 2200
*/
ALTER TABLE HISTORIC_LECTURES_COMPTADORS ADD
	(
	CONSTRAINT CK_his_lec_com_any CHECK (anyhis>=2000 AND anyhis<=2200)
	);



prompt <---------------------------  Creaci de la taula HISTORIC_PRODUIDA_MES

CREATE TABLE HISTORIC_PRODUIDA_MES
	(
	central 			CONSTRAINT FK_his_prod_m_central REFERENCES CENTRALS_PRODUCCIO(id),
	mes  		NUMBER(2)	CONSTRAINT NN_his_prod_m_mes NOT NULL,
	anyhis		NUMBER(4)	CONSTRAINT NN_his_prod_m_anyhis NOT NULL,
	potencia	NUMBER(9,2)	CONSTRAINT NN_his_prod_m_potencia NOT NULL
	);

/*     Clau principal composta
*/
ALTER TABLE HISTORIC_PRODUIDA_MES ADD
	(
	CONSTRAINT PK_produccio_centrals_m PRIMARY KEY (central,anyhis,mes)
	);

/*     Control del nombre del mes
*/
ALTER TABLE HISTORIC_PRODUIDA_MES ADD
	(
	CONSTRAINT CK_his_pro_m_mes CHECK (mes>=1 AND mes<=12)
	);

/* Control de l'any entre el 2000 i el 2200
*/
ALTER TABLE HISTORIC_PRODUIDA_MES ADD
	(
	CONSTRAINT CK_his_pro_m_any CHECK (anyhis>=2000 AND anyhis<=2200)
	);



prompt <---------------------------  Creaci de la taula HISTORIC_PRODUIDA_ANUAL

CREATE TABLE HISTORIC_PRODUIDA_ANUAL
	(
	central 			CONSTRAINT FK_his_prod_a_central REFERENCES CENTRALS_PRODUCCIO(id),
	anyhis		NUMBER(4)	CONSTRAINT NN_his_prod_a_anyhis NOT NULL,
	potencia	NUMBER(9,2)	CONSTRAINT NN_his_prod_a_potencia NOT NULL
	);

/*     Clau principal composta
*/
ALTER TABLE HISTORIC_PRODUIDA_ANUAL ADD
	(
	CONSTRAINT PK_produccio_centrals_a PRIMARY KEY (central,anyhis)
	);

/* Control de l'any entre el 2000 i el 2200
*/
ALTER TABLE HISTORIC_PRODUIDA_ANUAL ADD
	(
	CONSTRAINT CK_his_pro_a_any CHECK (anyhis>=2000 AND anyhis<=2200)
	);


prompt <---------------------------  Creaci de la taula HISTORIC_CARREGA_LINIA_MES

CREATE TABLE HISTORIC_CARREGA_LINIA_MES
	(
	linia				CONSTRAINT FK_his_car_linia_m_linia REFERENCES LINIES(id),
	mes  		NUMBER(2)	CONSTRAINT NN_his_car_linia_m_mes NOT NULL,
	anyhis		NUMBER(4)	CONSTRAINT NN_his_car_linia_m_anyhis NOT NULL,
	carrega		NUMBER(9,2)	CONSTRAINT NN_his_car_linia_m_carrega NOT NULL
	);

/*     Clau principal composta
*/
ALTER TABLE HISTORIC_CARREGA_LINIA_MES ADD
	(
	CONSTRAINT PK_his_car_linia_m PRIMARY KEY (linia,anyhis,mes)
	);

/*     Control del nombre del mes
*/
ALTER TABLE HISTORIC_CARREGA_LINIA_MES ADD
	(
	CONSTRAINT CK_his_car_linia_m_mes CHECK (mes>=1 AND mes<=12)
	);

/* Control de l'any entre el 2000 i el 2200
*/
ALTER TABLE HISTORIC_CARREGA_LINIA_MES ADD
	(
	CONSTRAINT CK_his_car_linia_m_any CHECK (anyhis>=2000 AND anyhis<=2200)
	);


prompt <---------------------------  Creaci de la taula HISTORIC_CARREGA_LINIA_ANUAL

CREATE TABLE HISTORIC_CARREGA_LINIA_ANUAL
	(
	linia					CONSTRAINT FK_his_car_linia_a_linia REFERENCES LINIES(id),
	anyhis			NUMBER(4)	CONSTRAINT NN_his_car_linia_a_anyhis NOT NULL,
	carrega_mitja_anual	NUMBER(9,2)	CONSTRAINT NN_his_car_linia_a_ca_mtj_mes NOT NULL,
	carrega			NUMBER(9,2)	CONSTRAINT NN_his_car_linia_a_carrega NOT NULL
	);

/*     Clau principal composta
*/
ALTER TABLE HISTORIC_CARREGA_LINIA_ANUAL ADD
	(
	CONSTRAINT PK_his_car_linia_a PRIMARY KEY (linia,anyhis)
	);

/* Control de l'any entre el 2000 i el 2200
*/
ALTER TABLE HISTORIC_CARREGA_LINIA_ANUAL ADD
	(
	CONSTRAINT CK_his_car_linia_a_any CHECK (anyhis>=2000 AND anyhis<=2200)
	);




prompt <-------------------------------  Creaci de la taula LECTURES_COMPTADORS

CREATE SEQUENCE s_lectures
	INCREMENT BY 1
	START WITH 1;

CREATE TABLE LECTURES_COMPTADORS
(	id_lectura		NUMBER			CONSTRAINT PK_lectura_comptadors PRIMARY KEY,
	numero_contracte		 		CONSTRAINT FK_lec_comp_numero_contracte REFERENCES COMPTADORS(numero_contracte),
	data_lectura 		DATE 			CONSTRAINT NN_lec_comp_data_lectura NOT NULL,
	consum 			NUMBER(9,2)		CONSTRAINT NN_lec_comp_consum NOT NULL,
	modalitat_lectora 	CHAR(1 CHAR) 		CONSTRAINT NN_lec_comp_mod_lec NOT NULL,
	companyia		VARCHAR2(20 CHAR)
); 


/*	Creaci de la seqncia de log_aplicacio
*/
CREATE OR REPLACE TRIGGER lectures_insert
	   BEFORE INSERT ON LECTURES_COMPTADORS
	   FOR EACH ROW
	BEGIN
	   SELECT TO_CHAR(s_lectures.NEXTVAL) INTO :NEW.id_lectura FROM DUAL;
END lectures_insert;
/


/*	Comprobaci de tipus de modalitat
	 T => Telemtica
	 P => Presencial
*/
ALTER TABLE LECTURES_COMPTADORS ADD
	(
	CONSTRAINT CK_lec_comp_modalitat CHECK (UPPER(modalitat_lectora) IN ('T','P'))
	);

prompt <-------------------------------  Creaci de la taula de log_aplicacio (log)



CREATE SEQUENCE s_log_aplicacio
	INCREMENT BY 1
	START WITH 1;

CREATE TABLE LOG_APLICACIO
	(
	id_log			NUMBER	 		CONSTRAINT PK_log_aplicacio PRIMARY KEY,
	usuari 			VARCHAR2(20 CHAR) 	CONSTRAINT NN_log_aplicacio_usuari NOT NULL,
	data_hora 		TIMESTAMP		CONSTRAINT NN_log_aplicacio_data_hora NOT NULL,
	procediment		VARCHAR2(35 CHAR) 	CONSTRAINT NN_log_aplicacio_procediment NOT NULL,
	entrada			VARCHAR2(200 CHAR) 	CONSTRAINT NN_log_aplicacio_entrada NOT NULL,
	sortida	 		VARCHAR2(200 CHAR) 	
	);


/*	Creaci de la seqncia de log_aplicacio
*/
CREATE OR REPLACE TRIGGER log_aplicacio_insert
	   BEFORE INSERT ON log_APLICACIO
	   FOR EACH ROW
	BEGIN
	   SELECT TO_CHAR(s_log_aplicacio.NEXTVAL) INTO :NEW.id_log FROM DUAL;
END log_aplicacio_insert;
/

prompt
prompt

DISCONNECT;

